@@ -410,7 +410,7 @@ gdk_content_register_deserializer (const char                *mime_type,
   g_return_if_fail (mime_type != NULL);
   g_return_if_fail (deserialize != NULL);
 
-  deserializer = g_slice_new0 (Deserializer);
+  deserializer = g_new0 (Deserializer, 1);
 
   deserializer->mime_type = g_intern_string (mime_type);
   deserializer->type = type;
@@ -462,17 +462,20 @@ gdk_content_formats_union_deserialize_gtypes (GdkContentFormats *formats)
 
   g_return_val_if_fail (formats != NULL, NULL);
 
-  init ();
-
   builder = gdk_content_formats_builder_new ();
   gdk_content_formats_builder_add_formats (builder, formats);
 
-  for (l = g_queue_peek_head_link (&deserializers); l; l = l->next)
+  if (!gdk_content_formats_is_empty (formats))
     {
-      Deserializer *deserializer = l->data;
+      init ();
 
-      if (gdk_content_formats_contain_mime_type (formats, deserializer->mime_type))
-        gdk_content_formats_builder_add_gtype (builder, deserializer->type);
+      for (l = g_queue_peek_head_link (&deserializers); l; l = l->next)
+        {
+          Deserializer *deserializer = l->data;
+
+          if (gdk_content_formats_contain_mime_type (formats, deserializer->mime_type))
+            gdk_content_formats_builder_add_gtype (builder, deserializer->type);
+        }
     }
 
   gdk_content_formats_unref (formats);
@@ -497,17 +500,20 @@ gdk_content_formats_union_deserialize_mime_types (GdkContentFormats *formats)
 
   g_return_val_if_fail (formats != NULL, NULL);
 
-  init ();
-
   builder = gdk_content_formats_builder_new ();
   gdk_content_formats_builder_add_formats (builder, formats);
 
-  for (l = g_queue_peek_head_link (&deserializers); l; l = l->next)
+  if (!gdk_content_formats_is_empty (formats))
     {
-      Deserializer *deserializer = l->data;
+      init ();
 
-      if (gdk_content_formats_contain_gtype (formats, deserializer->type))
-        gdk_content_formats_builder_add_mime_type (builder, deserializer->mime_type);
+      for (l = g_queue_peek_head_link (&deserializers); l; l = l->next)
+        {
+          Deserializer *deserializer = l->data;
+
+          if (gdk_content_formats_contain_gtype (formats, deserializer->type))
+            gdk_content_formats_builder_add_mime_type (builder, deserializer->mime_type);
+        }
     }
 
   gdk_content_formats_unref (formats);
@@ -533,16 +539,13 @@ deserialize_not_found (GdkContentDeserializer *deserializer)
  * @type: the GType to deserialize from
  * @io_priority: the I/O priority of the operation
  * @cancellable: (nullable): optional `GCancellable` object
- * @callback: (scope async): callback to call when the operation is done
- * @user_data: (closure): data to pass to the callback function
+ * @callback: (scope async) (closure user_data): callback to call when the operation is done
+ * @user_data: data to pass to the callback function
  *
  * Read content from the given input stream and deserialize it, asynchronously.
  *
  * The default I/O priority is %G_PRIORITY_DEFAULT (i.e. 0), and lower numbers
  * indicate a higher priority.
- *
- * When the operation is finished, @callback will be called. You must then
- * call [func@Gdk.content_deserialize_finish] to get the result of the operation.
  */
 void
 gdk_content_deserialize_async (GInputStream        *stream,
