@@ -539,16 +539,13 @@ serialize_not_found (GdkContentSerializer *serializer)
  * @value: the content to serialize
  * @io_priority: the I/O priority of the operation
  * @cancellable: (nullable): optional `GCancellable` object
- * @callback: (scope async): callback to call when the operation is done
- * @user_data: (closure): data to pass to the callback function
+ * @callback: (scope async) (closure): callback to call when the operation is done
+ * @user_data: data to pass to the callback function
  *
  * Serialize content and write it to the given output stream, asynchronously.
  *
  * The default I/O priority is %G_PRIORITY_DEFAULT (i.e. 0), and lower numbers
  * indicate a higher priority.
- *
- * When the operation is finished, @callback will be called. You must then
- * call [func@Gdk.content_serialize_finish] to get the result of the operation.
  */
 void
 gdk_content_serialize_async (GOutputStream       *stream,
@@ -797,11 +794,25 @@ file_serializer_finish (GObject      *source,
     gdk_content_serializer_return_success (serializer);
 }
 
+
+static const char *
+file_force_uri_schemas[] = { "content", "http", "https" };
 static char *
 file_get_native_uri (GFile *file)
 {
   char *path;
 
+  /*
+   * We want to convert URIs to file:// URIs, so other applications
+   * that do not use GIO/GVFS can read text/uri-list from the clipboard.
+   *
+   * But we don't want to do that for URIs that are widely understood,
+   * so for lack of better APIs, we whitelist those.
+   */
+  for (gsize i = 0; i < G_N_ELEMENTS (file_force_uri_schemas); i++)
+    if (g_file_has_uri_scheme (file, file_force_uri_schemas[i]))
+        return g_file_get_uri (file);
+
   path = g_file_get_path (file);
   if (path != NULL)
     {
