@@ -88,6 +88,9 @@ enum {
   PROP_NUM_LOCK_STATE,
   PROP_SCROLL_LOCK_STATE,
   PROP_MODIFIER_STATE,
+  PROP_LAYOUT_NAMES,
+  PROP_ACTIVE_LAYOUT_INDEX,
+
   LAST_PROP
 };
 
@@ -104,7 +107,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
   object_class->get_property = gdk_device_get_property;
 
   /**
-   * GdkDevice:display: (attributes org.gtk.Property.get=gdk_device_get_display)
+   * GdkDevice:display:
    *
    * The `GdkDisplay` the `GdkDevice` pertains to.
    */
@@ -114,7 +117,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                            G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:name: (attributes org.gtk.Property.get=gdk_device_get_name)
+   * GdkDevice:name:
    *
    * The device name.
    */
@@ -125,7 +128,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                            G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:source: (attributes org.gtk.Property.get=gdk_device_get_source)
+   * GdkDevice:source:
    *
    * Source type for the device.
    */
@@ -137,7 +140,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                          G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GdkDevice:has-cursor: (attributes org.gtk.Property.get=gdk_device_get_has_cursor)
+   * GdkDevice:has-cursor:
    *
    * Whether the device is represented by a cursor on the screen.
    */
@@ -159,7 +162,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:vendor-id: (attributes org.gtk.Property.get=gdk_device_get_vendor_id)
+   * GdkDevice:vendor-id:
    *
    * Vendor ID of this device.
    *
@@ -172,7 +175,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                            G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:product-id: (attributes org.gtk.Property.get=gdk_device_get_product_id)
+   * GdkDevice:product-id:
    *
    * Product ID of this device.
    *
@@ -185,7 +188,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                            G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:seat: (attributes org.gtk.Property.get=gdk_device_get_seat)
+   * GdkDevice:seat:
    *
    * `GdkSeat` of this device.
    */
@@ -196,7 +199,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                            G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:num-touches: (attributes org.gtk.Property.get=gdk_device_get_num_touches)
+   * GdkDevice:num-touches:
    *
    * The maximal number of concurrent touches on a touch device.
    *
@@ -211,7 +214,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                          G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:tool: (attributes org.gtk.Property.get=gdk_device_get_device_tool)
+   * GdkDevice:tool: (getter get_device_tool)
    *
    * The `GdkDeviceTool` that is currently used with this device.
    */
@@ -221,7 +224,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:direction: (attributes org.gtk.Property.get=gdk_device_get_direction)
+   * GdkDevice:direction:
    *
    * The direction of the current layout.
    *
@@ -233,7 +236,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:has-bidi-layouts: (attributes org.gtk.Property.get=gdk_device_has_bidi_layouts)
+   * GdkDevice:has-bidi-layouts:
    *
    * Whether the device has both right-to-left and left-to-right layouts.
    *
@@ -245,7 +248,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                             G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:caps-lock-state: (attributes org.gtk.Property.get=gdk_device_get_caps_lock_state)
+   * GdkDevice:caps-lock-state:
    *
    * Whether Caps Lock is on.
    *
@@ -257,7 +260,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                             G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:num-lock-state: (attributes org.gtk.Property.get=gdk_device_get_num_lock_state)
+   * GdkDevice:num-lock-state:
    *
    * Whether Num Lock is on.
    *
@@ -269,7 +272,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                             G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:scroll-lock-state: (attributes org.gtk.Property.get=gdk_device_get_scroll_lock_state)
+   * GdkDevice:scroll-lock-state:
    *
    * Whether Scroll Lock is on.
    *
@@ -281,7 +284,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                             G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDevice:modifier-state: (attributes org.gtk.Property.get=gdk_device_get_modifier_state)
+   * GdkDevice:modifier-state:
    *
    * The current modifier state of the device.
    *
@@ -293,6 +296,36 @@ gdk_device_class_init (GdkDeviceClass *klass)
                           GDK_NO_MODIFIER_MASK,
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * GdkDevice:active-layout-index:
+   *
+   * The index of the keyboard active layout of a `GdkDevice`.
+   *
+   * Will be -1 if there is no valid active layout.
+   *
+   * This is only relevant for keyboard devices.
+   *
+   * Since: 4.18
+   */
+  device_props[PROP_ACTIVE_LAYOUT_INDEX] =
+      g_param_spec_int ("active-layout-index", NULL, NULL,
+                        -1, G_MAXINT, 0,
+                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * GdkDevice:layout-names:
+   *
+   * The names of the keyboard layouts of a `GdkDevice`.
+   *
+   * This is only relevant for keyboard devices.
+   *
+   * Since: 4.18
+   */
+  device_props[PROP_LAYOUT_NAMES] =
+      g_param_spec_boxed ("layout-names", NULL, NULL,
+                          G_TYPE_STRV,
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (object_class, LAST_PROP, device_props);
 
   /**
@@ -479,6 +512,13 @@ gdk_device_get_property (GObject    *object,
     case PROP_MODIFIER_STATE:
       g_value_set_flags (value, gdk_device_get_modifier_state (device));
       break;
+    case PROP_ACTIVE_LAYOUT_INDEX:
+      g_value_set_int (value, gdk_device_get_active_layout_index (device));
+      break;
+    case PROP_LAYOUT_NAMES:
+      g_value_set_boxed (value, gdk_device_get_layout_names (device));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -540,7 +580,7 @@ gdk_device_get_name (GdkDevice *device)
 }
 
 /**
- * gdk_device_get_has_cursor: (attributes org.gtk.Method.get_property=has-cursor)
+ * gdk_device_get_has_cursor:
  * @device: a `GdkDevice`
  *
  * Determines whether the pointer follows device motion.
@@ -559,7 +599,7 @@ gdk_device_get_has_cursor (GdkDevice *device)
 }
 
 /**
- * gdk_device_get_source: (attributes org.gtk.Method.get_property=source)
+ * gdk_device_get_source:
  * @device: a `GdkDevice`
  *
  * Determines the type of the device.
@@ -599,7 +639,7 @@ gdk_device_get_axis_use (GdkDevice *device,
 }
 
 /**
- * gdk_device_get_display: (attributes org.gtk.Method.get_property=display)
+ * gdk_device_get_display:
  * @device: a `GdkDevice`
  *
  * Returns the `GdkDisplay` to which @device pertains.
@@ -1082,7 +1122,7 @@ _gdk_device_surface_at_position (GdkDevice       *device,
 }
 
 /**
- * gdk_device_get_vendor_id: (attributes org.gtk.Method.get_property=vendor-id)
+ * gdk_device_get_vendor_id:
  * @device: a physical `GdkDevice`
  *
  * Returns the vendor ID of this device.
@@ -1124,7 +1164,7 @@ gdk_device_get_vendor_id (GdkDevice *device)
 }
 
 /**
- * gdk_device_get_product_id: (attributes org.gtk.Method.get_property=product-id)
+ * gdk_device_get_product_id:
  * @device: a physical `GdkDevice`
  *
  * Returns the product ID of this device.
@@ -1157,7 +1197,7 @@ gdk_device_set_seat (GdkDevice *device,
 }
 
 /**
- * gdk_device_get_seat: (attributes org.gtk.Method.get_property=seat)
+ * gdk_device_get_seat:
  * @device: A `GdkDevice`
  *
  * Returns the `GdkSeat` the device belongs to.
@@ -1202,7 +1242,7 @@ gdk_device_get_num_touches (GdkDevice *device)
 }
 
 /**
- * gdk_device_get_device_tool: (attributes org.gtk.Method.get_property=tool)
+ * gdk_device_get_device_tool: (get-property tool)
  * @device: a `GdkDevice`
  *
  * Retrieves the current tool for @device.
@@ -1218,7 +1258,7 @@ gdk_device_get_device_tool (GdkDevice *device)
 }
 
 /**
- * gdk_device_get_caps_lock_state: (attributes org.gtk.Method.get_property=caps-lock-state)
+ * gdk_device_get_caps_lock_state:
  * @device: a `GdkDevice`
  *
  * Retrieves whether the Caps Lock modifier of the keyboard is locked.
@@ -1239,7 +1279,7 @@ gdk_device_get_caps_lock_state (GdkDevice *device)
 }
 
 /**
- * gdk_device_get_num_lock_state: (attributes org.gtk.Method.get_property=num-lock-state)
+ * gdk_device_get_num_lock_state:
  * @device: a ``GdkDevice`
  *
  * Retrieves whether the Num Lock modifier of the keyboard is locked.
@@ -1260,7 +1300,7 @@ gdk_device_get_num_lock_state (GdkDevice *device)
 }
 
 /**
- * gdk_device_get_scroll_lock_state: (attributes org.gtk.Method.get_property=scroll-lock-state)
+ * gdk_device_get_scroll_lock_state:
  * @device: a `GdkDevice`
  *
  * Retrieves whether the Scroll Lock modifier of the keyboard is locked.
@@ -1281,7 +1321,7 @@ gdk_device_get_scroll_lock_state (GdkDevice *device)
 }
 
 /**
- * gdk_device_get_modifier_state: (attributes org.gtk.Method.get_property=modifier-state)
+ * gdk_device_get_modifier_state:
  * @device: a `GdkDevice`
  *
  * Retrieves the current modifier state of the keyboard.
@@ -1302,7 +1342,57 @@ gdk_device_get_modifier_state (GdkDevice *device)
 }
 
 /**
- * gdk_device_get_direction: (attributes org.gtk.Method.get_property=direction)
+ * gdk_device_get_active_layout_index:
+ * @device: a `GdkDevice`
+ *
+ * Retrieves the index of the active layout of the keyboard.
+ *
+ * If there is no valid active layout for the `GdkDevice`, this function will
+ * return -1;
+ *
+ * This is only relevant for keyboard devices.
+ *
+ * Returns: The layout index of the active layout or -1.
+ *
+ * Since: 4.18
+ */
+gint
+gdk_device_get_active_layout_index (GdkDevice *device)
+{
+  GdkKeymap *keymap = gdk_display_get_keymap (device->display);
+
+  if (device->source == GDK_SOURCE_KEYBOARD)
+    return gdk_keymap_get_active_layout_index (keymap);
+
+  return -1;
+}
+
+/**
+ * gdk_device_get_layout_names:
+ * @device: a `GdkDevice`
+ *
+ * Retrieves the names of the layouts of the keyboard.
+ *
+ * This is only relevant for keyboard devices.
+ *
+ * Returns: (transfer full) (nullable) (array zero-terminated=1):
+ *   %NULL-terminated array of strings of layouts,
+ *
+ * Since: 4.18
+ */
+char **
+gdk_device_get_layout_names (GdkDevice *device)
+{
+  GdkKeymap *keymap = gdk_display_get_keymap (device->display);
+
+  if (device->source == GDK_SOURCE_KEYBOARD)
+    return gdk_keymap_get_layout_names (keymap);
+
+  return NULL;
+}
+
+/**
+ * gdk_device_get_direction:
  * @device: a `GdkDevice`
  *
  * Returns the direction of effective layout of the keyboard.
@@ -1328,7 +1418,7 @@ gdk_device_get_direction (GdkDevice *device)
 }
 
 /**
- * gdk_device_has_bidi_layouts: (attributes org.gtk.Method.get_property=has-bidi-layouts)
+ * gdk_device_has_bidi_layouts:
  * @device: a `GdkDevice`
  *
  * Determines if layouts for both right-to-left and
