@@ -28,6 +28,7 @@
 #include "gdkdisplaymanagerprivate.h"
 #include "gdkdisplayprivate.h"
 #include "gdkkeysprivate.h"
+#include "gdkprivate.h"
 #include <glib/gi18n-lib.h>
 
 #ifdef GDK_WINDOWING_X11
@@ -35,6 +36,10 @@
 #include "x11/gdkprivate-x11.h"
 #endif
 
+#ifdef GDK_WINDOWING_ANDROID
+#include "android/gdkandroiddisplay-private.h"
+#endif
+
 #ifdef GDK_WINDOWING_BROADWAY
 #include "broadway/gdkprivate-broadway.h"
 #endif
@@ -149,7 +154,7 @@ gdk_display_manager_class_init (GdkDisplayManagerClass *klass)
                   GDK_TYPE_DISPLAY);
 
   /**
-   * GdkDisplayManager:default-display: (attributes org.gtk.Property.get=gdk_display_manager_get_default_display)
+   * GdkDisplayManager:default-display:
    *
    * The default display.
    */
@@ -254,7 +259,12 @@ struct _GdkBackend {
   GdkDisplay * (* open_display) (const char *name);
 };
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 static GdkBackend gdk_backends[] = {
+#ifdef GDK_WINDOWING_ANDROID
+  { "android", _gdk_android_display_open },
+#endif
 #ifdef GDK_WINDOWING_MACOS
   { "macos",   _gdk_macos_display_open },
 #endif
@@ -274,6 +284,8 @@ static GdkBackend gdk_backends[] = {
   { NULL, NULL }
 };
 
+G_GNUC_END_IGNORE_DEPRECATIONS
+
 /**
  * gdk_display_manager_get:
  *
@@ -294,6 +306,8 @@ gdk_display_manager_get (void)
 {
   static GdkDisplayManager *manager = NULL;
 
+  gdk_ensure_initialized ();
+
   if (manager == NULL)
     manager = g_object_new (GDK_TYPE_DISPLAY_MANAGER, NULL);
 
@@ -301,7 +315,7 @@ gdk_display_manager_get (void)
 }
 
 /**
- * gdk_display_manager_get_default_display: (attributes org.gtk.Method.get_property=default-display)
+ * gdk_display_manager_get_default_display:
  * @manager: a `GdkDisplayManager`
  *
  * Gets the default `GdkDisplay`.
@@ -329,6 +343,9 @@ gdk_display_manager_get_default_display (GdkDisplayManager *manager)
 GdkDisplay *
 gdk_display_get_default (void)
 {
+  if (!gdk_is_initialized ())
+    return NULL;
+
   return gdk_display_manager_get_default_display (gdk_display_manager_get ());
 }
 
