@@ -300,6 +300,10 @@ surface_present_mode_to_string (VkPresentModeKHR present_mode)
       return "VK_PRESENT_MODE_FIFO_KHR";
     case VK_PRESENT_MODE_FIFO_RELAXED_KHR:
       return "VK_PRESENT_MODE_FIFO_RELAXED_KHR";
+#if VK_HEADER_VERSION >= 304
+    case VK_PRESENT_MODE_FIFO_LATEST_READY_EXT:
+      return "VK_PRESENT_MODE_FIFO_LATEST_READY_EXT";
+#endif
     case VK_PRESENT_MODE_SHARED_DEMAND_REFRESH_KHR:
     case VK_PRESENT_MODE_SHARED_CONTINUOUS_REFRESH_KHR:
     case VK_PRESENT_MODE_MAX_ENUM_KHR:
@@ -1681,7 +1685,7 @@ gdk_display_create_vulkan_device (GdkDisplay  *display,
   return FALSE;
 }
 
-static VkBool32 VKAPI_CALL
+static VKAPI_ATTR VkBool32 VKAPI_CALL
 gdk_vulkan_debug_report (VkDebugReportFlagsEXT      flags,
                          VkDebugReportObjectTypeEXT objectType,
                          uint64_t                   object,
