@@ -32,7 +32,6 @@
 
 #include "gdk/gdkeventsprivate.h"
 
-#define GDK_MOD2_MASK (1 << 4)
 #define GRIP_WIDTH 15
 #define GRIP_HEIGHT 15
 #define GDK_LION_RESIZE 5
@@ -43,42 +42,11 @@ test_resize (NSEvent         *event,
              int              x,
              int              y)
 {
-  NSWindow *window;
-
   g_assert (event != NULL);
   g_assert (GDK_IS_MACOS_SURFACE (surface));
 
-  window = _gdk_macos_surface_get_native (surface);
-
-  /* Resizing from the resize indicator only begins if an NSLeftMouseButton
-   * event is received in the resizing area.
-   */
-  if ([event type] == NSEventTypeLeftMouseDown &&
-      [window showsResizeIndicator])
-    {
-      NSRect frame;
-
-      /* If the resize indicator is visible and the event is in the lower
-       * right 15x15 corner, we leave these events to Cocoa as to be
-       * handled as resize events.  Applications may have widgets in this
-       * area.  These will most likely be larger than 15x15 and for scroll
-       * bars there are also other means to move the scroll bar.  Since
-       * the resize indicator is the only way of resizing windows on Mac
-       * OS, it is too important to not make functional.
-       */
-      frame = [[window contentView] bounds];
-      if (x > frame.size.width - GRIP_WIDTH &&
-          x < frame.size.width &&
-          y > frame.size.height - GRIP_HEIGHT &&
-          y < frame.size.height)
-        return TRUE;
-     }
-
   /* If we're on Lion and within 5 pixels of an edge, then assume that the
    * user wants to resize, and return NULL to let Quartz get on with it.
-   * We check the selector isRestorable to see if we're on 10.7.  This
-   * extra check is in case the user starts dragging before GDK recognizes
-   * the grab.
    *
    * We perform this check for a button press of all buttons, because we
    * do receive, for instance, a right mouse down event for a GDK surface
@@ -175,7 +143,7 @@ get_keyboard_modifiers_from_ns_flags (NSUInteger nsflags)
   if (nsflags & NSEventModifierFlagOption)
     modifiers |= GDK_ALT_MASK;
   if (nsflags & NSEventModifierFlagCommand)
-    modifiers |= GDK_MOD2_MASK;
+    modifiers |= GDK_META_MASK;
 
   return modifiers;
 }
@@ -314,13 +282,6 @@ get_group_from_ns_event (NSEvent *nsevent)
   return ([nsevent modifierFlags] & NSEventModifierFlagOption) ? 1 : 0;
 }
 
-static void
-add_virtual_modifiers (GdkModifierType *state)
-{
-  if (*state & GDK_MOD2_MASK)
-    *state |= GDK_META_MASK;
-}
-
 static GdkEvent *
 fill_key_event (GdkMacosDisplay *display,
                 GdkMacosSurface *surface,
@@ -367,7 +328,7 @@ fill_key_event (GdkMacosDisplay *display,
         {
         case GDK_KEY_Meta_R:
         case GDK_KEY_Meta_L:
-          mask = GDK_MOD2_MASK;
+          mask = GDK_META_MASK;
           break;
         case GDK_KEY_Shift_R:
         case GDK_KEY_Shift_L:
@@ -395,7 +356,6 @@ fill_key_event (GdkMacosDisplay *display,
     }
 
   state |= _gdk_macos_display_get_current_mouse_modifiers (display);
-  add_virtual_modifiers (&state);
 
   translated.keyval = keyval;
   translated.consumed = consumed;
@@ -1265,6 +1225,14 @@ _gdk_macos_display_translate (GdkMacosDisplay *self,
           _gdk_macos_display_clear_sorting (self);
         }
     }
+  else if (is_motion_event(event_type))
+    {
+      NSWindow *orig_window = [nsevent window];
+
+      if (orig_window && GDK_IS_MACOS_WINDOW (orig_window))
+        [NSApp sendEvent:nsevent];
+    }
+
   return fill_event (self, window, nsevent, x, y);
 }
 
