@@ -40,6 +40,9 @@
 #include <gdk/wayland/fractional-scale-v1-client-protocol.h>
 #include <gdk/wayland/viewporter-client-protocol.h>
 #include <gdk/wayland/presentation-time-client-protocol.h>
+#include <gdk/wayland/single-pixel-buffer-v1-client-protocol.h>
+#include <gdk/wayland/xdg-dialog-v1-client-protocol.h>
+#include <gdk/wayland/xdg-system-bell-v1-client-protocol.h>
 
 #include <glib.h>
 #include <gdk/gdkkeys.h>
@@ -48,6 +51,7 @@
 
 #include "gdkdisplayprivate.h"
 #include "gdkwaylanddevice.h"
+#include "gdkdmabuf-wayland-private.h"
 #include "cursor/wayland-cursor.h"
 
 #include <epoxy/egl.h>
@@ -56,6 +60,7 @@ G_BEGIN_DECLS
 
 #define GDK_ZWP_POINTER_GESTURES_V1_VERSION 3
 
+typedef struct _GdkWaylandColor GdkWaylandColor;
 typedef struct _GdkWaylandSelection GdkWaylandSelection;
 
 typedef struct {
@@ -72,13 +77,6 @@ typedef enum _GdkWaylandShellVariant
   GDK_WAYLAND_SHELL_VARIANT_ZXDG_SHELL_V6
 } GdkWaylandShellVariant;
 
-typedef struct
-{
-  uint32_t fourcc;
-  uint32_t padding;
-  uint64_t modifier;
-} LinuxDmabufFormat;
-
 struct _GdkWaylandDisplay
 {
   GdkDisplay parent_instance;
@@ -104,12 +102,12 @@ struct _GdkWaylandDisplay
   struct wl_compositor *compositor;
   struct wl_shm *shm;
   struct zwp_linux_dmabuf_v1 *linux_dmabuf;
-  struct zwp_linux_dmabuf_feedback_v1 *linux_dmabuf_feedback;
-  gsize linux_dmabuf_n_formats;
-  LinuxDmabufFormat *linux_dmabuf_formats;
+  DmabufFormatsInfo *dmabuf_formats_info;
   struct xdg_wm_base *xdg_wm_base;
   struct zxdg_shell_v6 *zxdg_shell_v6;
+  struct xdg_wm_dialog_v1 *xdg_wm_dialog;
   struct gtk_shell1 *gtk_shell;
+  struct xdg_system_bell_v1 *system_bell;
   struct wl_data_device_manager *data_device_manager;
   struct wl_subcompositor *subcompositor;
   struct zwp_pointer_gestures_v1 *pointer_gestures;
@@ -127,6 +125,8 @@ struct _GdkWaylandDisplay
   struct wp_fractional_scale_manager_v1 *fractional_scale;
   struct wp_viewporter *viewporter;
   struct wp_presentation *presentation;
+  struct wp_single_pixel_buffer_manager_v1 *single_pixel_buffer;
+  GdkWaylandColor *color;
 
   GList *async_roundtrips;
 
@@ -148,6 +148,7 @@ struct _GdkWaylandDisplay
   GHashTable *cursor_surface_cache;
 
   GSource *event_source;
+  GSource *poll_source;
 
   uint32_t server_decoration_mode;
 
