@@ -18,8 +18,8 @@
 /**
  * GskRenderer:
  *
- * `GskRenderer` is a class that renders a scene graph defined via a
- * tree of [class@Gsk.RenderNode] instances.
+ * A class that renders a scene graph defined via a tree of
+ * [class@Gsk.RenderNode] instances.
  *
  * Typically you will use a `GskRenderer` instance to repeatedly call
  * [method@Gsk.Renderer.render] to update the contents of its associated
@@ -43,7 +43,7 @@
 
 #include "gskenumtypes.h"
 
-#include "gl/gskglrenderer.h"
+#include "gpu/gskglrenderer.h"
 #include "gpu/gskvulkanrenderer.h"
 #include "gdk/gdkvulkancontextprivate.h"
 #include "gdk/gdkdisplayprivate.h"
@@ -209,13 +209,13 @@ gsk_renderer_init (GskRenderer *self)
 
 /**
  * gsk_renderer_get_surface:
- * @renderer: a `GskRenderer`
+ * @renderer: a renderer
  *
- * Retrieves the `GdkSurface` set using gsk_enderer_realize().
+ * Retrieves the surface that the renderer is associated with.
  *
- * If the renderer has not been realized yet, %NULL will be returned.
+ * If the renderer has not been realized yet, `NULL` will be returned.
  *
- * Returns: (transfer none) (nullable): a `GdkSurface`
+ * Returns: (transfer none) (nullable): the surface
  */
 GdkSurface *
 gsk_renderer_get_surface (GskRenderer *renderer)
@@ -229,11 +229,11 @@ gsk_renderer_get_surface (GskRenderer *renderer)
 
 /**
  * gsk_renderer_is_realized: (get-property realized)
- * @renderer: a `GskRenderer`
+ * @renderer: a renderer
  *
- * Checks whether the @renderer is realized or not.
+ * Checks whether the renderer is realized or not.
  *
- * Returns: %TRUE if the `GskRenderer` was realized, and %FALSE otherwise
+ * Returns: true if the renderer was realized, false otherwise
  */
 gboolean
 gsk_renderer_is_realized (GskRenderer *renderer)
@@ -273,22 +273,21 @@ gsk_renderer_do_realize (GskRenderer  *renderer,
 
 /**
  * gsk_renderer_realize:
- * @renderer: a `GskRenderer`
- * @surface: (nullable): the `GdkSurface` renderer will be used on
+ * @renderer: a renderer
+ * @surface: (nullable): the surface that renderer will be used on
  * @error: return location for an error
  *
- * Creates the resources needed by the @renderer to render the scene
- * graph.
+ * Creates the resources needed by the renderer.
  *
  * Since GTK 4.6, the surface may be `NULL`, which allows using
- * renderers without having to create a surface.
- * Since GTK 4.14, it is recommended to use [method@Gsk.Renderer.realize_for_display]
- * instead.
+ * renderers without having to create a surface. Since GTK 4.14,
+ * it is recommended to use [method@Gsk.Renderer.realize_for_display]
+ * for this case.
  *
- * Note that it is mandatory to call [method@Gsk.Renderer.unrealize] before
- * destroying the renderer.
+ * Note that it is mandatory to call [method@Gsk.Renderer.unrealize]
+ * before destroying the renderer.
  *
- * Returns: Whether the renderer was successfully realized
+ * Returns: whether the renderer was successfully realized
  */
 gboolean
 gsk_renderer_realize (GskRenderer  *renderer,
@@ -318,17 +317,16 @@ gsk_renderer_realize (GskRenderer  *renderer,
 
 /**
  * gsk_renderer_realize_for_display:
- * @renderer: a `GskRenderer`
- * @display: the `GdkDisplay` renderer will be used on
+ * @renderer: a renderer
+ * @display: the display that the renderer will be used on
  * @error: return location for an error
  *
- * Creates the resources needed by the @renderer to render the scene
- * graph.
+ * Creates the resources needed by the renderer.
  *
- * Note that it is mandatory to call [method@Gsk.Renderer.unrealize] before
- * destroying the renderer.
+ * Note that it is mandatory to call [method@Gsk.Renderer.unrealize]
+ * before destroying the renderer.
  *
- * Returns: Whether the renderer was successfully realized
+ * Returns: whether the renderer was successfully realized
  *
  * Since: 4.14
  */
@@ -347,9 +345,9 @@ gsk_renderer_realize_for_display (GskRenderer  *renderer,
 
 /**
  * gsk_renderer_unrealize:
- * @renderer: a `GskRenderer`
+ * @renderer: a renderer
  *
- * Releases all the resources created by gsk_renderer_realize().
+ * Releases all the resources created by [method@Gsk.Renderer.realize].
  */
 void
 gsk_renderer_unrealize (GskRenderer *renderer)
@@ -378,20 +376,20 @@ gsk_renderer_unrealize (GskRenderer *renderer)
 
 /**
  * gsk_renderer_render_texture:
- * @renderer: a realized `GskRenderer`
- * @root: a `GskRenderNode`
- * @viewport: (nullable): the section to draw or %NULL to use @root's bounds
+ * @renderer: a realized renderer
+ * @root: the render node to render
+ * @viewport: (nullable): the section to draw or `NULL` to use @root's bounds
  *
- * Renders the scene graph, described by a tree of `GskRenderNode` instances,
- * to a `GdkTexture`.
+ * Renders a scene graph, described by a tree of `GskRenderNode` instances,
+ * to a texture.
  *
- * The @renderer will acquire a reference on the `GskRenderNode` tree while
+ * The renderer will acquire a reference on the `GskRenderNode` tree while
  * the rendering is in progress.
  *
  * If you want to apply any transformations to @root, you should put it into a
  * transform node and pass that node instead.
  *
- * Returns: (transfer full): a `GdkTexture` with the rendered contents of @root.
+ * Returns: (transfer full): a texture with the rendered contents of @root
  */
 GdkTexture *
 gsk_renderer_render_texture (GskRenderer           *renderer,
@@ -421,13 +419,13 @@ gsk_renderer_render_texture (GskRenderer           *renderer,
 
 /**
  * gsk_renderer_render:
- * @renderer: a realized `GskRenderer`
- * @root: a `GskRenderNode`
- * @region: (nullable): the `cairo_region_t` that must be redrawn or %NULL
- *   for the whole window
+ * @renderer: a realized renderer
+ * @root: the render node to render
+ * @region: (nullable): the `cairo_region_t` that must be redrawn or `NULL`
+ *   for the whole surface
  *
  * Renders the scene graph, described by a tree of `GskRenderNode` instances
- * to the renderer's surface,  ensuring that the given @region gets redrawn.
+ * to the renderer's surface, ensuring that the given region gets redrawn.
  *
  * If the renderer has no associated surface, this function does nothing.
  *
@@ -436,7 +434,7 @@ gsk_renderer_render_texture (GskRenderer           *renderer,
  * free to not redraw any pixel outside of @region if they can guarantee that
  * it didn't change.
  *
- * The @renderer will acquire a reference on the `GskRenderNode` tree while
+ * The renderer will acquire a reference on the `GskRenderNode` tree while
  * the rendering is in progress.
  */
 void
@@ -489,7 +487,7 @@ gsk_renderer_render (GskRenderer          *renderer,
 
 /*< private >
  * gsk_renderer_get_profiler:
- * @renderer: a `GskRenderer`
+ * @renderer: a renderer
  *
  * Retrieves a pointer to the `GskProfiler` instance of the renderer.
  *
@@ -516,32 +514,39 @@ get_renderer_for_name (const char *renderer_name)
 #endif
   else if (g_ascii_strcasecmp (renderer_name, "cairo") == 0)
     return GSK_TYPE_CAIRO_RENDERER;
-  else if (g_ascii_strcasecmp (renderer_name, "opengl") == 0 ||
-           g_ascii_strcasecmp (renderer_name, "gl") == 0)
+  else if (g_ascii_strcasecmp (renderer_name, "ngl") == 0 ||
+           g_ascii_strcasecmp (renderer_name, "opengl") == 0)
     return GSK_TYPE_GL_RENDERER;
-  else if (g_ascii_strcasecmp (renderer_name, "ngl") == 0)
-    return gsk_ngl_renderer_get_type ();
+  else if (g_ascii_strcasecmp (renderer_name, "gl") == 0)
+    {
+      g_warning ("The old GL renderer has been removed. Try GSK_RENDERER=help");
+      return GSK_TYPE_GL_RENDERER;
+    }
 #ifdef GDK_RENDERING_VULKAN
   else if (g_ascii_strcasecmp (renderer_name, "vulkan") == 0)
     return GSK_TYPE_VULKAN_RENDERER;
 #endif
   else if (g_ascii_strcasecmp (renderer_name, "help") == 0)
     {
-      g_print ("Supported arguments for GSK_RENDERER environment variable:\n");
+      fprintf (stderr, "Supported arguments for GSK_RENDERER environment variable:\n");
 #ifdef GDK_WINDOWING_BROADWAY
-      g_print ("  broadway - Use the Broadway specific renderer\n");
+      fprintf (stderr, "  broadway - Use the Broadway specific renderer\n");
 #else
-      g_print ("  broadway - Disabled during GTK build\n");
+      fprintf (stderr, "  broadway - Disabled during GTK build\n");
 #endif
-      g_print ("     cairo - Use the Cairo fallback renderer\n");
-      g_print ("       ngl - Use the OpenGL renderer\n");
+      fprintf (stderr, "     cairo - Use the Cairo fallback renderer\n");
+      fprintf (stderr, "    opengl - Use the OpenGL renderer\n");
+      fprintf (stderr, "       ngl - Use the OpenGL renderer\n");
+      fprintf (stderr, "        gl - Use the OpenGL renderer\n");
 #ifdef GDK_RENDERING_VULKAN
-      g_print ("    vulkan - Use the Vulkan renderer\n");
+      fprintf (stderr, "    vulkan - Use the Vulkan renderer\n");
 #else
-      g_print ("    vulkan - Disabled during GTK build\n");
+      fprintf (stderr, "    vulkan - Disabled during GTK build\n");
 #endif
-      g_print ("      help - Print this help\n\n");
-      g_print ("Other arguments will cause a warning and be ignored.\n");
+      fprintf (stderr, "      help - Print this help\n\n");
+      fprintf (stderr, "The old OpenGL renderer has been removed in GTK 4.18, so using\n");
+      fprintf (stderr, "GSK_RENDERER=gl will cause a warning and use the new OpenGL renderer.\n\n");
+      fprintf (stderr, "Other arguments will cause a warning and be ignored.\n");
     }
   else
     {
@@ -628,19 +633,19 @@ gl_supported_platform (GdkSurface *surface,
 static GType
 get_renderer_for_gl (GdkSurface *surface)
 {
-  if (!gl_supported_platform (surface, gsk_ngl_renderer_get_type (), FALSE))
+  if (!gl_supported_platform (surface, GSK_TYPE_GL_RENDERER, FALSE))
     return G_TYPE_INVALID;
 
-  return gsk_ngl_renderer_get_type ();
+  return GSK_TYPE_GL_RENDERER;
 }
 
 static GType
 get_renderer_for_gl_fallback (GdkSurface *surface)
 {
-  if (!gl_supported_platform (surface, gsk_ngl_renderer_get_type (), TRUE))
+  if (!gl_supported_platform (surface, GSK_TYPE_GL_RENDERER, TRUE))
     return G_TYPE_INVALID;
 
-  return gsk_ngl_renderer_get_type ();
+  return GSK_TYPE_GL_RENDERER;
 }
 
 #ifdef GDK_RENDERING_VULKAN
@@ -741,9 +746,9 @@ static struct {
 
 /**
  * gsk_renderer_new_for_surface:
- * @surface: a `GdkSurface`
+ * @surface: a surface
  *
- * Creates an appropriate `GskRenderer` instance for the given @surface.
+ * Creates an appropriate `GskRenderer` instance for the given surface.
  *
  * If the `GSK_RENDERER` environment variable is set, GSK will
  * try that renderer first, before trying the backend-specific
@@ -751,7 +756,7 @@ static struct {
  *
  * The renderer will be realized before it is returned.
  *
- * Returns: (transfer full) (nullable): a `GskRenderer`
+ * Returns: (transfer full) (nullable): the realized renderer
  */
 GskRenderer *
 gsk_renderer_new_for_surface (GdkSurface *surface)
@@ -814,4 +819,3 @@ gsk_renderer_set_debug_flags (GskRenderer   *renderer,
 
   priv->debug_flags = flags;
 }
-
