@@ -36,35 +36,29 @@
 #include "gtkbutton.h"
 #include "gtkgrid.h"
 #include "gtkbox.h"
-#include "gtkicontheme.h"
 #include "gtkimage.h"
 #include "gtklabel.h"
 #include "gtkmarshalers.h"
 #include "gtkstack.h"
-#include "gtkorientable.h"
-#include "gtkscrolledwindow.h"
 #include "gtktextview.h"
-#include "gtkshow.h"
-#include "gtkmain.h"
-#include "gtktogglebutton.h"
 #include "gtktypebuiltins.h"
 #include "gtkstack.h"
-#include "gtkstackswitcher.h"
 #include "gtksettings.h"
-#include "gtkheaderbar.h"
+#include "gtkurilauncher.h"
 #include "gtkprivate.h"
 #include <glib/gi18n-lib.h>
 #include "gtkeventcontrollermotion.h"
 #include "gtkeventcontrollerkey.h"
 #include "gtkgestureclick.h"
-#include "gtkstylecontext.h"
+#include "gtkcssnodeprivate.h"
+#include "gtkwidgetprivate.h"
+#include "gtkcsscolorvalueprivate.h"
 
 
 /**
  * GtkAboutDialog:
  *
- * The `GtkAboutDialog` offers a simple way to display information about
- * a program.
+ * A simple way to display information about a program.
  *
  * The shown information includes the programs' logo, name, copyright,
  * website and license. It is also possible to give credits to the authors,
@@ -76,7 +70,7 @@
  * ![An example GtkAboutDialog](aboutdialog.png)
  *
  * About dialogs often contain links and email addresses. `GtkAboutDialog`
- * displays these as clickable links. By default, it calls [func@Gtk.show_uri]
+ * displays these as clickable links. By default, it calls [method@Gtk.FileLauncher.launch]
  * when a user clicks one. The behaviour can be overridden with the
  * [signal@Gtk.AboutDialog::activate-link] signal.
  *
@@ -84,14 +78,14 @@
  * `Edgar Allan Poe <edgar@poe.com>`. To specify a website with a title,
  * use a string like `GTK team https://www.gtk.org`.
  *
- * To make constructing a `GtkAboutDialog` as convenient as possible, you can
+ * To make constructing an about dialog as convenient as possible, you can
  * use the function [func@Gtk.show_about_dialog] which constructs and shows
  * a dialog and keeps it around so that it can be shown again.
  *
  * Note that GTK sets a default title of `_("About %s")` on the dialog
  * window (where `%s` is replaced by the name of the application, but in
  * order to ensure proper translation of the title, applications should
- * set the title property explicitly when constructing a `GtkAboutDialog`,
+ * set the title property explicitly when constructing an about dialog,
  * as shown in the following example:
  *
  * ```c
@@ -106,6 +100,12 @@
  *                        NULL);
  * ```
  *
+ * ## Shortcuts and Gestures
+ *
+ * `GtkAboutDialog` supports the following keyboard shortcuts:
+ *
+ * - <kbd>Escape</kbd> closes the window.
+ *
  * ## CSS nodes
  *
  * `GtkAboutDialog` has a single CSS node with the name `window` and style
@@ -138,11 +138,12 @@ static const LicenseInfo gtk_license_info [] = {
   { N_("GNU Affero General Public License, version 3 only"), "https://www.gnu.org/licenses/agpl-3.0.html" },
   { N_("BSD 3-Clause License"), "https://opensource.org/licenses/BSD-3-Clause" },
   { N_("Apache License, Version 2.0"), "https://opensource.org/licenses/Apache-2.0" },
-  { N_("Mozilla Public License 2.0"), "https://opensource.org/licenses/MPL-2.0" }
+  { N_("Mozilla Public License 2.0"), "https://opensource.org/licenses/MPL-2.0" },
+  { N_("BSD Zero-Clause License"), "https://opensource.org/license/0bsd" }
 };
 /* Keep this static assertion updated with the last element of the
  * enumeration, and make sure it matches the last element of the array */
-G_STATIC_ASSERT (G_N_ELEMENTS (gtk_license_info) - 1 == GTK_LICENSE_MPL_2_0);
+G_STATIC_ASSERT (G_N_ELEMENTS (gtk_license_info) - 1 == GTK_LICENSE_0BSD);
 
 typedef struct
 {
@@ -197,11 +198,18 @@ struct _GtkAboutDialog
 
   GPtrArray *visited_links;
 
+  GtkCssNode *link_node;
+  GtkCssNode *visited_link_node;
+
   GtkLicense license_type;
 
   guint hovering_over_link : 1;
   guint wrap_license : 1;
   guint in_child_changed : 1;
+
+  GSList *link_tags;
+
+  guint update_links_cb_id;
 };
 
 struct _GtkAboutDialogClass
@@ -350,13 +358,13 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 
   /**
    * GtkAboutDialog::activate-link:
-   * @label: The object on which the signal was emitted
+   * @label: the about dialog which emitted the signal
    * @uri: the URI that is activated
    *
    * Emitted every time a URL is activated.
    *
    * Applications may connect to it to override the default behaviour,
-   * which is to call [func@Gtk.show_uri].
+   * which is to call [method@Gtk.FileLauncher.launch].
    *
    * Returns: `TRUE` if the link has been activated
    */
@@ -368,14 +376,17 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                   _gtk_boolean_handled_accumulator, NULL,
                   _gtk_marshal_BOOLEAN__STRING,
                   G_TYPE_BOOLEAN, 1, G_TYPE_STRING);
+  g_signal_set_va_marshaller (signals[ACTIVATE_LINK],
+                              G_TYPE_FROM_CLASS (object_class),
+                              _gtk_marshal_BOOLEAN__STRINGv);
 
   /**
-   * GtkAboutDialog:program-name: (attributes org.gtk.Property.get=gtk_about_dialog_get_program_name org.gtk.Property.set=gtk_about_dialog_set_program_name)
+   * GtkAboutDialog:program-name:
    *
    * The name of the program.
    *
    * If this is not set, it defaults to the value returned by
-   * `g_get_application_name()`.
+   * [func@GLib.get_application_name].
    */
   props[PROP_NAME] =
     g_param_spec_string ("program-name", NULL, NULL,
@@ -383,7 +394,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:version: (attributes org.gtk.Property.get=gtk_about_dialog_get_version org.gtk.Property.set=gtk_about_dialog_set_version)
+   * GtkAboutDialog:version:
    *
    * The version of the program.
    */
@@ -393,7 +404,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:copyright: (attributes org.gtk.Property.get=gtk_about_dialog_get_copyright org.gtk.Property.set=gtk_about_dialog_set_copyright)
+   * GtkAboutDialog:copyright:
    *
    * Copyright information for the program.
    */
@@ -403,7 +414,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:comments: (attributes org.gtk.Property.get=gtk_about_dialog_get_comments org.gtk.Property.set=gtk_about_dialog_set_comments)
+   * GtkAboutDialog:comments:
    *
    * Comments about the program.
    *
@@ -417,7 +428,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:license: (attributes org.gtk.Property.get=gtk_about_dialog_get_license org.gtk.Property.set=gtk_about_dialog_set_license)
+   * GtkAboutDialog:license:
    *
    * The license of the program, as free-form text.
    *
@@ -428,7 +439,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
    *
    * When setting this property to a non-`NULL` value, the
    * [property@Gtk.AboutDialog:license-type] property is set to
-   * `GTK_LICENSE_CUSTOM` as a side effect.
+   * [enum@Gtk.License.custom] as a side effect.
    *
    * The text may contain links in this format `<http://www.some.place/>`
    * and email references in the form `<mail-to@some.body>`, and these will
@@ -440,7 +451,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:system-information: (attributes org.gtk.Property.get=gtk_about_dialog_get_system_information org.gtk.Property.set=gtk_about_dialog_set_system_information)
+   * GtkAboutDialog:system-information:
    *
    * Information about the system on which the program is running.
    *
@@ -458,7 +469,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:license-type: (attributes org.gtk.Property.get=gtk_about_dialog_get_license_type org.gtk.Property.set=gtk_about_dialog_set_license_type)
+   * GtkAboutDialog:license-type:
    *
    * The license of the program.
    *
@@ -466,15 +477,15 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
    * and link the user to the appropriate online resource for the license
    * text.
    *
-   * If `GTK_LICENSE_UNKNOWN` is used, the link used will be the same
+   * If [enum@Gtk.License.unknown] is used, the link used will be the same
    * specified in the [property@Gtk.AboutDialog:website] property.
    *
-   * If `GTK_LICENSE_CUSTOM` is used, the current contents of the
+   * If [enum@Gtk.License.custom] is used, the current contents of the
    * [property@Gtk.AboutDialog:license] property are used.
    *
    * For any other [enum@Gtk.License] value, the contents of the
-   * [property@Gtk.AboutDialog:license] property are also set by this property as
-   * a side effect.
+   * [property@Gtk.AboutDialog:license] property are also set by
+   * this property as a side effect.
    */
   props[PROP_LICENSE_TYPE] =
     g_param_spec_enum ("license-type", NULL, NULL,
@@ -483,7 +494,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                        GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:website: (attributes org.gtk.Property.get=gtk_about_dialog_get_website org.gtk.Property.set=gtk_about_dialog_set_website)
+   * GtkAboutDialog:website:
    *
    * The URL for the link to the website of the program.
    *
@@ -495,7 +506,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:website-label: (attributes org.gtk.Property.get=gtk_about_dialog_get_website_label org.gtk.Property.set=gtk_about_dialog_set_website_label)
+   * GtkAboutDialog:website-label:
    *
    * The label for the link to the website of the program.
    */
@@ -505,9 +516,9 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:authors: (attributes org.gtk.Property.get=gtk_about_dialog_get_authors org.gtk.Property.set=gtk_about_dialog_set_authors)
+   * GtkAboutDialog:authors:
    *
-   * The authors of the program, as a `NULL`-terminated array of strings.
+   * The authors of the program.
    *
    * Each string may contain email addresses and URLs, which will be displayed
    * as links, see the introduction for more details.
@@ -518,9 +529,9 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                         GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:documenters: (attributes org.gtk.Property.get=gtk_about_dialog_get_documenters org.gtk.Property.set=gtk_about_dialog_set_documenters)
+   * GtkAboutDialog:documenters:
    *
-   * The people documenting the program, as a `NULL`-terminated array of strings.
+   * The people documenting the program.
    *
    * Each string may contain email addresses and URLs, which will be displayed
    * as links, see the introduction for more details.
@@ -531,10 +542,9 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                         GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:artists: (attributes org.gtk.Property.get=gtk_about_dialog_get_artists org.gtk.Property.set=gtk_about_dialog_set_artists)
+   * GtkAboutDialog:artists:
    *
-   * The people who contributed artwork to the program, as a `NULL`-terminated
-   * array of strings.
+   * The people who contributed artwork to the program.
    *
    * Each string may contain email addresses and URLs, which will be displayed
    * as links.
@@ -545,7 +555,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                         GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:translator-credits: (attributes org.gtk.Property.get=gtk_about_dialog_get_translator_credits org.gtk.Property.set=gtk_about_dialog_set_translator_credits)
+   * GtkAboutDialog:translator-credits:
    *
    * Credits to the translators.
    *
@@ -560,12 +570,12 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:logo: (attributes org.gtk.Property.get=gtk_about_dialog_get_logo org.gtk.Property.set=gtk_about_dialog_set_logo)
+   * GtkAboutDialog:logo:
    *
    * A logo for the about box.
    *
    * If it is `NULL`, the default window icon set with
-   * [id@gtk_window_set_default_icon_name] will be used.
+   * [func@Gtk.Window.set_default_icon_name] will be used.
    */
   props[PROP_LOGO] =
     g_param_spec_object ("logo", NULL, NULL,
@@ -573,7 +583,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:logo-icon-name: (attributes org.gtk.Property.get=gtk_about_dialog_get_logo_icon_name org.gtk.Property.set=gtk_about_dialog_set_logo_icon_name)
+   * GtkAboutDialog:logo-icon-name:
    *
    * A named icon to use as the logo for the about box.
    *
@@ -585,7 +595,7 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkAboutDialog:wrap-license: (attributes org.gtk.Property.get=gtk_about_dialog_get_wrap_license org.gtk.Property.set=gtk_about_dialog_set_wrap_license)
+   * GtkAboutDialog:wrap-license:
    *
    * Whether to wrap the text in the license dialog.
    */
@@ -704,9 +714,58 @@ update_credits_button_visibility (GtkAboutDialog *about)
   update_stack_switcher_visibility (about);
 }
 
+static void
+update_links_cb (GtkAboutDialog *about)
+{
+  GtkCssStyle *style;
+  GdkRGBA link_color, visited_link_color;
+  GSList *l;
+
+  style = gtk_css_node_get_style (about->link_node);
+  link_color = *gtk_css_color_value_get_rgba (style->used->color);
+
+  style = gtk_css_node_get_style (about->visited_link_node);
+  visited_link_color = *gtk_css_color_value_get_rgba (style->used->color);
+
+  for (l = about->link_tags; l != NULL; l = l->next)
+    {
+      GtkTextTag *tag = l->data;
+      GdkRGBA color;
+      const char *uri = g_object_get_data (G_OBJECT (tag), "uri");
+
+      if (uri && g_ptr_array_find_with_equal_func (about->visited_links, uri, (GCompareFunc)strcmp, NULL))
+        color = visited_link_color;
+      else
+        color = link_color;
+
+      g_object_set (G_OBJECT (tag), "foreground-rgba", &color, NULL);
+    }
+
+  about->update_links_cb_id = 0;
+}
+
+static void
+link_style_changed_cb (GtkCssNode        *node,
+                       GtkCssStyleChange *change,
+                       GtkAboutDialog    *about)
+{
+  if (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_REDRAW))
+    {
+      /* If we access the node right here, we'll end up with infinite recursion */
+      if (about->link_tags && !about->update_links_cb_id)
+        {
+          about->update_links_cb_id =
+            g_idle_add_once ((GSourceOnceFunc) update_links_cb, about);
+        }
+    }
+}
+
 static void
 gtk_about_dialog_init (GtkAboutDialog *about)
 {
+  GtkCssNode *node;
+  GtkStateFlags state;
+
   /* Data */
   about->name = NULL;
   about->version = NULL;
@@ -735,6 +794,25 @@ gtk_about_dialog_init (GtkAboutDialog *about)
   /* force defaults */
   gtk_about_dialog_set_program_name (about, NULL);
   gtk_about_dialog_set_logo (about, NULL);
+
+  node = gtk_widget_get_css_node (GTK_WIDGET (about));
+  state = gtk_css_node_get_state (node);
+
+  about->link_node = gtk_css_node_new ();
+  gtk_css_node_set_name (about->link_node, g_quark_from_static_string ("link"));
+  gtk_css_node_set_parent (about->link_node, node);
+  gtk_css_node_set_state (about->link_node, state | GTK_STATE_FLAG_LINK);
+  g_signal_connect (about->link_node, "style-changed",
+                    G_CALLBACK (link_style_changed_cb), about);
+  g_object_unref (about->link_node);
+
+  about->visited_link_node = gtk_css_node_new ();
+  gtk_css_node_set_name (about->visited_link_node, g_quark_from_static_string ("link"));
+  gtk_css_node_set_parent (about->visited_link_node, node);
+  gtk_css_node_set_state (about->visited_link_node, state | GTK_STATE_FLAG_LINK);
+  g_signal_connect (about->visited_link_node, "style-changed",
+                    G_CALLBACK (link_style_changed_cb), about);
+  g_object_unref (about->visited_link_node);
 }
 
 static void
@@ -743,7 +821,7 @@ destroy_credit_section (gpointer data)
   CreditSection *cs = data;
   g_free (cs->heading);
   g_strfreev (cs->people);
-  g_slice_free (CreditSection, data);
+  g_free (data);
 }
 
 static void
@@ -768,6 +846,10 @@ gtk_about_dialog_finalize (GObject *object)
   g_slist_free_full (about->credit_sections, destroy_credit_section);
   g_ptr_array_unref (about->visited_links);
 
+  g_slist_free (about->link_tags);
+
+  g_clear_handle_id (&about->update_links_cb_id, g_source_remove);
+
   G_OBJECT_CLASS (gtk_about_dialog_parent_class)->finalize (object);
 }
 
@@ -909,18 +991,25 @@ static gboolean
 gtk_about_dialog_activate_link (GtkAboutDialog *about,
                                 const char     *uri)
 {
-  gtk_show_uri (GTK_WINDOW (about), uri, GDK_CURRENT_TIME);
+  GtkUriLauncher *launcher;
+
+  launcher = gtk_uri_launcher_new (uri);
+
+  gtk_uri_launcher_launch (launcher, GTK_WINDOW (about), NULL, NULL, NULL);
+
+  g_object_unref (launcher);
+
   return TRUE;
 }
 
 static void
 update_website (GtkAboutDialog *about)
 {
-  gtk_widget_show (about->website_label);
+  gtk_widget_set_visible (about->website_label, TRUE);
 
   if (about->website_url)
     {
-      char *markup;
+      char *markup, *tooltip;
 
       if (about->website_text)
         {
@@ -939,23 +1028,29 @@ update_website (GtkAboutDialog *about)
 
       gtk_label_set_markup (GTK_LABEL (about->website_label), markup);
       g_free (markup);
+
+      tooltip = g_uri_unescape_string (about->website_url, NULL);
+      gtk_widget_set_tooltip_text (about->website_label, tooltip);
+      g_free (tooltip);
     }
   else
     {
       if (about->website_text)
         gtk_label_set_text (GTK_LABEL (about->website_label), about->website_text);
       else
-        gtk_widget_hide (about->website_label);
+        gtk_widget_set_visible (about->website_label, FALSE);
+
+      gtk_widget_set_tooltip_text (about->website_label, NULL);
     }
 }
 
 /**
- * gtk_about_dialog_get_program_name: (attributes org.gtk.Method.get_property=program-name)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_program_name:
+ * @about: an about dialog
  *
  * Returns the program name displayed in the about dialog.
  *
- * Returns: (nullable): The program name
+ * Returns: (nullable): the program name
  */
 const char *
 gtk_about_dialog_get_program_name (GtkAboutDialog *about)
@@ -974,13 +1069,9 @@ update_name_version (GtkAboutDialog *about)
   gtk_window_set_title (GTK_WINDOW (about), title_string);
   g_free (title_string);
 
+  gtk_widget_set_visible (about->version_label, about->version != NULL);
   if (about->version != NULL)
-    {
-      gtk_label_set_markup (GTK_LABEL (about->version_label), about->version);
-      gtk_widget_show (about->version_label);
-    }
-  else
-    gtk_widget_hide (about->version_label);
+    gtk_label_set_markup (GTK_LABEL (about->version_label), about->version);
 
   name_string = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
                                          about->name);
@@ -989,8 +1080,8 @@ update_name_version (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_program_name: (attributes org.gtk.Method.set_property=program-name)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_program_name:
+ * @about: an about dialog
  * @name: (nullable): the program name
  *
  * Sets the name to display in the about dialog.
@@ -1017,8 +1108,8 @@ gtk_about_dialog_set_program_name (GtkAboutDialog *about,
 
 
 /**
- * gtk_about_dialog_get_version: (attributes org.gtk.Method.get_property=version)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_version:
+ * @about: an about dialog
  *
  * Returns the version string.
  *
@@ -1033,8 +1124,8 @@ gtk_about_dialog_get_version (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_version: (attributes org.gtk.Method.set_property=version)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_version:
+ * @about: an about dialog
  * @version: (nullable): the version string
  *
  * Sets the version string to display in the about dialog.
@@ -1057,8 +1148,8 @@ gtk_about_dialog_set_version (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_copyright: (attributes org.gtk.Method.get_property=copyright)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_copyright:
+ * @about: an about dialog
  *
  * Returns the copyright string.
  *
@@ -1073,8 +1164,8 @@ gtk_about_dialog_get_copyright (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_copyright: (attributes org.gtk.Method.set_property=copyright)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_copyright:
+ * @about: an about dialog
  * @copyright: (nullable): the copyright string
  *
  * Sets the copyright string to display in the about dialog.
@@ -1093,24 +1184,21 @@ gtk_about_dialog_set_copyright (GtkAboutDialog *about,
   about->copyright = g_strdup (copyright);
   g_free (tmp);
 
+  gtk_widget_set_visible (about->copyright_label, about->copyright != NULL);
   if (about->copyright != NULL)
     {
       copyright_string = g_markup_printf_escaped ("<span size=\"small\">%s</span>",
                                                   about->copyright);
       gtk_label_set_markup (GTK_LABEL (about->copyright_label), copyright_string);
       g_free (copyright_string);
-
-      gtk_widget_show (about->copyright_label);
     }
-  else
-    gtk_widget_hide (about->copyright_label);
 
   g_object_notify_by_pspec (G_OBJECT (about), props[PROP_COPYRIGHT]);
 }
 
 /**
- * gtk_about_dialog_get_comments: (attributes org.gtk.Method.set_property=comments)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_comments:
+ * @about: an about dialog
  *
  * Returns the comments string.
  *
@@ -1125,8 +1213,8 @@ gtk_about_dialog_get_comments (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_comments: (attributes org.gtk.Method.set_property=comments)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_comments:
+ * @about: an about dialog
  * @comments: (nullable): a comments string
  *
  * Sets the comments string to display in the about dialog.
@@ -1146,21 +1234,21 @@ gtk_about_dialog_set_comments (GtkAboutDialog *about,
     {
       about->comments = g_strdup (comments);
       gtk_label_set_text (GTK_LABEL (about->comments_label), about->comments);
-      gtk_widget_show (about->comments_label);
     }
   else
     {
       about->comments = NULL;
-      gtk_widget_hide (about->comments_label);
     }
+
+  gtk_widget_set_visible (about->comments_label, about->comments != NULL);
   g_free (tmp);
 
   g_object_notify_by_pspec (G_OBJECT (about), props[PROP_COMMENTS]);
 }
 
 /**
- * gtk_about_dialog_get_license: (attributes org.gtk.Method.get_property=license)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_license:
+ * @about: an about dialog
  *
  * Returns the license information.
  *
@@ -1175,8 +1263,8 @@ gtk_about_dialog_get_license (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_license: (attributes org.gtk.Method.set_property=license)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_license:
+ * @about: an about dialog
  * @license: (nullable): the license information
  *
  * Sets the license information to be displayed in the
@@ -1205,7 +1293,7 @@ gtk_about_dialog_set_license (GtkAboutDialog *about,
     }
   g_free (tmp);
 
-  gtk_widget_hide (about->license_label);
+  gtk_widget_set_visible (about->license_label, FALSE);
 
   update_license_button_visibility (about);
 
@@ -1214,8 +1302,8 @@ gtk_about_dialog_set_license (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_system_information: (attributes org.gtk.Method.get_property=system-information)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_system_information:
+ * @about: an about dialog
  *
  * Returns the system information that is shown in the about dialog.
  *
@@ -1230,8 +1318,8 @@ gtk_about_dialog_get_system_information (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_system_information: (attributes org.gtk.Method.set_property=system-information)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_system_information:
+ * @about: an about dialog
  * @system_information: (nullable): system information
  *
  * Sets the system information to be displayed in the about
@@ -1256,8 +1344,8 @@ gtk_about_dialog_set_system_information (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_wrap_license: (attributes org.gtk.Method.get_property=wrap-license)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_wrap_license:
+ * @about: an about dialog
  *
  * Returns whether the license text in the about dialog is
  * automatically wrapped.
@@ -1273,8 +1361,8 @@ gtk_about_dialog_get_wrap_license (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_wrap_license: (attributes org.gtk.Method.set_property=wrap-license)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_wrap_license:
+ * @about: an about dialog
  * @wrap_license: whether to wrap the license
  *
  * Sets whether the license text in the about dialog should be
@@ -1297,8 +1385,8 @@ gtk_about_dialog_set_wrap_license (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_website: (attributes org.gtk.Method.get_property=website)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_website:
+ * @about: an about dialog
  *
  * Returns the website URL.
  *
@@ -1313,8 +1401,8 @@ gtk_about_dialog_get_website (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_website: (attributes org.gtk.Method.set_property=website)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_website:
+ * @about: an about dialog
  * @website: (nullable): a URL string starting with `http://`
  *
  * Sets the URL to use for the website link.
@@ -1337,8 +1425,8 @@ gtk_about_dialog_set_website (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_website_label: (attributes org.gtk.Method.get_property=website-label)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_website_label:
+ * @about: an about dialog
  *
  * Returns the label used for the website link.
  *
@@ -1353,8 +1441,8 @@ gtk_about_dialog_get_website_label (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_website_label: (attributes org.gtk.Method.set_property=website-label)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_website_label:
+ * @about: an about dialog
  * @website_label: the label used for the website link
  *
  * Sets the label to be used for the website link.
@@ -1377,8 +1465,8 @@ gtk_about_dialog_set_website_label (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_authors: (attributes org.gtk.Method.get_property=authors)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_authors:
+ * @about: an about dialog
  *
  * Returns the names of the authors which are displayed
  * in the credits page.
@@ -1395,8 +1483,8 @@ gtk_about_dialog_get_authors (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_authors: (attributes org.gtk.Method.set_property=authors)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_authors:
+ * @about: an about dialog
  * @authors: (array zero-terminated=1): the authors of the application
  *
  * Sets the names of the authors which are displayed
@@ -1420,8 +1508,8 @@ gtk_about_dialog_set_authors (GtkAboutDialog  *about,
 }
 
 /**
- * gtk_about_dialog_get_documenters: (attributes org.gtk.Method.get_property=documenters)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_documenters:
+ * @about: an about dialog
  *
  * Returns the name of the documenters which are displayed
  * in the credits page.
@@ -1438,8 +1526,8 @@ gtk_about_dialog_get_documenters (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_documenters: (attributes org.gtk.Method.set_property=documenters)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_documenters:
+ * @about: an about dialog
  * @documenters: (array zero-terminated=1): the authors of the documentation
  *   of the application
  *
@@ -1464,8 +1552,8 @@ gtk_about_dialog_set_documenters (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_artists: (attributes org.gtk.Method.get_property=artists)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_artists:
+ * @about: an about dialog
  *
  * Returns the names of the artists which are displayed
  * in the credits page.
@@ -1482,8 +1570,8 @@ gtk_about_dialog_get_artists (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_artists: (attributes org.gtk.Method.set_property=artists)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_artists:
+ * @about: an about dialog
  * @artists: (array zero-terminated=1): the authors of the artwork
  *   of the application
  *
@@ -1508,8 +1596,8 @@ gtk_about_dialog_set_artists (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_translator_credits: (attributes org.gtk.Method.get_property=translator-credits)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_translator_credits:
+ * @about: an about dialog
  *
  * Returns the translator credits string which is displayed
  * in the credits page.
@@ -1525,8 +1613,8 @@ gtk_about_dialog_get_translator_credits (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_translator_credits: (attributes org.gtk.Method.set_property=translator-credits)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_translator_credits:
+ * @about: an about dialog
  * @translator_credits: (nullable): the translator credits
  *
  * Sets the translator credits string which is displayed in
@@ -1566,8 +1654,8 @@ gtk_about_dialog_set_translator_credits (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_logo: (attributes org.gtk.Method.get_property=logo)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_logo:
+ * @about: an about dialog
  *
  * Returns the paintable displayed as logo in the about dialog.
  *
@@ -1587,8 +1675,8 @@ gtk_about_dialog_get_logo (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_logo: (attributes org.gtk.Method.set_property=logo)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_logo:
+ * @about: an about dialog
  * @logo: (nullable): a `GdkPaintable`
  *
  * Sets the logo in the about dialog.
@@ -1613,8 +1701,8 @@ gtk_about_dialog_set_logo (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_logo_icon_name: (attributes org.gtk.Method.get_property=logo-icon-name)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_logo_icon_name:
+ * @about: an about dialog
  *
  * Returns the icon name displayed as logo in the about dialog.
  *
@@ -1633,8 +1721,8 @@ gtk_about_dialog_get_logo_icon_name (GtkAboutDialog *about)
 }
 
 /**
- * gtk_about_dialog_set_logo_icon_name: (attributes org.gtk.Method.set_property=logo-icon-name)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_logo_icon_name:
+ * @about: an about dialog
  * @icon_name: (nullable): an icon name
  *
  * Sets the icon name to be displayed as logo in the about dialog.
@@ -1676,14 +1764,12 @@ follow_if_link (GtkAboutDialog *about,
 
       if (uri && !g_ptr_array_find_with_equal_func (about->visited_links, uri, (GCompareFunc)strcmp, NULL))
         {
-          GdkRGBA visited_link_color;
-          GtkStyleContext *context = gtk_widget_get_style_context (GTK_WIDGET (about));
-          gtk_style_context_save (context);
-          gtk_style_context_set_state (context, gtk_style_context_get_state (context) | GTK_STATE_FLAG_VISITED);
-          gtk_style_context_get_color (context, &visited_link_color);
-          gtk_style_context_restore (context);
+          const GdkRGBA *visited_link_color;
+          GtkCssStyle *style;
 
-          g_object_set (G_OBJECT (tag), "foreground-rgba", &visited_link_color, NULL);
+          style = gtk_css_node_get_style (about->visited_link_node);
+          visited_link_color = gtk_css_color_value_get_rgba (style->used->color);
+          g_object_set (G_OBJECT (tag), "foreground-rgba", visited_link_color, NULL);
 
           g_ptr_array_add (about->visited_links, g_strdup (uri));
         }
@@ -1818,20 +1904,19 @@ text_buffer_new (GtkAboutDialog  *about,
   char **p;
   char *q0, *q1, *q2, *r1, *r2;
   GtkTextBuffer *buffer;
-  GdkRGBA color;
-  GdkRGBA link_color;
-  GdkRGBA visited_link_color;
+  const GdkRGBA *color;
+  const GdkRGBA *link_color;
+  const GdkRGBA *visited_link_color;
   GtkTextIter start_iter, end_iter;
   GtkTextTag *tag;
-  GtkStateFlags state = gtk_widget_get_state_flags (GTK_WIDGET (about));
-  GtkStyleContext *context = gtk_widget_get_style_context (GTK_WIDGET (about));
+  GtkCssStyle *style;
+
+  style = gtk_css_node_get_style (about->link_node);
+  link_color = gtk_css_color_value_get_rgba (style->used->color);
+
+  style = gtk_css_node_get_style (about->visited_link_node);
+  visited_link_color = gtk_css_color_value_get_rgba (style->used->color);
 
-  gtk_style_context_save (context);
-  gtk_style_context_set_state (context, state | GTK_STATE_FLAG_LINK);
-  gtk_style_context_get_color (context, &link_color);
-  gtk_style_context_set_state (context, state | GTK_STATE_FLAG_VISITED);
-  gtk_style_context_get_color (context, &visited_link_color);
-  gtk_style_context_restore (context);
   buffer = gtk_text_buffer_new (NULL);
 
   for (p = strings; *p; p++)
@@ -1891,9 +1976,12 @@ text_buffer_new (GtkAboutDialog  *about,
                 color = link_color;
 
               tag = gtk_text_buffer_create_tag (buffer, NULL,
-                                                "foreground-rgba", &color,
+                                                "foreground-rgba", color,
                                                 "underline", PANGO_UNDERLINE_SINGLE,
                                                 NULL);
+
+              about->link_tags = g_slist_prepend (about->link_tags, tag);
+
               if (strcmp (link_type, "email") == 0)
                 {
                   char *escaped;
@@ -2053,7 +2141,7 @@ add_credits_section (GtkAboutDialog  *about,
       gtk_widget_set_halign (label, GTK_ALIGN_START);
       gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
       gtk_grid_attach (grid, label, 1, *row, 1, 1);
-      gtk_widget_show (label);
+      gtk_widget_set_visible (label, TRUE);
       (*row)++;
     }
 
@@ -2148,8 +2236,7 @@ close_cb (GtkAboutDialog *about,
           gpointer        user_data)
 {
   gtk_stack_set_visible_child_name (GTK_STACK (about->stack), "main");
-
-  gtk_widget_hide (GTK_WIDGET (about));
+  gtk_widget_set_visible (GTK_WIDGET (about), FALSE);
 
   return TRUE;
 }
@@ -2213,12 +2300,12 @@ gtk_show_about_dialog (GtkWindow   *parent,
 }
 
 /**
- * gtk_about_dialog_set_license_type: (attributes org.gtk.Method.set_property=license-type)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_set_license_type:
+ * @about: an about dialog
  * @license_type: the type of license
  *
- * Sets the license of the application showing the about dialog from a
- * list of known licenses.
+ * Sets the license of the application showing the about dialog
+ * from a list of known licenses.
  *
  * This function overrides the license set using
  * [method@Gtk.AboutDialog.set_license].
@@ -2237,6 +2324,7 @@ gtk_about_dialog_set_license_type (GtkAboutDialog *about,
 
       about->license_type = license_type;
 
+      gtk_widget_set_visible (about->license_label, TRUE);
       /* custom licenses use the contents of the :license property */
       if (about->license_type != GTK_LICENSE_CUSTOM)
         {
@@ -2264,15 +2352,10 @@ gtk_about_dialog_set_license_type (GtkAboutDialog *about,
                                             about->license);
           gtk_label_set_markup (GTK_LABEL (about->license_label), license_string);
           g_free (license_string);
-          gtk_widget_show (about->license_label);
 
           g_object_notify_by_pspec (G_OBJECT (about), props[PROP_WRAP_LICENSE]);
           g_object_notify_by_pspec (G_OBJECT (about), props[PROP_LICENSE]);
         }
-      else
-        {
-          gtk_widget_show (about->license_label);
-        }
 
       update_license_button_visibility (about);
 
@@ -2283,8 +2366,8 @@ gtk_about_dialog_set_license_type (GtkAboutDialog *about,
 }
 
 /**
- * gtk_about_dialog_get_license_type: (attributes org.gtk.Method.get_property=license-type)
- * @about: a `GtkAboutDialog`
+ * gtk_about_dialog_get_license_type:
+ * @about: an about dialog
  *
  * Retrieves the license type.
  *
@@ -2300,9 +2383,9 @@ gtk_about_dialog_get_license_type (GtkAboutDialog *about)
 
 /**
  * gtk_about_dialog_add_credit_section:
- * @about: A `GtkAboutDialog`
+ * @about: an about dialog
  * @section_name: The name of the section
- * @people: (array zero-terminated=1): The people who belong to that section
+ * @people: (array zero-terminated=1): the people who belong to that section
  *
  * Creates a new section in the "Credits" page.
  */
@@ -2317,7 +2400,7 @@ gtk_about_dialog_add_credit_section (GtkAboutDialog  *about,
   g_return_if_fail (section_name != NULL);
   g_return_if_fail (people != NULL);
 
-  new_entry = g_slice_new (CreditSection);
+  new_entry = g_new (CreditSection, 1);
   new_entry->heading = g_strdup ((char *)section_name);
   new_entry->people = g_strdupv ((char **)people);
 
