@@ -49,6 +49,9 @@
 #include "gtkscalebutton.h"
 #include "print/gtkprinteroptionwidgetprivate.h"
 
+#ifdef HAVE_ACCESSKIT
+#include "a11y/gtkaccesskitcontextprivate.h"
+#endif
 #if defined(GDK_WINDOWING_X11) || defined(GDK_WINDOWING_WAYLAND)
 #include "a11y/gtkatspicontextprivate.h"
 #endif
@@ -305,7 +308,7 @@ gtk_at_context_class_init (GtkATContextClass *klass)
 }
 
 #define N_PROPERTIES    (GTK_ACCESSIBLE_PROPERTY_VALUE_TEXT + 1)
-#define N_RELATIONS     (GTK_ACCESSIBLE_RELATION_SET_SIZE + 1)
+#define N_RELATIONS     (GTK_ACCESSIBLE_RELATION_FLOW_FROM + 1)
 #define N_STATES        (GTK_ACCESSIBLE_STATE_SELECTED + 1)
 
 static const char *property_attrs[] = {
@@ -368,6 +371,12 @@ static const char *relation_attrs[] = {
   [GTK_ACCESSIBLE_RELATION_ROW_INDEX_TEXT]      = "rowindextext",
   [GTK_ACCESSIBLE_RELATION_ROW_SPAN]            = "rowspan",
   [GTK_ACCESSIBLE_RELATION_SET_SIZE]            = "setsize",
+  [GTK_ACCESSIBLE_RELATION_LABEL_FOR]           = "labelfor",
+  [GTK_ACCESSIBLE_RELATION_DESCRIPTION_FOR]     = "descriptionfor",
+  [GTK_ACCESSIBLE_RELATION_CONTROLLED_BY]       = "controlledby",
+  [GTK_ACCESSIBLE_RELATION_DETAILS_FOR]         = "detailsfor",
+  [GTK_ACCESSIBLE_RELATION_ERROR_MESSAGE_FOR]   = "errormessagefor",
+  [GTK_ACCESSIBLE_RELATION_FLOW_FROM]           = "flowfrom",
 };
 
 /*< private >
@@ -382,7 +391,7 @@ const char *
 gtk_accessible_relation_get_attribute_name (GtkAccessibleRelation relation)
 {
   g_return_val_if_fail (relation >= GTK_ACCESSIBLE_RELATION_ACTIVE_DESCENDANT &&
-                        relation <= GTK_ACCESSIBLE_RELATION_SET_SIZE,
+                        relation <= GTK_ACCESSIBLE_RELATION_FLOW_FROM,
                         "<none>");
 
   return relation_attrs[relation];
@@ -512,6 +521,40 @@ gtk_at_context_get_accessible_parent (GtkATContext *self)
 
 static GtkATContext * get_parent_context (GtkATContext *self);
 
+static inline void
+maybe_realize_context (GtkATContext *self)
+{
+  if (GTK_IS_WIDGET (self->accessible))
+    {
+      GtkATContext *parent_context = get_parent_context (self);
+
+      if (parent_context && parent_context->realized)
+        gtk_at_context_realize (self);
+
+      g_clear_object (&parent_context);
+    }
+  else
+    {
+      GtkAccessible *accessible_parent;
+
+      gtk_at_context_realize (self);
+
+      accessible_parent = self->accessible_parent;
+      while (accessible_parent && !GTK_IS_WIDGET (accessible_parent))
+        {
+          GtkATContext *parent_context = gtk_accessible_get_at_context (accessible_parent);
+
+          if (!parent_context)
+            break;
+
+          gtk_at_context_realize (parent_context);
+          accessible_parent = parent_context->accessible_parent;
+
+          g_clear_object (&parent_context);
+        }
+    }
+}
+
 /*< private >
  * gtk_at_context_set_accessible_parent:
  * @self: a `GtkAtContext`
@@ -534,15 +577,10 @@ gtk_at_context_set_accessible_parent (GtkATContext *self,
       self->accessible_parent = parent;
       if (self->accessible_parent != NULL)
         {
-          GtkATContext *parent_context = NULL;
-
           g_object_add_weak_pointer (G_OBJECT (self->accessible_parent),
                                      (gpointer *) &self->accessible_parent);
 
-          parent_context = get_parent_context (self);
-          if (parent_context && parent_context->realized)
-            gtk_at_context_realize (self);
-          g_clear_object (&parent_context);
+          maybe_realize_context (self);
         }
     }
 }
@@ -643,6 +681,9 @@ static const struct {
 } a11y_backends[] = {
 #if defined(GDK_WINDOWING_WAYLAND) || defined(GDK_WINDOWING_X11)
   { "AT-SPI", "atspi", gtk_at_spi_create_context },
+#endif
+#ifdef HAVE_ACCESSKIT
+  { "AccessKit", "accesskit", gtk_accesskit_create_context },
 #endif
   { "Test", "test", gtk_test_at_context_new },
 };
@@ -678,13 +719,19 @@ gtk_at_context_create (GtkAccessibleRole  accessible_role,
       if (g_ascii_strcasecmp (gtk_a11y_env, "help") == 0)
         {
           g_print ("Supported arguments for GTK_A11Y environment variable:\n");
-
-#if defined(GDK_WINDOWING_X11) || defined(GDK_WINDOWING_WAYLAND)
-          g_print ("   atspi - Use the AT-SPI accessibility backend\n");
+#ifdef HAVE_ACCESSKIT
+          g_print ("   accesskit - Use the AccessKit accessibility backend\n");
+#else
+          g_print ("   accesskit - Disabled during GTK build\n");
 #endif
-          g_print ("    test - Use the test accessibility backend\n");
-          g_print ("    none - Disable the accessibility backend\n");
-          g_print ("    help - Print this help\n\n");
+#if defined(GDK_WINDOWING_X11) || defined(GDK_WINDOWING_WAYLAND)
+          g_print ("       atspi - Use the AT-SPI accessibility backend\n");
+#else
+          g_print ("       atspi - Not available on this platform\n");
+#endif
+          g_print ("        test - Use the test accessibility backend\n");
+          g_print ("        none - Disable the accessibility backend\n");
+          g_print ("        help - Print this help\n\n");
           g_print ("Other arguments will cause a warning and be ignored.\n");
 
           gtk_a11y_env = "0";
@@ -972,6 +1019,91 @@ gtk_at_context_get_accessible_property (GtkATContext          *self,
   return gtk_accessible_attribute_set_get_value (self->properties, property);
 }
 
+static void
+append_to_accessible_relation (GtkATContext          *self,
+                                              GtkAccessibleRelation  relation,
+                                              GtkAccessible         *accessible)
+{
+  g_return_if_fail (GTK_IS_AT_CONTEXT (self));
+  GtkAccessibleValue * target_value;
+
+  if (gtk_accessible_attribute_set_contains (self->relations, relation))
+    target_value = gtk_accessible_attribute_set_get_value (self->relations, relation);
+  else
+    {
+      target_value = gtk_reference_list_accessible_value_new (NULL);
+      gtk_accessible_attribute_set_add (self->relations, relation, target_value);
+    }
+  gtk_reference_list_accessible_value_append (target_value, accessible);
+
+  self->updated_relations |= (1 << relation);
+}
+
+static void
+remove_from_accessible_relation (GtkATContext          *self,
+                                 GtkAccessibleRelation  relation,
+                                 GtkAccessible         *accessible)
+{
+  g_return_if_fail (GTK_IS_AT_CONTEXT (self));
+  GtkAccessibleValue * target_value;
+
+  if (!gtk_accessible_attribute_set_contains (self->relations, relation))
+    return;
+
+  target_value = gtk_accessible_attribute_set_get_value (self->relations, relation);
+  gtk_reference_list_accessible_value_remove (target_value, accessible);
+
+  self->updated_relations |= (1 << relation);
+}
+
+static void
+update_reverse_relation (GtkATContext *self, GtkAccessibleRelation relation, GtkAccessibleValue *value)
+{
+  struct {
+    GtkAccessibleRelation rel;
+    GtkAccessibleRelation reverse_rel;
+  } reverse_rels_map[] = {
+    { GTK_ACCESSIBLE_RELATION_LABELLED_BY, GTK_ACCESSIBLE_RELATION_LABEL_FOR },
+    { GTK_ACCESSIBLE_RELATION_DESCRIBED_BY, GTK_ACCESSIBLE_RELATION_DESCRIPTION_FOR },
+    { GTK_ACCESSIBLE_RELATION_CONTROLS, GTK_ACCESSIBLE_RELATION_CONTROLLED_BY },
+    { GTK_ACCESSIBLE_RELATION_DETAILS, GTK_ACCESSIBLE_RELATION_DETAILS_FOR },
+    { GTK_ACCESSIBLE_RELATION_ERROR_MESSAGE, GTK_ACCESSIBLE_RELATION_ERROR_MESSAGE_FOR },
+    { GTK_ACCESSIBLE_RELATION_FLOW_TO, GTK_ACCESSIBLE_RELATION_FLOW_FROM },
+  };
+
+  GList *l;
+  GtkATContext *related_context;
+  for (int i = 0; i < G_N_ELEMENTS (reverse_rels_map); i++)
+    {
+      if (relation == reverse_rels_map[i].rel)
+        {
+          if (value)
+            {
+              for (l = gtk_reference_list_accessible_value_get (value); l; l = l->next)
+                {
+                  related_context = gtk_accessible_get_at_context (l->data);
+                  append_to_accessible_relation (related_context, reverse_rels_map[i].reverse_rel, self->accessible);
+                  g_clear_object (&related_context);
+                }
+            }
+          else
+            {
+              if (gtk_accessible_attribute_set_contains (self->relations, relation))
+                {
+                  GtkAccessibleValue *val = gtk_accessible_attribute_set_get_value (self->relations, relation);
+                  for (l = gtk_reference_list_accessible_value_get (val); l; l = l->next)
+                    {
+                      related_context = gtk_accessible_get_at_context (l->data);
+                      remove_from_accessible_relation (related_context, reverse_rels_map[i].reverse_rel, self->accessible);
+                      g_clear_object (&related_context);
+                    }
+                }
+            }
+          break;
+        }
+    }
+}
+
 /*< private >
  * gtk_at_context_set_accessible_relation:
  * @self: a `GtkATContext`
@@ -994,6 +1126,11 @@ gtk_at_context_set_accessible_relation (GtkATContext          *self,
 
   gboolean res = FALSE;
 
+  /* We update the reverse relation first, so we can still access
+  * the current value if we're removing it.
+  */
+  update_reverse_relation (self, relation, value);
+
   if (value != NULL)
     res = gtk_accessible_attribute_set_add (self->relations, relation, value);
   else
@@ -1001,6 +1138,7 @@ gtk_at_context_set_accessible_relation (GtkATContext          *self,
 
   if (res)
     self->updated_relations |= (1 << relation);
+
 }
 
 /*< private >
@@ -1186,8 +1324,8 @@ gtk_accessible_role_get_naming (GtkAccessibleRole role)
   return (GtkAccessibleNaming) (naming[role] & ~(NAME_FROM_AUTHOR|NAME_FROM_CONTENT));
 }
 
-static gboolean
-is_nested_button (GtkATContext *self)
+gboolean
+gtk_at_context_is_nested_button (GtkATContext *self)
 {
   GtkAccessible *accessible;
   GtkWidget *widget, *parent;
@@ -1435,11 +1573,11 @@ gboolean              check_duplicates)
    * ui file and carries all the a11y attributes, but the
    * focus ends up on the toggle button.
    */
-  if (is_nested_button (self))
+  if (gtk_at_context_is_nested_button (self))
     {
       parent = get_parent_context (self);
       self = parent;
-      if (is_nested_button (self))
+      if (gtk_at_context_is_nested_button (self))
         {
           parent = get_parent_context (parent);
           g_object_unref (self);
