@@ -484,7 +484,7 @@ gtk_column_view_allocate (GtkWidget *widget,
                           int        baseline)
 {
   GtkColumnView *self = GTK_COLUMN_VIEW (widget);
-  int full_width, header_height, min, nat, x;
+  int full_width, header_height, min, nat, x, dx;
 
   x = gtk_adjustment_get_value (self->hadjustment);
   full_width = gtk_column_view_allocate_columns (self, width);
@@ -494,12 +494,15 @@ gtk_column_view_allocate (GtkWidget *widget,
     header_height = min;
   else
     header_height = nat;
+
+  dx = (_gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL) ? -x : width - full_width + x;
+
   gtk_widget_allocate (self->header, full_width, header_height, -1,
-                       gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (-x, 0)));
+                       gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (dx, 0)));
 
   gtk_widget_allocate (GTK_WIDGET (self->listview),
                        full_width, height - header_height, -1,
-                       gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (-x, header_height)));
+                       gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (dx, header_height)));
 
   gtk_adjustment_configure (self->hadjustment,  x, 0, full_width, width * 0.1, width * 0.9, width);
 }
@@ -817,7 +820,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                              g_object_interface_find_property (iface, "vscroll-policy"));
 
   /**
-   * GtkColumnView:columns: (attributes org.gtk.Property.get=gtk_column_view_get_columns)
+   * GtkColumnView:columns:
    *
    * The list of columns.
    */
@@ -827,7 +830,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                          G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnView:enable-rubberband: (attributes org.gtk.Property.get=gtk_column_view_get_enable_rubberband org.gtk.Property.set=gtk_column_view_set_enable_rubberband)
+   * GtkColumnView:enable-rubberband:
    *
    * Allow rubberband selection.
    */
@@ -837,7 +840,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkColumnView:model: (attributes org.gtk.Property.get=gtk_column_view_get_model org.gtk.Property.set=gtk_column_view_set_model)
+   * GtkColumnView:model:
    *
    * Model for the items displayed.
    */
@@ -847,7 +850,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnView:reorderable: (attributes org.gtk.Property.get=gtk_column_view_get_reorderable org.gtk.Property.set=gtk_column_view_set_reorderable)
+   * GtkColumnView:reorderable:
    *
    * Whether columns are reorderable.
    */
@@ -857,10 +860,12 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnView:row-factory: (attributes org.gtk.Property.get=gtk_column_view_get_row_factory org.gtk.Property.set=gtk_column_view_set_row_factory)
+   * GtkColumnView:row-factory:
    *
    * The factory used for configuring rows.
    *
+   * The factory must be for configuring [class@Gtk.ColumnViewRow] objects.
+   *
    * Since: 4.12
    */
   properties[PROP_ROW_FACTORY] =
@@ -869,7 +874,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnView:show-row-separators: (attributes org.gtk.Property.get=gtk_column_view_get_show_row_separators org.gtk.Property.set=gtk_column_view_set_show_row_separators)
+   * GtkColumnView:show-row-separators:
    *
    * Show separators between rows.
    */
@@ -879,7 +884,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkColumnView:show-column-separators: (attributes org.gtk.Property.get=gtk_column_view_get_show_column_separators org.gtk.Property.set=gtk_column_view_set_show_column_separators)
+   * GtkColumnView:show-column-separators:
    *
    * Show separators between columns.
    */
@@ -889,7 +894,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkColumnView:sorter: (attributes org.gtk.Property.get=gtk_column_view_get_sorter)
+   * GtkColumnView:sorter:
    *
    * Sorter with the sorting choices of the user.
    */
@@ -899,7 +904,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                          G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkColumnView:single-click-activate: (attributes org.gtk.Property.get=gtk_column_view_get_single_click_activate org.gtk.Property.set=gtk_column_view_set_single_click_activate)
+   * GtkColumnView:single-click-activate:
    *
    * Activate rows on single click and select them on hover.
    */
@@ -909,7 +914,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkColumnView:tab-behavior: (attributes org.gtk.Property.get=gtk_column_view_get_tab_behavior org.gtk.Property.set=gtk_column_view_set_tab_behavior)
+   * GtkColumnView:tab-behavior:
    *
    * Behavior of the <kbd>Tab</kbd> key
    *
@@ -922,10 +927,12 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkColumnView:header-factory: (attributes org.gtk.Property.get=gtk_column_view_get_header_factory org.gtk.Property.set=gtk_column_view_set_header_factory)
+   * GtkColumnView:header-factory:
    *
    * Factory for creating header widgets.
    *
+   * The factory must be for configuring [class@Gtk.ListHeader] objects.
+   *
    * Since: 4.12
    */
   properties[PROP_HEADER_FACTORY] =
@@ -1065,7 +1072,9 @@ gtk_column_view_in_resize_rect (GtkColumnView       *self,
   gtk_column_view_column_get_allocation (column, NULL, &width);
   rect.size.width = width;
 
-  rect.origin.x += rect.size.width - DRAG_WIDTH / 2;
+  if (_gtk_widget_get_direction (GTK_WIDGET (self)) != GTK_TEXT_DIR_RTL)
+    rect.origin.x += rect.size.width;
+  rect.origin.x -= DRAG_WIDTH / 2;
   rect.size.width = DRAG_WIDTH;
 
   return graphene_rect_contains_point (&rect, &(graphene_point_t) { x, y});
@@ -1142,7 +1151,14 @@ header_drag_begin (GtkGestureDrag *gesture,
           gtk_column_view_column_set_fixed_width (column, size);
 
           self->drag_pos = i;
-          self->drag_x = start_x - size;
+          if (_gtk_widget_get_direction (GTK_WIDGET (self)) != GTK_TEXT_DIR_RTL)
+            self->drag_x = start_x - size;
+          else
+            {
+              int width = gtk_widget_get_width (GTK_WIDGET (self->header));
+
+              self->drag_x = width - (start_x + size);
+            }
           self->in_column_resize = TRUE;
 
           set_resize_cursor (self, TRUE);
@@ -1254,6 +1270,13 @@ update_column_resize (GtkColumnView *self,
 {
   GtkColumnViewColumn *column;
 
+  if (_gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL)
+    {
+      int width = gtk_widget_get_width (GTK_WIDGET (self->header));
+
+      x = width - x;
+    }
+
   column = g_list_model_get_item (G_LIST_MODEL (self->columns), self->drag_pos);
   gtk_column_view_column_set_fixed_width (column, MAX (x - self->drag_x, 0));
   g_object_unref (column);
@@ -1520,7 +1543,7 @@ gtk_column_view_new (GtkSelectionModel *model)
 }
 
 /**
- * gtk_column_view_get_model: (attributes org.gtk.Method.get_property=model)
+ * gtk_column_view_get_model:
  * @self: a `GtkColumnView`
  *
  * Gets the model that's currently used to read the items displayed.
@@ -1536,7 +1559,7 @@ gtk_column_view_get_model (GtkColumnView *self)
 }
 
 /**
- * gtk_column_view_set_model: (attributes org.gtk.Method.set_property=model)
+ * gtk_column_view_set_model:
  * @self: a `GtkColumnView`
  * @model: (nullable) (transfer none): the model to use
  *
@@ -1560,7 +1583,7 @@ gtk_column_view_set_model (GtkColumnView     *self,
 }
 
 /**
- * gtk_column_view_get_columns: (attributes org.gtk.Method.get_property=columns)
+ * gtk_column_view_get_columns:
  * @self: a `GtkColumnView`
  *
  * Gets the list of columns in this column view.
@@ -1580,7 +1603,7 @@ gtk_column_view_get_columns (GtkColumnView *self)
 }
 
 /**
- * gtk_column_view_set_show_row_separators: (attributes org.gtk.Method.set_property=show-row-separators)
+ * gtk_column_view_set_show_row_separators:
  * @self: a `GtkColumnView`
  * @show_row_separators: %TRUE to show row separators
  *
@@ -1602,7 +1625,7 @@ gtk_column_view_set_show_row_separators (GtkColumnView *self,
 }
 
 /**
- * gtk_column_view_get_show_row_separators: (attributes org.gtk.Method.get_property=show-row-separators)
+ * gtk_column_view_get_show_row_separators:
  * @self: a `GtkColumnView`
  *
  * Returns whether the list should show separators
@@ -1619,7 +1642,7 @@ gtk_column_view_get_show_row_separators (GtkColumnView *self)
 }
 
 /**
- * gtk_column_view_set_show_column_separators: (attributes org.gtk.Method.set_property=show-column-separators)
+ * gtk_column_view_set_show_column_separators:
  * @self: a `GtkColumnView`
  * @show_column_separators: %TRUE to show column separators
  *
@@ -1646,7 +1669,7 @@ gtk_column_view_set_show_column_separators (GtkColumnView *self,
 }
 
 /**
- * gtk_column_view_get_show_column_separators: (attributes org.gtk.Method.get_property=show-column-separators)
+ * gtk_column_view_get_show_column_separators:
  * @self: a `GtkColumnView`
  *
  * Returns whether the list should show separators
@@ -1869,7 +1892,7 @@ gtk_column_view_get_list_view (GtkColumnView *self)
 }
 
 /**
- * gtk_column_view_get_sorter: (attributes org.gtk.Method.get_property=sorter)
+ * gtk_column_view_get_sorter:
  * @self: a `GtkColumnView`
  *
  * Returns a special sorter that reflects the users sorting
@@ -1939,7 +1962,7 @@ gtk_column_view_sort_by_column (GtkColumnView       *self,
 }
 
 /**
- * gtk_column_view_set_single_click_activate: (attributes org.gtk.Method.set_property=single-click-activate)
+ * gtk_column_view_set_single_click_activate:
  * @self: a `GtkColumnView`
  * @single_click_activate: %TRUE to activate items on single click
  *
@@ -1961,7 +1984,7 @@ gtk_column_view_set_single_click_activate (GtkColumnView *self,
 }
 
 /**
- * gtk_column_view_get_single_click_activate: (attributes org.gtk.Method.get_property=single-click-activate)
+ * gtk_column_view_get_single_click_activate:
  * @self: a `GtkColumnView`
  *
  * Returns whether rows will be activated on single click and
@@ -1978,7 +2001,7 @@ gtk_column_view_get_single_click_activate (GtkColumnView *self)
 }
 
 /**
- * gtk_column_view_set_reorderable: (attributes org.gtk.Method.set_property=reorderable)
+ * gtk_column_view_set_reorderable:
  * @self: a `GtkColumnView`
  * @reorderable: whether columns should be reorderable
  *
@@ -1999,7 +2022,7 @@ gtk_column_view_set_reorderable (GtkColumnView *self,
 }
 
 /**
- * gtk_column_view_get_reorderable: (attributes org.gtk.Method.get_property=reorderable)
+ * gtk_column_view_get_reorderable:
  * @self: a `GtkColumnView`
  *
  * Returns whether columns are reorderable.
@@ -2015,7 +2038,7 @@ gtk_column_view_get_reorderable (GtkColumnView *self)
 }
 
 /**
- * gtk_column_view_set_enable_rubberband: (attributes org.gtk.Method.set_property=enable-rubberband)
+ * gtk_column_view_set_enable_rubberband:
  * @self: a `GtkColumnView`
  * @enable_rubberband: %TRUE to enable rubberband selection
  *
@@ -2036,7 +2059,7 @@ gtk_column_view_set_enable_rubberband (GtkColumnView *self,
 }
 
 /**
- * gtk_column_view_get_enable_rubberband: (attributes org.gtk.Method.get_property=enable-rubberband)
+ * gtk_column_view_get_enable_rubberband:
  * @self: a `GtkColumnView`
  *
  * Returns whether rows can be selected by dragging with the mouse.
@@ -2052,7 +2075,7 @@ gtk_column_view_get_enable_rubberband (GtkColumnView *self)
 }
 
 /**
- * gtk_column_view_set_row_factory: (attributes org.gtk.Method.set_property=row-factory)
+ * gtk_column_view_set_row_factory:
  * @self: a `GtkColumnView`
  * @factory: (nullable): The row factory
  *
@@ -2081,7 +2104,7 @@ gtk_column_view_set_row_factory (GtkColumnView      *self,
 }
 
 /**
- * gtk_column_view_get_row_factory: (attributes org.gtk.Method.get_property=row-factory)
+ * gtk_column_view_get_row_factory:
  * @self: a `GtkColumnView`
  *
  * Gets the factory set via [method@Gtk.ColumnView.set_row_factory].
@@ -2099,7 +2122,7 @@ gtk_column_view_get_row_factory (GtkColumnView *self)
 }
 
 /**
- * gtk_column_view_set_tab_behavior: (attributes org.gtk.Method.set_property=tab-behavior)
+ * gtk_column_view_set_tab_behavior:
  * @self: a `GtkColumnView`
  * @tab_behavior: The desired tab behavior
  *
@@ -2122,7 +2145,7 @@ gtk_column_view_set_tab_behavior (GtkColumnView      *self,
 }
 
 /**
- * gtk_column_view_get_tab_behavior: (attributes org.gtk.Method.get_property=tab-behavior)
+ * gtk_column_view_get_tab_behavior:
  * @self: a `GtkColumnView`
  *
  * Gets the behavior set for the <kbd>Tab</kbd> key.
@@ -2140,7 +2163,7 @@ gtk_column_view_get_tab_behavior (GtkColumnView *self)
 }
 
 /**
- * gtk_column_view_get_header_factory: (attributes org.gtk.Method.get_property=header-factory)
+ * gtk_column_view_get_header_factory:
  * @self: a `GtkColumnView`
  *
  * Gets the factory that's currently used to populate section headers.
@@ -2158,7 +2181,7 @@ gtk_column_view_get_header_factory (GtkColumnView *self)
 }
 
 /**
- * gtk_column_view_set_header_factory: (attributes org.gtk.Method.set_property=header-factory)
+ * gtk_column_view_set_header_factory:
  * @self: a `GtkColumnView`
  * @factory: (nullable) (transfer none): the factory to use
  *
