@@ -1267,9 +1267,9 @@ gtk_css_provider_load_internal (GtkCssProvider *self,
 
   if (GDK_PROFILER_IS_RUNNING)
     {
-      char *uri = g_file_get_uri (file);
+      const char *uri G_GNUC_UNUSED;
+      uri = file ? g_file_peek_path (file) : NULL;
       gdk_profiler_end_mark (before, "CSS theme load", uri);
-      g_free (uri);
     }
 }
 
@@ -1539,7 +1539,10 @@ _gtk_css_find_theme (const char *name,
   /* Next look in the user's home directory */
   path = _gtk_css_find_theme_dir (g_get_home_dir (), ".themes", name, file);
   if (path)
-    return path;
+    {
+      GTK_DEBUG (CSS, "Loading custom CSS from $HOME/.themes/ is deprecated");
+      return path;
+    }
 
   /* Look in system data directories */
   dirs = g_get_system_data_dirs ();
