@@ -22,14 +22,14 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef __GTK_ENUMS_H__
-#define __GTK_ENUMS_H__
+#pragma once
 
 #if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
 #include <glib-object.h>
+#include <gdk/version/gdkversionmacros.h>
 
 
 G_BEGIN_DECLS
@@ -41,8 +41,6 @@ G_BEGIN_DECLS
  * @GTK_ALIGN_START: snap to left or top side, leaving space on right or bottom
  * @GTK_ALIGN_END: snap to right or bottom side, leaving space on left or top
  * @GTK_ALIGN_CENTER: center natural width of widget inside the allocation
- * @GTK_ALIGN_BASELINE: align the widget according to the baseline.
- *   See [class@Gtk.Widget].
  *
  * Controls how a widget deals with extra space in a single dimension.
  *
@@ -53,12 +51,34 @@ G_BEGIN_DECLS
  * could be scaled and stretched, it could be centered, or it could be
  * positioned to one side of the space.
  *
- * Note that in horizontal context %GTK_ALIGN_START and %GTK_ALIGN_END
+ * Note that in horizontal context `GTK_ALIGN_START` and `GTK_ALIGN_END`
  * are interpreted relative to text direction.
  *
- * %GTK_ALIGN_BASELINE support is optional for containers and widgets, and
- * it is only supported for vertical alignment.  When it's not supported by
- * a child or a container it is treated as %GTK_ALIGN_FILL.
+ * Baseline support is optional for containers and widgets, and is only available
+ * for vertical alignment. `GTK_ALIGN_BASELINE_CENTER` and `GTK_ALIGN_BASELINE_FILL`
+ * are treated similar to `GTK_ALIGN_CENTER` and `GTK_ALIGN_FILL`, except that it
+ * positions the widget to line up the baselines, where that is supported.
+ */
+/**
+ * GTK_ALIGN_BASELINE:
+ *
+ * align the widget according to the baseline.
+ *
+ * Deprecated: 4.12: Use `GTK_ALIGN_BASELINE_FILL` instead
+ */
+/**
+ * GTK_ALIGN_BASELINE_FILL:
+ *
+ * a different name for `GTK_ALIGN_BASELINE`.
+ *
+ * Since: 4.12
+ */
+/**
+ * GTK_ALIGN_BASELINE_CENTER:
+ *
+ * stretch to fill all space, but align the baseline.
+ *
+ * Since: 4.12
  */
 typedef enum
 {
@@ -66,7 +86,9 @@ typedef enum
   GTK_ALIGN_START,
   GTK_ALIGN_END,
   GTK_ALIGN_CENTER,
-  GTK_ALIGN_BASELINE
+  GTK_ALIGN_BASELINE_FILL GDK_AVAILABLE_ENUMERATOR_IN_4_12,
+  GTK_ALIGN_BASELINE GDK_DEPRECATED_ENUMERATOR_IN_4_12_FOR(GTK_ALIGN_BASELINE_FILL) = GTK_ALIGN_CENTER + 1,
+  GTK_ALIGN_BASELINE_CENTER GDK_AVAILABLE_ENUMERATOR_IN_4_12,
 } GtkAlign;
 
 /**
@@ -265,6 +287,49 @@ typedef enum
   GTK_JUSTIFY_FILL
 } GtkJustification;
 
+/**
+ * GtkListTabBehavior:
+ * @GTK_LIST_TAB_ALL: Cycle through all focusable items of the list
+ * @GTK_LIST_TAB_ITEM: Cycle through a single list element, then move
+ *   focus out of the list. Moving focus between items needs to be
+ *   done with the arrow keys.
+ * @GTK_LIST_TAB_CELL: Cycle only through a single cell, then
+ *   move focus out of the list. Moving focus between cells needs to
+ *   be done with the arrow keys. This is only relevant for
+ *   cell-based widgets like #GtkColumnView, otherwise it behaves
+ *   like `GTK_LIST_TAB_ITEM`.
+ *
+ * Used to configure the focus behavior in the `GTK_DIR_TAB_FORWARD`
+ * and `GTK_DIR_TAB_BACKWARD` direction, like the <kbd>Tab</kbd> key
+ * in a [class@Gtk.ListView].
+ *
+ * Since: 4.12
+ */
+typedef enum
+{
+  GTK_LIST_TAB_ALL,
+  GTK_LIST_TAB_ITEM,
+  GTK_LIST_TAB_CELL
+} GtkListTabBehavior;
+
+/**
+ * GtkListScrollFlags:
+ * @GTK_LIST_SCROLL_NONE: Don't do anything extra
+ * @GTK_LIST_SCROLL_FOCUS: Focus the target item
+ * @GTK_LIST_SCROLL_SELECT: Select the target item and
+ *   unselect all other items.
+ *
+ * List of actions to perform when scrolling to items in
+ * a list widget.
+ *
+ * Since: 4.12
+ */
+typedef enum {
+  GTK_LIST_SCROLL_NONE      = 0,
+  GTK_LIST_SCROLL_FOCUS     = 1 << 0,
+  GTK_LIST_SCROLL_SELECT    = 1 << 1
+} GtkListScrollFlags;
+
 /**
  * GtkMessageType:
  * @GTK_MESSAGE_INFO: Informational message
@@ -616,7 +681,7 @@ typedef enum {
 GtkOrdering     gtk_ordering_from_cmpfunc       (int cmpfunc_result);
 #else
 /**
- * gtk_ordering_from_cmpfunc:
+ * gtk_ordering_from_cmpfunc: (skip)
  * @cmpfunc_result: Result of a comparison function
  *
  * Converts the result of a `GCompareFunc` like strcmp() to a
@@ -1224,16 +1289,18 @@ typedef enum {
  * @GTK_ACCESSIBLE_ROLE_COMMAND: Abstract role.
  * @GTK_ACCESSIBLE_ROLE_COMPOSITE: Abstract role.
  * @GTK_ACCESSIBLE_ROLE_DIALOG: A dialog is a window that is designed to interrupt
- *    the current processing of an application in order to prompt the user to enter
- *    information or require a response.
- * @GTK_ACCESSIBLE_ROLE_DOCUMENT: Unused
+ *   the current processing of an application in order to prompt the user to enter
+ *   information or require a response.
+ * @GTK_ACCESSIBLE_ROLE_DOCUMENT: Content that assistive technology users may want to
+ *   browse in a reading mode.
  * @GTK_ACCESSIBLE_ROLE_FEED: Unused
  * @GTK_ACCESSIBLE_ROLE_FORM: Unused
- * @GTK_ACCESSIBLE_ROLE_GENERIC: Unused
+ * @GTK_ACCESSIBLE_ROLE_GENERIC: A nameless container that has no semantic meaning
+ *   of its own. This is the role that GTK uses by default for widgets.
  * @GTK_ACCESSIBLE_ROLE_GRID: A grid of items.
  * @GTK_ACCESSIBLE_ROLE_GRID_CELL: An item in a grid or tree grid.
- * @GTK_ACCESSIBLE_ROLE_GROUP: An element that groups multiple widgets. GTK uses
- *   this role for various containers, like [class@Box], [class@Viewport], and [class@HeaderBar].
+ * @GTK_ACCESSIBLE_ROLE_GROUP: An element that groups multiple related widgets. GTK uses
+ *   this role for various containers, like [class@Gtk.HeaderBar] or [class@Gtk.Notebook].
  * @GTK_ACCESSIBLE_ROLE_HEADING: Unused
  * @GTK_ACCESSIBLE_ROLE_IMG: An image.
  * @GTK_ACCESSIBLE_ROLE_INPUT: Abstract role.
@@ -1256,13 +1323,15 @@ typedef enum {
  * @GTK_ACCESSIBLE_ROLE_MENU_ITEM_RADIO: A radio item in a menu.
  * @GTK_ACCESSIBLE_ROLE_NAVIGATION: Unused
  * @GTK_ACCESSIBLE_ROLE_NONE: An element that is not represented to accessibility technologies.
+ *   This role is synonymous to @GTK_ACCESSIBLE_ROLE_PRESENTATION.
  * @GTK_ACCESSIBLE_ROLE_NOTE: Unused
  * @GTK_ACCESSIBLE_ROLE_OPTION: Unused
  * @GTK_ACCESSIBLE_ROLE_PRESENTATION: An element that is not represented to accessibility technologies.
+ *   This role is synonymous to @GTK_ACCESSIBLE_ROLE_NONE.
  * @GTK_ACCESSIBLE_ROLE_PROGRESS_BAR: An element that displays the progress
- *    status for tasks that take a long time.
+ *   status for tasks that take a long time.
  * @GTK_ACCESSIBLE_ROLE_RADIO: A checkable input in a group of radio roles,
- *    only one of which can be checked at a time.
+ *   only one of which can be checked at a time.
  * @GTK_ACCESSIBLE_ROLE_RADIO_GROUP: Unused
  * @GTK_ACCESSIBLE_ROLE_RANGE: Abstract role.
  * @GTK_ACCESSIBLE_ROLE_REGION: Unused
@@ -1270,30 +1339,30 @@ typedef enum {
  * @GTK_ACCESSIBLE_ROLE_ROW_GROUP: Unused
  * @GTK_ACCESSIBLE_ROLE_ROW_HEADER: Unused
  * @GTK_ACCESSIBLE_ROLE_SCROLLBAR: A graphical object that controls the scrolling
- *    of content within a viewing area, regardless of whether the content is fully
- *    displayed within the viewing area.
+ *   of content within a viewing area, regardless of whether the content is fully
+ *   displayed within the viewing area.
  * @GTK_ACCESSIBLE_ROLE_SEARCH: Unused
  * @GTK_ACCESSIBLE_ROLE_SEARCH_BOX: A type of textbox intended for specifying
- *    search criteria.
+ *   search criteria.
  * @GTK_ACCESSIBLE_ROLE_SECTION: Abstract role.
  * @GTK_ACCESSIBLE_ROLE_SECTION_HEAD: Abstract role.
  * @GTK_ACCESSIBLE_ROLE_SELECT: Abstract role.
  * @GTK_ACCESSIBLE_ROLE_SEPARATOR: A divider that separates and distinguishes
- *    sections of content or groups of menuitems.
+ *   sections of content or groups of menuitems.
  * @GTK_ACCESSIBLE_ROLE_SLIDER: A user input where the user selects a value
- *    from within a given range.
+ *   from within a given range.
  * @GTK_ACCESSIBLE_ROLE_SPIN_BUTTON: A form of range that expects the user to
- *    select from among discrete choices.
+ *   select from among discrete choices.
  * @GTK_ACCESSIBLE_ROLE_STATUS: Unused
  * @GTK_ACCESSIBLE_ROLE_STRUCTURE: Abstract role.
  * @GTK_ACCESSIBLE_ROLE_SWITCH: A type of checkbox that represents on/off values,
- *    as opposed to checked/unchecked values.
+ *   as opposed to checked/unchecked values.
  * @GTK_ACCESSIBLE_ROLE_TAB: An item in a list of tab used for switching pages.
  * @GTK_ACCESSIBLE_ROLE_TABLE: Unused
  * @GTK_ACCESSIBLE_ROLE_TAB_LIST: A list of tabs for switching pages.
  * @GTK_ACCESSIBLE_ROLE_TAB_PANEL: A page in a notebook or stack.
  * @GTK_ACCESSIBLE_ROLE_TEXT_BOX: A type of input that allows free-form text
- *    as its value.
+ *   as its value.
  * @GTK_ACCESSIBLE_ROLE_TIME: Unused
  * @GTK_ACCESSIBLE_ROLE_TIMER: Unused
  * @GTK_ACCESSIBLE_ROLE_TOOLBAR: Unused
@@ -1301,18 +1370,75 @@ typedef enum {
  * @GTK_ACCESSIBLE_ROLE_TREE: Unused
  * @GTK_ACCESSIBLE_ROLE_TREE_GRID: A treeview-like, columned list.
  * @GTK_ACCESSIBLE_ROLE_TREE_ITEM: Unused
- * @GTK_ACCESSIBLE_ROLE_WIDGET: An interactive component of a graphical user
- *    interface. This is the role that GTK uses by default for widgets.
- * @GTK_ACCESSIBLE_ROLE_WINDOW: An application window.
- * @GTK_ACCESSIBLE_ROLE_TOGGLE_BUTTON: A type of push button
- *    which stays pressed until depressed by a second activation.
- *    Since: 4.10
+ * @GTK_ACCESSIBLE_ROLE_WIDGET: Abstract role for interactive components of a
+ *   graphical user interface
+ * @GTK_ACCESSIBLE_ROLE_WINDOW: Abstract role for windows.
  *
  * The accessible role for a [iface@Accessible] implementation.
  *
  * Abstract roles are only used as part of the ontology; application
  * developers must not use abstract roles in their code.
  */
+
+/**
+ * GTK_ACCESSIBLE_ROLE_TOGGLE_BUTTON:
+ *
+ * A type of push button which stays pressed until depressed by a second
+ * activation.
+ *
+ * Since: 4.10
+ */
+
+/**
+ * GTK_ACCESSIBLE_ROLE_APPLICATION:
+ *
+ * A toplevel element of a graphical user interface.
+ *
+ * This is the role that GTK uses by default for windows.
+ *
+ * Since: 4.12
+ */
+
+/**
+ * GTK_ACCESSIBLE_ROLE_PARAGRAPH:
+ *
+ * A paragraph of content.
+ *
+ * Since: 4.14
+ */
+
+/**
+ * GTK_ACCESSIBLE_ROLE_BLOCK_QUOTE:
+ *
+ * A section of content that is quoted from another source.
+ *
+ * Since: 4.14
+ */
+
+/**
+ * GTK_ACCESSIBLE_ROLE_ARTICLE:
+ *
+ * A section of a page that consists of a composition that forms an independent
+ * part of a document, page, or site.
+ *
+ * Since: 4.14
+ */
+
+/**
+ * GTK_ACCESSIBLE_ROLE_COMMENT:
+ *
+ * A comment contains content expressing reaction to other content.
+ *
+ * Since: 4.14
+ */
+
+/**
+ * GTK_ACCESSIBLE_ROLE_TERMINAL:
+ *
+ * A virtual terminal.
+ *
+ * Since: 4.14
+ */
 typedef enum {
   GTK_ACCESSIBLE_ROLE_ALERT,
   GTK_ACCESSIBLE_ROLE_ALERT_DIALOG,
@@ -1392,7 +1518,13 @@ typedef enum {
   GTK_ACCESSIBLE_ROLE_TREE_ITEM,
   GTK_ACCESSIBLE_ROLE_WIDGET,
   GTK_ACCESSIBLE_ROLE_WINDOW,
-  GTK_ACCESSIBLE_ROLE_TOGGLE_BUTTON
+  GTK_ACCESSIBLE_ROLE_TOGGLE_BUTTON GDK_AVAILABLE_ENUMERATOR_IN_4_10,
+  GTK_ACCESSIBLE_ROLE_APPLICATION GDK_AVAILABLE_ENUMERATOR_IN_4_12,
+  GTK_ACCESSIBLE_ROLE_PARAGRAPH GDK_AVAILABLE_ENUMERATOR_IN_4_14,
+  GTK_ACCESSIBLE_ROLE_BLOCK_QUOTE GDK_AVAILABLE_ENUMERATOR_IN_4_14,
+  GTK_ACCESSIBLE_ROLE_ARTICLE GDK_AVAILABLE_ENUMERATOR_IN_4_14,
+  GTK_ACCESSIBLE_ROLE_COMMENT GDK_AVAILABLE_ENUMERATOR_IN_4_14,
+  GTK_ACCESSIBLE_ROLE_TERMINAL GDK_AVAILABLE_ENUMERATOR_IN_4_14
 } GtkAccessibleRole;
 
 /**
@@ -1421,6 +1553,14 @@ typedef enum {
  *
  * The possible accessible states of a [iface@Accessible].
  */
+/**
+  * GTK_ACCESSIBLE_STATE_VISITED:
+  *
+  * Indicates that a widget with the GTK_ACCESSIBLE_ROLE_LINK has been visited.
+  * Value type: boolean.
+  *
+  * Since: 4.12
+  */
 typedef enum {
   GTK_ACCESSIBLE_STATE_BUSY,
   GTK_ACCESSIBLE_STATE_CHECKED,
@@ -1429,7 +1569,8 @@ typedef enum {
   GTK_ACCESSIBLE_STATE_HIDDEN,
   GTK_ACCESSIBLE_STATE_INVALID,
   GTK_ACCESSIBLE_STATE_PRESSED,
-  GTK_ACCESSIBLE_STATE_SELECTED
+  GTK_ACCESSIBLE_STATE_SELECTED,
+  GTK_ACCESSIBLE_STATE_VISITED GDK_AVAILABLE_ENUMERATOR_IN_4_12
 } GtkAccessibleState;
 
 /**
@@ -1453,7 +1594,10 @@ typedef enum {
  *    by an element.
  * @GTK_ACCESSIBLE_PROPERTY_KEY_SHORTCUTS: Indicates keyboard shortcuts that an
  *    author has implemented to activate or give focus to an element. Value type:
- *    string
+ *    string. The format of the value is a space-separated list of shortcuts, with
+ *    each shortcut consisting of one or more modifiers (`Control`, `Alt` or `Shift`),
+ *    followed by a non-modifier key, all separated by `+`.
+ *    Examples: `F2`, `Alt-F`, `Control+Shift+N`
  * @GTK_ACCESSIBLE_PROPERTY_LABEL: Defines a string value that labels the current
  *    element. Value type: string
  * @GTK_ACCESSIBLE_PROPERTY_LEVEL: Defines the hierarchical level of an element
@@ -1491,6 +1635,15 @@ typedef enum {
  *
  * The possible accessible properties of a [iface@Accessible].
  */
+
+/**
+ * GTK_ACCESSIBLE_PROPERTY_HELP_TEXT:
+ *
+ * Defines a string value that provides a description of non-standard keyboard
+ * interactions of the current element. Value type: string
+ *
+ * Since: 4.16
+ */
 typedef enum {
   GTK_ACCESSIBLE_PROPERTY_AUTOCOMPLETE,
   GTK_ACCESSIBLE_PROPERTY_DESCRIPTION,
@@ -1510,7 +1663,8 @@ typedef enum {
   GTK_ACCESSIBLE_PROPERTY_VALUE_MAX,
   GTK_ACCESSIBLE_PROPERTY_VALUE_MIN,
   GTK_ACCESSIBLE_PROPERTY_VALUE_NOW,
-  GTK_ACCESSIBLE_PROPERTY_VALUE_TEXT
+  GTK_ACCESSIBLE_PROPERTY_VALUE_TEXT,
+  GTK_ACCESSIBLE_PROPERTY_HELP_TEXT
 } GtkAccessibleProperty;
 
 /**
@@ -1533,8 +1687,8 @@ typedef enum {
  *    that describes the object. Value type: reference
  * @GTK_ACCESSIBLE_RELATION_DETAILS: Identifies the element (or elements) that
  *    provide additional information related to the object. Value type: reference
- * @GTK_ACCESSIBLE_RELATION_ERROR_MESSAGE: Identifies the element that provides
- *    an error message for an object. Value type: reference
+ * @GTK_ACCESSIBLE_RELATION_ERROR_MESSAGE: Identifies the element (or elements) that
+ *    provide an error message for an object. Value type: reference
  * @GTK_ACCESSIBLE_RELATION_FLOW_TO: Identifies the next element (or elements)
  *    in an alternate reading order of content which, at the user's discretion,
  *    allows assistive technology to override the general default of reading in
@@ -1564,6 +1718,69 @@ typedef enum {
  * Accessible relations can be references to other widgets,
  * integers or strings.
  */
+
+/**
+ * GTK_ACCESSIBLE_RELATION_LABEL_FOR:
+ * 
+ * Identifies the element (or elements) that are labeled by the
+ * current element. Value type: reference
+ * 
+ * This relation is managed by GTK and should not be set from application code.
+ * 
+ * Since: 4.18
+ */
+/**
+ * GTK_ACCESSIBLE_RELATION_DESCRIPTION_FOR:
+ * 
+ * Identifies the element (or elements) that are described by
+ * the current element. Value type: reference
+ * 
+ * This relation is managed by GTK and should not be set from application code.
+ * 
+ * Since: 4.18
+ */
+/**
+ * GTK_ACCESSIBLE_RELATION_CONTROLLED_BY:
+ * 
+ * Identifies the element (or elements) that the current
+ * element is controlled by. Value type: reference
+ * 
+ * This relation is managed by GTK and should not be set from application code.
+ * 
+ * Since: 4.18
+ */
+/**
+ * GTK_ACCESSIBLE_RELATION_DETAILS_FOR:
+ * 
+ * Identifies the element (or elements) for which the current
+ * element provides additional information. Value type: reference
+ * 
+ * This relation is managed by GTK and should not be set from application code.
+ * 
+ * Since: 4.18
+ */
+/**
+ * GTK_ACCESSIBLE_RELATION_ERROR_MESSAGE_FOR:
+ * 
+ * Identifies the element (or elements) for which the current
+ * element provides an error message. Value type: reference
+ * 
+ * This relation is managed by GTK and should not be set from application code.
+ * 
+ * Since: 4.18
+ */
+/**
+ * GTK_ACCESSIBLE_RELATION_FLOW_FROM:
+ * 
+ * Identifies the previous element (or elements) in an alternate
+ * reading order of content which, at the user's discretion, allows
+ * assistive technology to override the general default of reading in
+ * document source order. Value type: reference
+ *
+ * This relation is managed by GTK and should not be set from application code.
+ *
+ * Since: 4.18
+ */
 typedef enum {
   GTK_ACCESSIBLE_RELATION_ACTIVE_DESCENDANT,
   GTK_ACCESSIBLE_RELATION_COL_COUNT,
@@ -1582,7 +1799,13 @@ typedef enum {
   GTK_ACCESSIBLE_RELATION_ROW_INDEX,
   GTK_ACCESSIBLE_RELATION_ROW_INDEX_TEXT,
   GTK_ACCESSIBLE_RELATION_ROW_SPAN,
-  GTK_ACCESSIBLE_RELATION_SET_SIZE
+  GTK_ACCESSIBLE_RELATION_SET_SIZE,
+  GTK_ACCESSIBLE_RELATION_LABEL_FOR,
+  GTK_ACCESSIBLE_RELATION_DESCRIPTION_FOR,
+  GTK_ACCESSIBLE_RELATION_CONTROLLED_BY,
+  GTK_ACCESSIBLE_RELATION_DETAILS_FOR,
+  GTK_ACCESSIBLE_RELATION_ERROR_MESSAGE_FOR,
+  GTK_ACCESSIBLE_RELATION_FLOW_FROM
 } GtkAccessibleRelation;
 
 /**
@@ -1668,6 +1891,88 @@ typedef enum { /*< prefix=GTK_ACCESSIBLE_SORT >*/
   GTK_ACCESSIBLE_SORT_OTHER
 } GtkAccessibleSort;
 
-G_END_DECLS
+/**
+ * GtkAccessibleAnnouncementPriority:
+ * @GTK_ACCESSIBLE_ANNOUNCEMENT_PRIORITY_LOW: The announcement is low priority,
+ *   and might be read only on the user's request.
+ * @GTK_ACCESSIBLE_ANNOUNCEMENT_PRIORITY_MEDIUM: The announcement is of medium
+ *   priority, and is usually spoken at the next opportunity, such as at the
+ *   end of speaking the current sentence or when the user pauses typing.
+ * @GTK_ACCESSIBLE_ANNOUNCEMENT_PRIORITY_HIGH: The announcement is of high
+ *   priority, and is usually spoken immediately. Because an interruption
+ *   might disorient users or cause them to not complete their current task,
+ *   authors SHOULD NOT use high priority announcements unless the
+ *   interruption is imperative. An example would be a notification about a
+ *   critical battery power level.
+ *
+ * The priority of an accessibility announcement.
+ *
+ * Since: 4.14
+ */
+typedef enum {
+  GTK_ACCESSIBLE_ANNOUNCEMENT_PRIORITY_LOW,
+  GTK_ACCESSIBLE_ANNOUNCEMENT_PRIORITY_MEDIUM,
+  GTK_ACCESSIBLE_ANNOUNCEMENT_PRIORITY_HIGH
+} GtkAccessibleAnnouncementPriority;
 
-#endif /* __GTK_ENUMS_H__ */
+/**
+ * GtkPopoverMenuFlags:
+ * @GTK_POPOVER_MENU_NESTED: Submenus are presented as traditional, nested
+ *   popovers.
+ *
+ * Flags that affect how [class@Gtk.PopoverMenu] widgets built from
+ * a [class@Gio.MenuModel] are created and displayed.
+ */
+/**
+ * GTK_POPOVER_MENU_SLIDING:
+ *
+ * Submenus are presented as sliding submenus that replace the main menu.
+ *
+ * Since: 4.14
+ */
+typedef enum { /*< prefix=GTK_POPOVER_MENU >*/
+  GTK_POPOVER_MENU_SLIDING = 0,
+  GTK_POPOVER_MENU_NESTED = 1 << 0
+} GtkPopoverMenuFlags;
+
+/**
+ * GtkFontRendering:
+ * @GTK_FONT_RENDERING_AUTOMATIC: Set up font rendering automatically,
+ *   taking factors like screen resolution and scale into account
+ * @GTK_FONT_RENDERING_MANUAL: Follow low-level font-related settings
+ *   when configuring font rendering
+ *
+ * Values for the [property@Gtk.Settings:gtk-font-rendering] setting
+ * that influence how GTK renders fonts.
+ *
+ * Since: 4.16
+ */
+typedef enum {
+  GTK_FONT_RENDERING_AUTOMATIC,
+  GTK_FONT_RENDERING_MANUAL,
+} GtkFontRendering;
+
+/**
+ * GtkTextBufferNotifyFlags:
+ * @GTK_TEXT_BUFFER_NOTIFY_BEFORE_INSERT: Be notified before text
+ *   is inserted into the underlying buffer.
+ * @GTK_TEXT_BUFFER_NOTIFY_AFTER_INSERT: Be notified after text
+ *   has been inserted into the underlying buffer.
+ * @GTK_TEXT_BUFFER_NOTIFY_BEFORE_DELETE: Be notified before text
+ *   is deleted from the underlying buffer.
+ * @GTK_TEXT_BUFFER_NOTIFY_AFTER_DELETE: Be notified after text
+ *   has been deleted from the underlying buffer.
+ *
+ * Values for [callback@Gtk.TextBufferCommitNotify] to denote the
+ * point of the notification.
+ *
+ * Since: 4.16
+ */
+typedef enum {
+  GTK_TEXT_BUFFER_NOTIFY_BEFORE_INSERT = 1 << 0,
+  GTK_TEXT_BUFFER_NOTIFY_AFTER_INSERT  = 1 << 1,
+  GTK_TEXT_BUFFER_NOTIFY_BEFORE_DELETE = 1 << 2,
+  GTK_TEXT_BUFFER_NOTIFY_AFTER_DELETE  = 1 << 3,
+} GtkTextBufferNotifyFlags;
+
+G_END_DECLS
