@@ -27,6 +27,7 @@
 #include <glib/gi18n-lib.h>
 #include "gdk/gdkprivate.h"
 #include "gdk/gdkdebugprivate.h"
+#include "gtktextencodingprivate.h"
 
 /**
  * GtkFileDialog:
@@ -795,6 +796,21 @@ cancelled_cb (GCancellable *cancellable,
   response_cb (task, GTK_RESPONSE_CLOSE);
 }
 
+typedef struct {
+  GListModel *files;
+  char **choices;
+} TaskResult;
+
+static void
+task_result_free (gpointer data)
+{
+  TaskResult *res = data;
+
+  g_object_unref (res->files);
+  g_strfreev (res->choices);
+  g_free (res);
+}
+
 static void
 response_cb (GTask *task,
              int    response)
@@ -810,11 +826,23 @@ response_cb (GTask *task,
     {
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       GtkFileChooser *chooser;
-      GListModel *files;
+      TaskResult *res;
+      char **choices;
+
+      res = g_new0 (TaskResult, 1);
 
       chooser = GTK_FILE_CHOOSER (g_task_get_task_data (task));
-      files = gtk_file_chooser_get_files (chooser);
-      g_task_return_pointer (task, files, g_object_unref);
+      res->files = gtk_file_chooser_get_files (chooser);
+
+      choices = (char **) g_object_get_data (G_OBJECT (chooser), "choices");
+      if (choices)
+        {
+          res->choices = g_new0 (char *, g_strv_length (choices) + 1);
+          for (guint i = 0; choices[i]; i++)
+            res->choices[i] = g_strdup (gtk_file_chooser_get_choice (chooser, choices[i]));
+        }
+
+      g_task_return_pointer (task, res, task_result_free);
 G_GNUC_END_IGNORE_DEPRECATIONS
     }
   else if (response == GTK_RESPONSE_CLOSE)
@@ -908,25 +936,29 @@ create_file_chooser (GtkFileDialog        *self,
 G_GNUC_END_IGNORE_DEPRECATIONS
 
 static GFile *
-finish_file_op (GtkFileDialog  *self,
-                GTask          *task,
-                GError        **error)
+finish_file_op (GtkFileDialog   *self,
+                GTask           *task,
+                char          ***choices,
+                GError         **error)
 {
-  GListModel *files;
+  TaskResult *res;
 
-  files = g_task_propagate_pointer (task, error);
-  if (files)
+  res = g_task_propagate_pointer (task, error);
+  if (res)
     {
       GFile *file = NULL;
 
-      if (g_list_model_get_n_items (files) > 0)
-        file = g_list_model_get_item (files, 0);
+      if (g_list_model_get_n_items (res->files) > 0)
+        file = g_list_model_get_item (res->files, 0);
       else
         g_set_error_literal (error,
                              GTK_DIALOG_ERROR, GTK_DIALOG_ERROR_FAILED,
                              "No file selected");
 
-      g_object_unref (files);
+      if (choices)
+        *choices = g_strdupv (res->choices);
+
+      task_result_free (res);
 
       return file;
     }
@@ -935,15 +967,34 @@ finish_file_op (GtkFileDialog  *self,
 }
 
 static GListModel *
-finish_multiple_files_op (GtkFileDialog  *self,
-                          GTask          *task,
-                          GError        **error)
+finish_multiple_files_op (GtkFileDialog   *self,
+                          GTask           *task,
+                          char          ***choices,
+                          GError         **error)
 {
-  return G_LIST_MODEL (g_task_propagate_pointer (task, error));
+  TaskResult *res;
+
+  res = g_task_propagate_pointer (task, error);
+
+  if (res)
+    {
+      GListModel *files;
+
+      files = G_LIST_MODEL (g_object_ref (res->files));
+
+      if (choices)
+        *choices = g_strdupv (res->choices);
+
+      task_result_free (res);
+
+      return files;
+    }
+
+  return NULL;
 }
 
 /* }}} */
-/* {{{ Async API */
+/* {{{ Public API */
 
 /**
  * gtk_file_dialog_open:
@@ -1010,7 +1061,7 @@ gtk_file_dialog_open_finish (GtkFileDialog   *self,
   g_return_val_if_fail (g_task_is_valid (result, self), NULL);
   g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == gtk_file_dialog_open, NULL);
 
-  return finish_file_op (self, G_TASK (result), error);
+  return finish_file_op (self, G_TASK (result), NULL, error);
 }
 
 /**
@@ -1082,7 +1133,7 @@ gtk_file_dialog_select_folder_finish (GtkFileDialog  *self,
   g_return_val_if_fail (g_task_is_valid (result, self), NULL);
   g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == gtk_file_dialog_select_folder, NULL);
 
-  return finish_file_op (self, G_TASK (result), error);
+  return finish_file_op (self, G_TASK (result), NULL, error);
 }
 
 /**
@@ -1150,7 +1201,7 @@ gtk_file_dialog_save_finish (GtkFileDialog   *self,
   g_return_val_if_fail (g_task_is_valid (result, self), NULL);
   g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == gtk_file_dialog_save, NULL);
 
-  return finish_file_op (self, G_TASK (result), error);
+  return finish_file_op (self, G_TASK (result), NULL, error);
 }
 
 /**
@@ -1203,7 +1254,7 @@ gtk_file_dialog_open_multiple (GtkFileDialog       *self,
 /**
  * gtk_file_dialog_open_multiple_finish:
  * @self: a file dialog
- * @result: the rsult
+ * @result: the result
  * @error: return location for a [enum@Gtk.DialogError] error
  *
  * Finishes the [method@Gtk.FileDialog.open] call.
@@ -1222,7 +1273,7 @@ gtk_file_dialog_open_multiple_finish (GtkFileDialog   *self,
   g_return_val_if_fail (g_task_is_valid (result, self), NULL);
   g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == gtk_file_dialog_open_multiple, NULL);
 
-  return finish_multiple_files_op (self, G_TASK (result), error);
+  return finish_multiple_files_op (self, G_TASK (result), NULL, error);
 }
 
 /**
@@ -1295,7 +1346,371 @@ gtk_file_dialog_select_multiple_folders_finish (GtkFileDialog   *self,
   g_return_val_if_fail (g_task_is_valid (result, self), NULL);
   g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == gtk_file_dialog_select_multiple_folders, NULL);
 
-  return finish_multiple_files_op (self, G_TASK (result), error);
+  return finish_multiple_files_op (self, G_TASK (result), NULL, error);
+}
+
+/* }}} */
+/* {{{ Text file variants */
+
+/**
+ * gtk_file_dialog_open_text_file:
+ * @self: a `GtkFileDialog`
+ * @parent: (nullable): the parent `GtkWindow`
+ * @cancellable: (nullable): a `GCancellable` to cancel the operation
+ * @callback: (scope async) (closure user_data): a callback to call when the
+ *   operation is complete
+ * @user_data: data to pass to @callback
+ *
+ * Initiates a file selection operation by presenting a file chooser
+ * dialog to the user.
+ *
+ * In contrast to [method@Gtk.FileDialog.open], this function
+ * lets the user select the text encoding for the file, if possible.
+ *
+ * The @callback will be called when the dialog is dismissed.
+ *
+ * Since: 4.18
+ */
+void
+gtk_file_dialog_open_text_file (GtkFileDialog       *self,
+                                GtkWindow           *parent,
+                                GCancellable        *cancellable,
+                                GAsyncReadyCallback  callback,
+                                gpointer             user_data)
+{
+  GtkFileChooserNative *chooser;
+  GTask *task;
+  char **names;
+  char **labels;
+  const char **choices;
+
+  g_return_if_fail (GTK_IS_FILE_DIALOG (self));
+
+  chooser = create_file_chooser (self, parent, GTK_FILE_CHOOSER_ACTION_OPEN, FALSE);
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  names = gtk_text_encoding_get_names ();
+  labels = gtk_text_encoding_get_labels ();
+  gtk_file_chooser_add_choice (GTK_FILE_CHOOSER (chooser),
+                               "encoding", _("Encoding"),
+                               (const char **) names,
+                               (const char **) labels);
+  gtk_file_chooser_set_choice (GTK_FILE_CHOOSER (chooser),
+                               "encoding", "automatic");
+  g_free (names);
+  g_free (labels);
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+  choices = g_new0 (const char *, 2);
+  choices[0] = "encoding";
+  g_object_set_data_full (G_OBJECT (chooser), "choices", choices, g_free);
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_check_cancellable (task, FALSE);
+  g_task_set_source_tag (task, gtk_file_dialog_open_text_file);
+  g_task_set_task_data (task, chooser, g_object_unref);
+
+  if (cancellable)
+    g_signal_connect (cancellable, "cancelled", G_CALLBACK (cancelled_cb), task);
+
+  g_signal_connect (chooser, "response", G_CALLBACK (dialog_response), task);
+
+  gtk_native_dialog_show (GTK_NATIVE_DIALOG (chooser));
+}
+
+/**
+ * gtk_file_dialog_open_text_file_finish:
+ * @self: a `GtkFileDialog`
+ * @result: a `GAsyncResult`
+ * @encoding: (out) (transfer none): return location for the text encoding to use
+ * @error: return location for a [enum@Gtk.DialogError] error
+ *
+ * Finishes the [method@Gtk.FileDialog.open_text_file] call
+ * and returns the resulting file and text encoding.
+ *
+ * If the user has explicitly selected a text encoding to use
+ * for the file, then @encoding will be set to a codeset name that
+ * is suitable for passing to iconv_open(). Otherwise, it will
+ * be `NULL`.
+ *
+ * Returns: (nullable) (transfer full): the file that was selected
+ *
+ * Since: 4.18
+ */
+GFile *
+gtk_file_dialog_open_text_file_finish (GtkFileDialog  *self,
+                                       GAsyncResult   *result,
+                                       const char    **encoding,
+                                       GError        **error)
+{
+  char **choices = NULL;
+  GFile *file;
+
+  g_return_val_if_fail (GTK_IS_FILE_DIALOG (self), NULL);
+  g_return_val_if_fail (g_task_is_valid (result, self), NULL);
+  g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == gtk_file_dialog_open_text_file, NULL);
+
+  file = finish_file_op (self, G_TASK (result), &choices, error);
+
+  if (choices)
+    {
+      *encoding = gtk_text_encoding_from_name (choices[0]);
+      g_strfreev (choices);
+    }
+  else
+    {
+      *encoding = NULL;
+    }
+
+  return file;
+}
+
+/**
+ * gtk_file_dialog_open_multiple_text_files:
+ * @self: a file dialog
+ * @parent: (nullable): the parent window
+ * @cancellable: (nullable): a cancellable to cancel the operation
+ * @callback: (scope async) (closure user_data): a callback to call when the
+ *   operation is complete
+ * @user_data: data to pass to @callback
+ *
+ * Presents a file chooser dialog to the user.
+ *
+ * The file chooser dialog will be set up to select multiple files.
+ *
+ * The file chooser dialog will initially be opened in the directory
+ * [property@Gtk.FileDialog:initial-folder].
+ *
+ * In contrast to [method@Gtk.FileDialog.open], this function
+ * lets the user select the text encoding for the files, if possible.
+ *
+ * The @callback will be called when the dialog is dismissed.
+ *
+ * Since: 4.18
+ */
+void
+gtk_file_dialog_open_multiple_text_files (GtkFileDialog       *self,
+                                          GtkWindow           *parent,
+                                          GCancellable        *cancellable,
+                                          GAsyncReadyCallback  callback,
+                                          gpointer             user_data)
+{
+  GtkFileChooserNative *chooser;
+  GTask *task;
+  char **names;
+  char **labels;
+  const char **choices;
+
+  g_return_if_fail (GTK_IS_FILE_DIALOG (self));
+
+  chooser = create_file_chooser (self, parent, GTK_FILE_CHOOSER_ACTION_OPEN, TRUE);
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  names = gtk_text_encoding_get_names ();
+  labels = gtk_text_encoding_get_labels ();
+  gtk_file_chooser_add_choice (GTK_FILE_CHOOSER (chooser),
+                               "encoding", _("Encoding"),
+                               (const char **) names,
+                               (const char **) labels);
+  gtk_file_chooser_set_choice (GTK_FILE_CHOOSER (chooser),
+                               "encoding", "automatic");
+  g_free (names);
+  g_free (labels);
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+  choices = g_new0 (const char *, 2);
+  choices[0] = "encoding";
+  g_object_set_data_full (G_OBJECT (chooser), "choices", choices, g_free);
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_check_cancellable (task, FALSE);
+  g_task_set_source_tag (task, gtk_file_dialog_open_multiple_text_files);
+  g_task_set_task_data (task, chooser, g_object_unref);
+
+  if (cancellable)
+    g_signal_connect (cancellable, "cancelled", G_CALLBACK (cancelled_cb), task);
+
+  g_signal_connect (chooser, "response", G_CALLBACK (dialog_response), task);
+
+  gtk_native_dialog_show (GTK_NATIVE_DIALOG (chooser));
+}
+
+/**
+ * gtk_file_dialog_open_multiple_text_files_finish:
+ * @self: a file dialog
+ * @result: the result
+ * @encoding: (out) (transfer none): return location for the text encoding to use
+ * @error: return location for a [enum@Gtk.DialogError] error
+ *
+ * Finishes the [method@Gtk.FileDialog.open] call.
+ *
+ * Returns: (nullable) (transfer full): the files that were selected,
+ *   as a list model of [iface@Gio.File]
+ *
+ * Since: 4.18
+ */
+GListModel *
+gtk_file_dialog_open_multiple_text_files_finish (GtkFileDialog   *self,
+                                                 GAsyncResult    *result,
+                                                 const char     **encoding,
+                                                 GError         **error)
+{
+  char **choices = NULL;
+  GListModel *files;
+
+  g_return_val_if_fail (GTK_IS_FILE_DIALOG (self), NULL);
+  g_return_val_if_fail (g_task_is_valid (result, self), NULL);
+  g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == gtk_file_dialog_open_multiple_text_files, NULL);
+
+  files = finish_multiple_files_op (self, G_TASK (result), &choices, error);
+
+  if (choices)
+    {
+      *encoding = gtk_text_encoding_from_name (choices[0]);
+      g_strfreev (choices);
+    }
+  else
+    {
+      *encoding = NULL;
+    }
+
+  return files;
+}
+
+/**
+ * gtk_file_dialog_save_text_file:
+ * @self: a `GtkFileDialog`
+ * @parent: (nullable): the parent `GtkWindow`
+ * @cancellable: (nullable): a `GCancellable` to cancel the operation
+ * @callback: (scope async) (closure user_data): a callback to call when the
+ *   operation is complete
+ * @user_data: data to pass to @callback
+ *
+ * Initiates a file save operation by presenting a file chooser
+ * dialog to the user.
+ *
+ * In contrast to [method@Gtk.FileDialog.save], this function
+ * lets the user select the text encoding and line endings for
+ * the text file, if possible.
+ *
+ * The @callback will be called when the dialog is dismissed.
+ *
+ * Since: 4.18
+ */
+void
+gtk_file_dialog_save_text_file (GtkFileDialog       *self,
+                                GtkWindow           *parent,
+                                GCancellable        *cancellable,
+                                GAsyncReadyCallback  callback,
+                                gpointer             user_data)
+{
+  GtkFileChooserNative *chooser;
+  GTask *task;
+  char **names;
+  char **labels;
+  const char **choices;
+
+  g_return_if_fail (GTK_IS_FILE_DIALOG (self));
+
+  chooser = create_file_chooser (self, parent, GTK_FILE_CHOOSER_ACTION_SAVE, FALSE);
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  names = gtk_text_encoding_get_names ();
+  labels = gtk_text_encoding_get_labels ();
+  gtk_file_chooser_add_choice (GTK_FILE_CHOOSER (chooser),
+                               "encoding", _("Encoding"),
+                               (const char **) names,
+                               (const char **) labels);
+  gtk_file_chooser_set_choice (GTK_FILE_CHOOSER (chooser),
+                               "encoding", "automatic");
+  g_free (names);
+  g_free (labels);
+
+  names = gtk_line_ending_get_names ();
+  labels = gtk_line_ending_get_labels ();
+  gtk_file_chooser_add_choice (GTK_FILE_CHOOSER (chooser),
+                               "line_ending", _("Line Ending"),
+                               (const char **) names,
+                               (const char **) labels);
+  gtk_file_chooser_set_choice (GTK_FILE_CHOOSER (chooser),
+                               "line_ending", "as-is");
+
+  g_free (names);
+  g_free (labels);
+
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+  choices = g_new0 (const char *, 3);
+  choices[0] = "encoding";
+  choices[1] = "line_ending";
+  g_object_set_data_full (G_OBJECT (chooser), "choices", choices, g_free);
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_check_cancellable (task, FALSE);
+  g_task_set_source_tag (task, gtk_file_dialog_save_text_file);
+  g_task_set_task_data (task, chooser, g_object_unref);
+
+  if (cancellable)
+    g_signal_connect (cancellable, "cancelled", G_CALLBACK (cancelled_cb), task);
+
+  g_signal_connect (chooser, "response", G_CALLBACK (dialog_response), task);
+
+  gtk_native_dialog_show (GTK_NATIVE_DIALOG (chooser));
+}
+
+/**
+ * gtk_file_dialog_save_text_file_finish:
+ * @self: a `GtkFileDialog`
+ * @result: a `GAsyncResult`
+ * @encoding: (out) (transfer none): return location for the text encoding to use
+ * @line_ending: (out) (transfer none): return location for the line endings to use
+ * @error: return location for a [enum@Gtk.DialogError] error
+ *
+ * Finishes the [method@Gtk.FileDialog.save_text_file] call
+ * and returns the resulting file, text encoding and line endings.
+ *
+ * If the user has explicitly selected a text encoding to use
+ * for the file, then @encoding will be set to a codeset name that
+ * is suitable for passing to iconv_open(). Otherwise, it will
+ * be `NULL`.
+ *
+ * The @line_ending will be set to one of "\n", "\r\n", "\r" or "",
+ * where the latter means to preserve existing line endings.
+ *
+ * Returns: (nullable) (transfer full): the file that was selected.
+ *   Otherwise, `NULL` is returned and @error is set
+ *
+ * Since: 4.18
+ */
+GFile *
+gtk_file_dialog_save_text_file_finish (GtkFileDialog  *self,
+                                       GAsyncResult   *result,
+                                       const char    **encoding,
+                                       const char    **line_ending,
+                                       GError        **error)
+{
+  char **choices = NULL;
+  GFile *file;
+
+  g_return_val_if_fail (GTK_IS_FILE_DIALOG (self), NULL);
+  g_return_val_if_fail (g_task_is_valid (result, self), NULL);
+  g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == gtk_file_dialog_save_text_file, NULL);
+
+  file = finish_file_op (self, G_TASK (result), &choices, error);
+
+  if (choices)
+    {
+      *encoding = gtk_text_encoding_from_name (choices[0]);
+      *line_ending = gtk_line_ending_from_name (choices[1]);
+      g_strfreev (choices);
+    }
+  else
+    {
+      *encoding = NULL;
+      *line_ending = "\n";
+    }
+
+  return file;
 }
 
 /* }}} */
