@@ -31,6 +31,7 @@
 #include "gtkbuildable.h"
 #include "gtkwidgetprivate.h"
 #include "gtklabel.h"
+#include "gtkbuilderprivate.h"
 
 /**
  * GtkFrame:
@@ -77,6 +78,10 @@
  * `GtkFrame` has a main CSS node with name “frame”, which is used to draw the
  * visible border. You can set the appearance of the border using CSS properties
  * like “border-style” on this node.
+ *
+ * # Accessibility
+ *
+ * `GtkFrame` uses the `GTK_ACCESSIBLE_ROLE_GROUP` role.
  */
 
 typedef struct
@@ -160,7 +165,7 @@ gtk_frame_class_init (GtkFrameClass *class)
   class->compute_child_allocation = gtk_frame_real_compute_child_allocation;
 
   /**
-   * GtkFrame:label: (attributes org.gtk.Property.get=gtk_frame_get_label org.gtk.Property.set=gtk_frame_set_label)
+   * GtkFrame:label:
    *
    * Text of the frame's label.
    */
@@ -170,7 +175,7 @@ gtk_frame_class_init (GtkFrameClass *class)
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFrame:label-xalign: (attributes org.gtk.Property.get=gtk_frame_get_label_align org.gtk.Property.set=gtk_frame_set_label_align)
+   * GtkFrame:label-xalign: (getter get_label_align) (setter set_label_align)
    *
    * The horizontal alignment of the label.
    */
@@ -181,28 +186,29 @@ gtk_frame_class_init (GtkFrameClass *class)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFrame:label-widget: (attributes org.gtk.Property.get=gtk_frame_get_label_widget org.gtk.Property.set=gtk_frame_set_label_widget)
+   * GtkFrame:label-widget:
    *
    * Widget to display in place of the usual frame label.
    */
   frame_props[PROP_LABEL_WIDGET] =
       g_param_spec_object ("label-widget", NULL, NULL,
                            GTK_TYPE_WIDGET,
-                           GTK_PARAM_READWRITE);
+                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFrame:child: (attributes org.gtk.Property.get=gtk_frame_get_child org.gtk.Property.set=gtk_frame_set_child)
+   * GtkFrame:child:
    *
    * The child widget.
    */
   frame_props[PROP_CHILD] =
       g_param_spec_object ("child", NULL, NULL,
                            GTK_TYPE_WIDGET,
-                           GTK_PARAM_READWRITE);
+                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (gobject_class, LAST_PROP, frame_props);
 
   gtk_widget_class_set_css_name (widget_class, I_("frame"));
+  gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_GROUP);
 }
 
 static GtkBuildableIface *parent_buildable_iface;
@@ -222,11 +228,19 @@ gtk_frame_buildable_add_child (GtkBuildable *buildable,
                                const char   *type)
 {
   if (type && strcmp (type, "label") == 0)
-    gtk_frame_set_label_widget (GTK_FRAME (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, "label", "label-widget");
+      gtk_frame_set_label_widget (GTK_FRAME (buildable), GTK_WIDGET (child));
+    }
   else if (GTK_IS_WIDGET (child))
-    gtk_frame_set_child (GTK_FRAME (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, NULL, "child");
+      gtk_frame_set_child (GTK_FRAME (buildable), GTK_WIDGET (child));
+    }
   else
-    parent_buildable_iface->add_child (buildable, builder, child, type);
+    {
+      parent_buildable_iface->add_child (buildable, builder, child, type);
+    }
 }
 
 static void
@@ -329,7 +343,7 @@ gtk_frame_new (const char *label)
 }
 
 /**
- * gtk_frame_set_label: (attributes org.gtk.Method.set_property=label)
+ * gtk_frame_set_label:
  * @frame: a `GtkFrame`
  * @label: (nullable): the text to use as the label of the frame
  *
@@ -349,7 +363,7 @@ gtk_frame_set_label (GtkFrame *frame,
 }
 
 /**
- * gtk_frame_get_label: (attributes org.gtk.Method.get_property=label)
+ * gtk_frame_get_label:
  * @frame: a `GtkFrame`
  *
  * Returns the frame labels text.
@@ -392,7 +406,7 @@ update_accessible_relation (GtkFrame *frame)
 }
 
 /**
- * gtk_frame_set_label_widget: (attributes org.gtk.Method.get_property=label-widget)
+ * gtk_frame_set_label_widget:
  * @frame: a `GtkFrame`
  * @label_widget: (nullable): the new label widget
  *
@@ -408,8 +422,7 @@ gtk_frame_set_label_widget (GtkFrame  *frame,
   GtkFramePrivate *priv = gtk_frame_get_instance_private (frame);
 
   g_return_if_fail (GTK_IS_FRAME (frame));
-  g_return_if_fail (label_widget == NULL || GTK_IS_WIDGET (label_widget));
-  g_return_if_fail (label_widget == NULL || gtk_widget_get_parent (label_widget) == NULL);
+  g_return_if_fail (label_widget == NULL || priv->label_widget == label_widget || gtk_widget_get_parent (label_widget) == NULL);
 
   if (priv->label_widget == label_widget)
     return;
@@ -434,7 +447,7 @@ gtk_frame_set_label_widget (GtkFrame  *frame,
 }
 
 /**
- * gtk_frame_get_label_widget: (attributes org.gtk.Method.get_property=label-widget)
+ * gtk_frame_get_label_widget:
  * @frame: a `GtkFrame`
  *
  * Retrieves the label widget for the frame.
@@ -452,7 +465,7 @@ gtk_frame_get_label_widget (GtkFrame *frame)
 }
 
 /**
- * gtk_frame_set_label_align: (attributes org.gtk.Method.set_property=label-xalign)
+ * gtk_frame_set_label_align: (set-property label-xalign)
  * @frame: a `GtkFrame`
  * @xalign: The position of the label along the top edge
  *   of the widget. A value of 0.0 represents left alignment;
@@ -480,7 +493,7 @@ gtk_frame_set_label_align (GtkFrame *frame,
 }
 
 /**
- * gtk_frame_get_label_align: (attributes org.gtk.Method.get_property=label-xalign)
+ * gtk_frame_get_label_align: (get-property label-xalign)
  * @frame: a `GtkFrame`
  *
  * Retrieves the X alignment of the frame’s label.
@@ -656,7 +669,7 @@ gtk_frame_get_request_mode (GtkWidget *widget)
 }
 
 /**
- * gtk_frame_set_child: (attributes org.gtk.Method.set_property=child)
+ * gtk_frame_set_child:
  * @frame: a `GtkFrame`
  * @child: (nullable): the child widget
  *
@@ -669,7 +682,7 @@ gtk_frame_set_child (GtkFrame  *frame,
   GtkFramePrivate *priv = gtk_frame_get_instance_private (frame);
 
   g_return_if_fail (GTK_IS_FRAME (frame));
-  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+  g_return_if_fail (child == NULL || priv->child == child || gtk_widget_get_parent (child) == NULL);
 
   if (priv->child == child)
     return;
@@ -688,7 +701,7 @@ gtk_frame_set_child (GtkFrame  *frame,
 }
 
 /**
- * gtk_frame_get_child: (attributes org.gtk.Method.get_property=child)
+ * gtk_frame_get_child:
  * @frame: a `GtkFrame`
  *
  * Gets the child widget of @frame.
