@@ -24,7 +24,7 @@
 #include "gtkglarea.h"
 #include <glib/gi18n-lib.h>
 #include "gtkmarshalers.h"
-#include "gtkmarshalers.h"
+#include "gdk/gdkmarshalers.h"
 #include "gtkprivate.h"
 #include "gtksnapshot.h"
 #include "gtknative.h"
@@ -32,6 +32,14 @@
 #include "gtksnapshot.h"
 #include "gtkrenderlayoutprivate.h"
 #include "gtkcssnodeprivate.h"
+#include "gdk/gdkgltextureprivate.h"
+#include "gdk/gdkglcontextprivate.h"
+#include "gdk/gdkdmabuftexturebuilderprivate.h"
+#include "gdk/gdkdmabuftextureprivate.h"
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #include <epoxy/gl.h>
 
@@ -45,6 +53,8 @@
  * `GtkGLArea` sets up its own [class@Gdk.GLContext], and creates a custom
  * GL framebuffer that the widget will do GL rendering onto. It also ensures
  * that this framebuffer is the default GL rendering target when rendering.
+ * The completed rendering is integrated into the larger GTK scene graph as
+ * a texture.
  *
  * In order to draw, you have to connect to the [signal@Gtk.GLArea::render]
  * signal, or subclass `GtkGLArea` and override the GtkGLAreaClass.render
@@ -62,6 +72,8 @@
  * The `render()` function will be called when the `GtkGLArea` is ready
  * for you to draw its content:
  *
+ * The initial contents of the framebuffer are transparent.
+ *
  * ```c
  * static gboolean
  * render (GtkGLArea *area, GdkGLContext *context)
@@ -143,10 +155,9 @@
  */
 
 typedef struct {
-  guint id;
-  int width;
-  int height;
-  GdkTexture *holder;
+  GdkGLTextureBuilder *builder;
+  GdkTexture *gl_texture;
+  GdkTexture *dmabuf_texture;
 } Texture;
 
 typedef struct {
@@ -169,6 +180,7 @@ typedef struct {
   gboolean needs_render;
   gboolean auto_render;
   gboolean use_es;
+  GdkGLAPI allowed_apis;
 } GtkGLAreaPrivate;
 
 enum {
@@ -178,6 +190,8 @@ enum {
   PROP_HAS_DEPTH_BUFFER,
   PROP_HAS_STENCIL_BUFFER,
   PROP_USE_ES,
+  PROP_ALLOWED_APIS,
+  PROP_API,
 
   PROP_AUTO_RENDER,
 
@@ -224,7 +238,13 @@ gtk_gl_area_set_property (GObject      *gobject,
       break;
 
     case PROP_USE_ES:
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       gtk_gl_area_set_use_es (self, g_value_get_boolean (value));
+G_GNUC_END_IGNORE_DEPRECATIONS
+      break;
+
+    case PROP_ALLOWED_APIS:
+      gtk_gl_area_set_allowed_apis (self, g_value_get_flags (value));
       break;
 
     default:
@@ -259,7 +279,17 @@ gtk_gl_area_get_property (GObject    *gobject,
       break;
 
     case PROP_USE_ES:
-      g_value_set_boolean (value, priv->use_es);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+      g_value_set_boolean (value, gtk_gl_area_get_use_es (GTK_GL_AREA (gobject)));
+G_GNUC_END_IGNORE_DEPRECATIONS
+      break;
+
+    case PROP_ALLOWED_APIS:
+      g_value_set_flags (value, priv->allowed_apis);
+      break;
+
+    case PROP_API:
+      g_value_set_flags (value, gtk_gl_area_get_api (GTK_GL_AREA (gobject)));
       break;
 
     default:
@@ -321,7 +351,7 @@ gtk_gl_area_real_create_context (GtkGLArea *area)
       return NULL;
     }
 
-  gdk_gl_context_set_allowed_apis (context, priv->use_es ? GDK_GL_API_GLES : GDK_GL_API_GL);
+  gdk_gl_context_set_allowed_apis (context, priv->allowed_apis);
   gdk_gl_context_set_required_version (context,
                                        priv->required_gl_version / 10,
                                        priv->required_gl_version % 10);
@@ -384,17 +414,22 @@ static void
 delete_one_texture (gpointer data)
 {
   Texture *texture = data;
+  guint id;
 
-  if (texture->holder)
-    gdk_gl_texture_release (GDK_GL_TEXTURE (texture->holder));
-
-  if (texture->id != 0)
+  if (texture->gl_texture)
     {
-      glDeleteTextures (1, &texture->id);
-      texture->id = 0;
+      gdk_gl_texture_release (GDK_GL_TEXTURE (texture->gl_texture));
+      texture->gl_texture = NULL;
     }
 
-  g_free (texture);
+  id = gdk_gl_texture_builder_get_id (texture->builder);
+  if (id != 0)
+    glDeleteTextures (1, &id);
+
+  g_clear_object (&texture->builder);
+
+  if (texture->gl_texture == NULL && texture->dmabuf_texture == NULL)
+    g_free (texture);
 }
 
 static void
@@ -419,7 +454,7 @@ gtk_gl_area_ensure_texture (GtkGLArea *area)
           link = l;
           l = l->next;
 
-          if (texture->holder)
+          if (texture->gl_texture)
             continue;
 
           priv->textures = g_list_delete_link (priv->textures, link);
@@ -433,13 +468,21 @@ gtk_gl_area_ensure_texture (GtkGLArea *area)
 
   if (priv->texture == NULL)
     {
+      GLuint id;
+
       priv->texture = g_new (Texture, 1);
+      priv->texture->gl_texture = NULL;
+      priv->texture->dmabuf_texture = NULL;
 
-      priv->texture->width = 0;
-      priv->texture->height = 0;
-      priv->texture->holder = NULL;
+      priv->texture->builder = gdk_gl_texture_builder_new ();
+      gdk_gl_texture_builder_set_context (priv->texture->builder, priv->context);
+      if (gdk_gl_context_get_api (priv->context) == GDK_GL_API_GLES)
+        gdk_gl_texture_builder_set_format (priv->texture->builder, GDK_MEMORY_R8G8B8A8_PREMULTIPLIED);
+      else
+        gdk_gl_texture_builder_set_format (priv->texture->builder, GDK_MEMORY_B8G8R8A8_PREMULTIPLIED);
 
-      glGenTextures (1, &priv->texture->id);
+      glGenTextures (1, &id);
+      gdk_gl_texture_builder_set_id (priv->texture->builder, id);
     }
 
   gtk_gl_area_allocate_texture (area);
@@ -487,28 +530,28 @@ gtk_gl_area_allocate_texture (GtkGLArea *area)
   if (priv->texture == NULL)
     return;
 
-  g_assert (priv->texture->holder == NULL);
+  g_assert (priv->texture->gl_texture == NULL);
 
   scale = gtk_widget_get_scale_factor (widget);
   width = gtk_widget_get_width (widget) * scale;
   height = gtk_widget_get_height (widget) * scale;
 
-  if (priv->texture->width != width ||
-      priv->texture->height != height)
+  if (gdk_gl_texture_builder_get_width (priv->texture->builder) != width ||
+      gdk_gl_texture_builder_get_height (priv->texture->builder) != height)
     {
-      glBindTexture (GL_TEXTURE_2D, priv->texture->id);
+      glBindTexture (GL_TEXTURE_2D, gdk_gl_texture_builder_get_id (priv->texture->builder));
       glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
       glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
       glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
       glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 
-      if (gdk_gl_context_get_use_es (priv->context))
+      if (gdk_gl_context_get_api (priv->context) == GDK_GL_API_GLES)
         glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
       else
         glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_BGRA, GL_UNSIGNED_BYTE, NULL);
 
-      priv->texture->width = width;
-      priv->texture->height = height;
+      gdk_gl_texture_builder_set_width (priv->texture->builder, width);
+      gdk_gl_texture_builder_set_height (priv->texture->builder, height);
     }
 }
 
@@ -552,7 +595,7 @@ gtk_gl_area_attach_buffers (GtkGLArea *area)
 
   if (priv->texture != NULL)
     glFramebufferTexture2D (GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
-                            GL_TEXTURE_2D, priv->texture->id, 0);
+                            GL_TEXTURE_2D, gdk_gl_texture_builder_get_id (priv->texture->builder), 0);
 
   if (priv->depth_stencil_buffer)
     {
@@ -669,10 +712,34 @@ gtk_gl_area_draw_error_screen (GtkGLArea   *area,
 }
 
 static void
-release_texture (gpointer data)
+release_gl_texture (gpointer data)
 {
   Texture *texture = data;
-  texture->holder = NULL;
+  gpointer sync;
+
+  sync = gdk_gl_texture_builder_get_sync (texture->builder);
+  if (sync)
+    {
+      glDeleteSync (sync);
+      gdk_gl_texture_builder_set_sync (texture->builder, NULL);
+    }
+
+  texture->gl_texture = NULL;
+}
+
+static void
+release_dmabuf_texture (gpointer data)
+{
+  Texture *texture = data;
+
+  g_clear_object (&texture->gl_texture);
+
+  if (texture->dmabuf_texture == NULL)
+    return;
+
+  gdk_dmabuf_close_fds ((GdkDmabuf *) gdk_dmabuf_texture_get_dmabuf (GDK_DMABUF_TEXTURE (texture->dmabuf_texture)));
+
+  texture->dmabuf_texture = NULL;
 }
 
 static void
@@ -717,6 +784,9 @@ gtk_gl_area_snapshot (GtkWidget   *widget,
   if (status == GL_FRAMEBUFFER_COMPLETE)
     {
       Texture *texture;
+      gpointer sync = NULL;
+      GdkDmabuf dmabuf;
+      GdkTexture *holder;
 
       if (priv->needs_render || priv->auto_render)
         {
@@ -735,11 +805,35 @@ gtk_gl_area_snapshot (GtkWidget   *widget,
       priv->texture = NULL;
       priv->textures = g_list_prepend (priv->textures, texture);
 
-      texture->holder = gdk_gl_texture_new (priv->context,
-                                            texture->id,
-                                            texture->width,
-                                            texture->height,
-                                            release_texture, texture);
+      sync = glFenceSync (GL_SYNC_GPU_COMMANDS_COMPLETE, 0);
+      gdk_gl_texture_builder_set_sync (texture->builder, sync);
+
+      texture->gl_texture = gdk_gl_texture_builder_build (texture->builder,
+                                                          release_gl_texture,
+                                                          texture);
+      holder = texture->gl_texture;
+
+      if (gdk_gl_context_export_dmabuf (priv->context,
+                                        gdk_gl_texture_builder_get_id (texture->builder),
+                                        &dmabuf))
+        {
+          GdkDmabufTextureBuilder *builder = gdk_dmabuf_texture_builder_new ();
+
+          gdk_dmabuf_texture_builder_set_display (builder, gdk_gl_context_get_display (priv->context));
+          gdk_dmabuf_texture_builder_set_width (builder, gdk_texture_get_width (texture->gl_texture));
+          gdk_dmabuf_texture_builder_set_height (builder, gdk_texture_get_height (texture->gl_texture));
+          gdk_dmabuf_texture_builder_set_premultiplied (builder, TRUE);
+          gdk_dmabuf_texture_builder_set_dmabuf (builder, &dmabuf);
+
+          texture->dmabuf_texture = gdk_dmabuf_texture_builder_build (builder, release_dmabuf_texture, texture, NULL);
+
+          g_object_unref (builder);
+
+          if (texture->dmabuf_texture != NULL)
+            holder = texture->dmabuf_texture;
+          else
+            gdk_dmabuf_close_fds (&dmabuf);
+        }
 
       /* Our texture is rendered by OpenGL, so it is upside down,
        * compared to what GSK expects, so flip it back.
@@ -748,13 +842,13 @@ gtk_gl_area_snapshot (GtkWidget   *widget,
       gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT (0, gtk_widget_get_height (widget)));
       gtk_snapshot_scale (snapshot, 1, -1);
       gtk_snapshot_append_texture (snapshot,
-                                   texture->holder,
+                                   holder,
                                    &GRAPHENE_RECT_INIT (0, 0,
                                                         gtk_widget_get_width (widget),
                                                         gtk_widget_get_height (widget)));
       gtk_snapshot_restore (snapshot);
 
-      g_object_unref (texture->holder);
+      g_object_unref (holder);
     }
   else
     {
@@ -804,7 +898,7 @@ gtk_gl_area_class_init (GtkGLAreaClass *klass)
                          G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkGLArea:auto-render: (attributes org.gtk.Property.get=gtk_gl_area_get_auto_render org.gtk.Property.set=gtk_gl_area_set_auto_render)
+   * GtkGLArea:auto-render:
    *
    * If set to %TRUE the ::render signal will be emitted every time
    * the widget draws.
@@ -825,7 +919,7 @@ gtk_gl_area_class_init (GtkGLAreaClass *klass)
                           G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkGLArea:has-depth-buffer: (attributes org.gtk.Property.get=gtk_gl_area_get_has_depth_buffer org.gtk.Property.set=gtk_gl_area_set_has_depth_buffer)
+   * GtkGLArea:has-depth-buffer:
    *
    * If set to %TRUE the widget will allocate and enable a depth buffer for the
    * target framebuffer.
@@ -842,7 +936,7 @@ gtk_gl_area_class_init (GtkGLAreaClass *klass)
                           G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkGLArea:has-stencil-buffer: (attributes org.gtk.Property.get=gtk_gl_area_get_has_stencil_buffer org.gtk.Property.set=gtk_gl_area_set_has_stencil_buffer)
+   * GtkGLArea:has-stencil-buffer:
    *
    * If set to %TRUE the widget will allocate and enable a stencil buffer for the
    * target framebuffer.
@@ -855,10 +949,12 @@ gtk_gl_area_class_init (GtkGLAreaClass *klass)
                           G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkGLArea:use-es: (attributes org.gtk.Property.get=gtk_gl_area_get_use_es org.gtk.Property.set=gtk_gl_area_set_use_es)
+   * GtkGLArea:use-es:
    *
    * If set to %TRUE the widget will try to create a `GdkGLContext` using
    * OpenGL ES instead of OpenGL.
+   *
+   * Deprecated: 4.12: Use [property@Gtk.GLArea:allowed-apis]
    */
   obj_props[PROP_USE_ES] =
     g_param_spec_boolean ("use-es", NULL, NULL,
@@ -867,6 +963,36 @@ gtk_gl_area_class_init (GtkGLAreaClass *klass)
                           G_PARAM_STATIC_STRINGS |
                           G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * GtkGLArea:allowed-apis:
+   *
+   * The allowed APIs.
+   *
+   * Since: 4.12
+   */
+  obj_props[PROP_ALLOWED_APIS] =
+    g_param_spec_flags ("allowed-apis", NULL, NULL,
+                        GDK_TYPE_GL_API,
+                        GDK_GL_API_GL | GDK_GL_API_GLES,
+                        G_PARAM_READWRITE |
+                        G_PARAM_STATIC_STRINGS |
+                        G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * GtkGLArea:api:
+   *
+   * The API currently in use.
+   *
+   * Since: 4.12
+   */
+  obj_props[PROP_API] =
+    g_param_spec_flags ("api", NULL, NULL,
+                        GDK_TYPE_GL_API,
+                        0,
+                        G_PARAM_READABLE |
+                        G_PARAM_STATIC_STRINGS |
+                        G_PARAM_EXPLICIT_NOTIFY);
+
   gobject_class->set_property = gtk_gl_area_set_property;
   gobject_class->get_property = gtk_gl_area_get_property;
   gobject_class->notify = gtk_gl_area_notify;
@@ -892,12 +1018,12 @@ gtk_gl_area_class_init (GtkGLAreaClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkGLAreaClass, render),
                   _gtk_boolean_handled_accumulator, NULL,
-                  _gtk_marshal_BOOLEAN__OBJECT,
+                  _gdk_marshal_BOOLEAN__OBJECT,
                   G_TYPE_BOOLEAN, 1,
                   GDK_TYPE_GL_CONTEXT);
   g_signal_set_va_marshaller (area_signals[RENDER],
                               G_TYPE_FROM_CLASS (klass),
-                              _gtk_marshal_BOOLEAN__OBJECTv);
+                              _gdk_marshal_BOOLEAN__OBJECTv);
 
   /**
    * GtkGLArea::resize:
@@ -923,11 +1049,11 @@ gtk_gl_area_class_init (GtkGLAreaClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkGLAreaClass, resize),
                   NULL, NULL,
-                  _gtk_marshal_VOID__INT_INT,
+                  _gdk_marshal_VOID__INT_INT,
                   G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
   g_signal_set_va_marshaller (area_signals[RESIZE],
                               G_TYPE_FROM_CLASS (klass),
-                              _gtk_marshal_VOID__INT_INTv);
+                              _gdk_marshal_VOID__INT_INTv);
 
   /**
    * GtkGLArea::create-context:
@@ -968,6 +1094,7 @@ gtk_gl_area_init (GtkGLArea *area)
   priv->auto_render = TRUE;
   priv->needs_render = TRUE;
   priv->required_gl_version = 0;
+  priv->allowed_apis = GDK_GL_API_GL | GDK_GL_API_GLES;
 }
 
 /**
@@ -1026,7 +1153,7 @@ gtk_gl_area_get_error (GtkGLArea *area)
 }
 
 /**
- * gtk_gl_area_set_use_es: (attributes org.gtk.Method.set_property=use-es)
+ * gtk_gl_area_set_use_es:
  * @area: a `GtkGLArea`
  * @use_es: whether to use OpenGL or OpenGL ES
  *
@@ -1034,6 +1161,8 @@ gtk_gl_area_get_error (GtkGLArea *area)
  *
  * You should check the capabilities of the `GdkGLContext` before drawing
  * with either API.
+ *
+ * Deprecated: 4.12: Use [method@Gtk.GLArea.set_allowed_apis]
  */
 void
 gtk_gl_area_set_use_es (GtkGLArea *area,
@@ -1044,18 +1173,17 @@ gtk_gl_area_set_use_es (GtkGLArea *area,
   g_return_if_fail (GTK_IS_GL_AREA (area));
   g_return_if_fail (!gtk_widget_get_realized (GTK_WIDGET (area)));
 
-  use_es = !!use_es;
+  if ((priv->allowed_apis == GDK_GL_API_GLES) == use_es)
+    return;
 
-  if (priv->use_es != use_es)
-    {
-      priv->use_es = use_es;
+  priv->allowed_apis = use_es ? GDK_GL_API_GLES : GDK_GL_API_GL;
 
-      g_object_notify_by_pspec (G_OBJECT (area), obj_props[PROP_USE_ES]);
-    }
+  g_object_notify_by_pspec (G_OBJECT (area), obj_props[PROP_USE_ES]);
+  g_object_notify_by_pspec (G_OBJECT (area), obj_props[PROP_ALLOWED_APIS]);
 }
 
 /**
- * gtk_gl_area_get_use_es: (attributes org.gtk.Method.get_property=use-es)
+ * gtk_gl_area_get_use_es:
  * @area: a `GtkGLArea`
  *
  * Returns whether the `GtkGLArea` should use OpenGL ES.
@@ -1064,6 +1192,8 @@ gtk_gl_area_set_use_es (GtkGLArea *area,
  *
  * Returns: %TRUE if the `GtkGLArea` should create an OpenGL ES context
  *   and %FALSE otherwise
+ *
+ * Deprecated: 4.12: Use [method@Gtk.GLArea.get_api]
  */
 gboolean
 gtk_gl_area_get_use_es (GtkGLArea *area)
@@ -1072,7 +1202,93 @@ gtk_gl_area_get_use_es (GtkGLArea *area)
 
   g_return_val_if_fail (GTK_IS_GL_AREA (area), FALSE);
 
-  return priv->use_es;
+  if (priv->context)
+    return gdk_gl_context_get_api (priv->context) == GDK_GL_API_GLES;
+  else
+    return priv->allowed_apis == GDK_GL_API_GLES;
+}
+
+/**
+ * gtk_gl_area_set_allowed_apis:
+ * @area: a `GtkGLArea`
+ * @apis: the allowed APIs
+ *
+ * Sets the allowed APIs to create a context with.
+ *
+ * You should check [property@Gtk.GLArea:api] before drawing
+ * with either API.
+ *
+ * By default, all APIs are allowed.
+ *
+ * Since: 4.12
+ */
+void
+gtk_gl_area_set_allowed_apis (GtkGLArea *area,
+                              GdkGLAPI   apis)
+{
+  GtkGLAreaPrivate *priv = gtk_gl_area_get_instance_private (area);
+  GdkGLAPI old_allowed_apis;
+
+  g_return_if_fail (GTK_IS_GL_AREA (area));
+  g_return_if_fail (!gtk_widget_get_realized (GTK_WIDGET (area)));
+
+  if (priv->allowed_apis == apis)
+    return;
+
+  old_allowed_apis = priv->allowed_apis;
+
+  priv->allowed_apis = apis;
+
+  if ((old_allowed_apis == GDK_GL_API_GLES) != (apis == GDK_GL_API_GLES))
+    g_object_notify_by_pspec (G_OBJECT (area), obj_props[PROP_USE_ES]);
+  g_object_notify_by_pspec (G_OBJECT (area), obj_props[PROP_ALLOWED_APIS]);
+}
+
+/**
+ * gtk_gl_area_get_allowed_apis:
+ * @area: a `GtkGLArea`
+ *
+ * Gets the allowed APIs.
+ *
+ * See [method@Gtk.GLArea.set_allowed_apis].
+ *
+ * Returns: the allowed APIs
+ *
+ * Since: 4.12
+ */
+GdkGLAPI
+gtk_gl_area_get_allowed_apis (GtkGLArea *area)
+{
+  GtkGLAreaPrivate *priv = gtk_gl_area_get_instance_private (area);
+
+  g_return_val_if_fail (GTK_IS_GL_AREA (area), 0);
+
+  return priv->allowed_apis;
+}
+
+/**
+ * gtk_gl_area_get_api:
+ * @area: a `GtkGLArea`
+ *
+ * Gets the API that is currently in use.
+ *
+ * If the GL area has not been realized yet, 0 is returned.
+ *
+ * Returns: the currently used API
+ *
+ * Since: 4.12
+ */
+GdkGLAPI
+gtk_gl_area_get_api (GtkGLArea *area)
+{
+  GtkGLAreaPrivate *priv = gtk_gl_area_get_instance_private (area);
+
+  g_return_val_if_fail (GTK_IS_GL_AREA (area), 0);
+
+  if (priv->context)
+    return gdk_gl_context_get_api (priv->context);
+  else
+    return 0;
 }
 
 /**
@@ -1125,7 +1341,7 @@ gtk_gl_area_get_required_version (GtkGLArea *area,
 }
 
 /**
- * gtk_gl_area_get_has_depth_buffer: (attributes org.gtk.Method.get_property=has-depth-buffer)
+ * gtk_gl_area_get_has_depth_buffer:
  * @area: a `GtkGLArea`
  *
  * Returns whether the area has a depth buffer.
@@ -1143,7 +1359,7 @@ gtk_gl_area_get_has_depth_buffer (GtkGLArea *area)
 }
 
 /**
- * gtk_gl_area_set_has_depth_buffer: (attributes org.gtk.Method.set_property=has-depth-buffer)
+ * gtk_gl_area_set_has_depth_buffer:
  * @area: a `GtkGLArea`
  * @has_depth_buffer: %TRUE to add a depth buffer
  *
@@ -1174,7 +1390,7 @@ gtk_gl_area_set_has_depth_buffer (GtkGLArea *area,
 }
 
 /**
- * gtk_gl_area_get_has_stencil_buffer: (attributes org.gtk.Method.get_property=has-stencil-buffer)
+ * gtk_gl_area_get_has_stencil_buffer:
  * @area: a `GtkGLArea`
  *
  * Returns whether the area has a stencil buffer.
@@ -1192,7 +1408,7 @@ gtk_gl_area_get_has_stencil_buffer (GtkGLArea *area)
 }
 
 /**
- * gtk_gl_area_set_has_stencil_buffer: (attributes org.gtk.Method.set_property=has-stencil-buffer)
+ * gtk_gl_area_set_has_stencil_buffer:
  * @area: a `GtkGLArea`
  * @has_stencil_buffer: %TRUE to add a stencil buffer
  *
@@ -1250,7 +1466,7 @@ gtk_gl_area_queue_render (GtkGLArea *area)
 
 
 /**
- * gtk_gl_area_get_auto_render: (attributes org.gtk.Method.get_property=auto-render)
+ * gtk_gl_area_get_auto_render:
  * @area: a `GtkGLArea`
  *
  * Returns whether the area is in auto render mode or not.
@@ -1268,7 +1484,7 @@ gtk_gl_area_get_auto_render (GtkGLArea *area)
 }
 
 /**
- * gtk_gl_area_set_auto_render: (attributes org.gtk.Method.set_property=auto-render)
+ * gtk_gl_area_set_auto_render:
  * @area: a `GtkGLArea`
  * @auto_render: a boolean
  *
