@@ -31,13 +31,14 @@
 #include "gtkwidgetprivate.h"
 #include "gtkwindowcontrols.h"
 #include "gtkwindowhandle.h"
+#include "gtkbuilderprivate.h"
 
 #include <string.h>
 
 /**
  * GtkHeaderBar:
  *
- * `GtkHeaderBar` is a widget for creating custom title bars for windows.
+ * A widget for creating custom title bars for windows.
  *
  * ![An example GtkHeaderBar](headerbar.png)
  *
@@ -107,7 +108,7 @@
  *
  * # Accessibility
  *
- * `GtkHeaderBar` uses the %GTK_ACCESSIBLE_ROLE_GROUP role.
+ * `GtkHeaderBar` uses the [enum@Gtk.AccessibleRole.group] role.
  */
 
 #define MIN_TITLE_CHARS 5
@@ -282,19 +283,19 @@ construct_title_label (GtkHeaderBar *bar)
 
 /**
  * gtk_header_bar_set_title_widget:
- * @bar: a `GtkHeaderBar`
+ * @bar: a header bar
  * @title_widget: (nullable): a widget to use for a title
  *
- * Sets the title for the `GtkHeaderBar`.
+ * Sets the title for the header bar.
  *
- * When set to %NULL, the headerbar will display the title of
+ * When set to `NULL`, the headerbar will display the title of
  * the window it is contained in.
  *
  * The title should help a user identify the current view.
  * To achieve the same style as the builtin title, use the
  * “title” style class.
  *
- * You should set the title widget to %NULL, for the window
+ * You should set the title widget to `NULL`, for the window
  * title label to be visible again.
  */
 void
@@ -330,13 +331,13 @@ gtk_header_bar_set_title_widget (GtkHeaderBar *bar,
 
 /**
  * gtk_header_bar_get_title_widget:
- * @bar: a `GtkHeaderBar`
+ * @bar: a header bar
  *
- * Retrieves the title widget of the header.
+ * Retrieves the title widget of the header bar.
  *
  * See [method@Gtk.HeaderBar.set_title_widget].
  *
- * Returns: (nullable) (transfer none): the title widget of the header
+ * Returns: (nullable) (transfer none): the title widget
  */
 GtkWidget *
 gtk_header_bar_get_title_widget (GtkHeaderBar *bar)
@@ -474,10 +475,10 @@ gtk_header_bar_pack (GtkHeaderBar *bar,
 
 /**
  * gtk_header_bar_remove:
- * @bar: a `GtkHeaderBar`
+ * @bar: a header bar
  * @child: the child to remove
  *
- * Removes a child from the `GtkHeaderBar`.
+ * Removes a child from the header bar.
  *
  * The child must have been added with
  * [method@Gtk.HeaderBar.pack_start],
@@ -562,13 +563,18 @@ gtk_header_bar_class_init (GtkHeaderBarClass *class)
   widget_class->unroot = gtk_header_bar_unroot;
   widget_class->get_request_mode = gtk_header_bar_get_request_mode;
 
+  /**
+   * GtkHeaderBar:title-widget:
+   *
+   * The title widget to display.
+   */
   header_bar_props[PROP_TITLE_WIDGET] =
       g_param_spec_object ("title-widget", NULL, NULL,
                            GTK_TYPE_WIDGET,
                            G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkHeaderBar:show-title-buttons: (attributes org.gtk.Property.get=gtk_header_bar_get_show_title_buttons org.gtk.Property.set=gtk_header_bar_set_show_title_buttons)
+   * GtkHeaderBar:show-title-buttons:
    *
    * Whether to show title buttons like close, minimize, maximize.
    *
@@ -583,7 +589,7 @@ gtk_header_bar_class_init (GtkHeaderBarClass *class)
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkHeaderBar:decoration-layout: (attributes org.gtk.Property.get=gtk_header_bar_get_decoration_layout org.gtk.Property.set=gtk_header_bar_set_decoration_layout)
+   * GtkHeaderBar:decoration-layout:
    *
    * The decoration layout for buttons.
    *
@@ -636,15 +642,26 @@ gtk_header_bar_buildable_add_child (GtkBuildable *buildable,
                                     const char   *type)
 {
   if (g_strcmp0 (type, "title") == 0)
-    gtk_header_bar_set_title_widget (GTK_HEADER_BAR (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, "title", "title-widget");
+      gtk_header_bar_set_title_widget (GTK_HEADER_BAR (buildable), GTK_WIDGET (child));
+    }
   else if (g_strcmp0 (type, "start") == 0)
-    gtk_header_bar_pack_start (GTK_HEADER_BAR (buildable), GTK_WIDGET (child));
+    {
+      gtk_header_bar_pack_start (GTK_HEADER_BAR (buildable), GTK_WIDGET (child));
+    }
   else if (g_strcmp0 (type, "end") == 0)
-    gtk_header_bar_pack_end (GTK_HEADER_BAR (buildable), GTK_WIDGET (child));
+    {
+      gtk_header_bar_pack_end (GTK_HEADER_BAR (buildable), GTK_WIDGET (child));
+    }
   else if (type == NULL && GTK_IS_WIDGET (child))
-    gtk_header_bar_pack_start (GTK_HEADER_BAR (buildable), GTK_WIDGET (child));
+    {
+      gtk_header_bar_pack_start (GTK_HEADER_BAR (buildable), GTK_WIDGET (child));
+    }
   else
-    parent_buildable_iface->add_child (buildable, builder, child, type);
+    {
+      parent_buildable_iface->add_child (buildable, builder, child, type);
+    }
 }
 
 static void
@@ -657,11 +674,10 @@ gtk_header_bar_buildable_init (GtkBuildableIface *iface)
 
 /**
  * gtk_header_bar_pack_start:
- * @bar: A `GtkHeaderBar`
- * @child: the `GtkWidget` to be added to @bar
+ * @bar: A header bar
+ * @child: the widget to be added to @bar
  *
- * Adds @child to @bar, packed with reference to the
- * start of the @bar.
+ * Adds a child to the header bar, packed with reference to the start.
  */
 void
 gtk_header_bar_pack_start (GtkHeaderBar *bar,
@@ -672,11 +688,10 @@ gtk_header_bar_pack_start (GtkHeaderBar *bar,
 
 /**
  * gtk_header_bar_pack_end:
- * @bar: A `GtkHeaderBar`
- * @child: the `GtkWidget` to be added to @bar
+ * @bar: A header bar
+ * @child: the widget to be added to @bar
  *
- * Adds @child to @bar, packed with reference to the
- * end of the @bar.
+ * Adds a child to the header bar, packed with reference to the end.
  */
 void
 gtk_header_bar_pack_end (GtkHeaderBar *bar,
@@ -699,13 +714,13 @@ gtk_header_bar_new (void)
 }
 
 /**
- * gtk_header_bar_get_show_title_buttons: (attributes org.gtk.Method.get_property=show-title-buttons)
- * @bar: a `GtkHeaderBar`
+ * gtk_header_bar_get_show_title_buttons:
+ * @bar: a header bar
  *
  * Returns whether this header bar shows the standard window
  * title buttons.
  *
- * Returns: %TRUE if title buttons are shown
+ * Returns: true if title buttons are shown
  */
 gboolean
 gtk_header_bar_get_show_title_buttons (GtkHeaderBar *bar)
@@ -716,9 +731,9 @@ gtk_header_bar_get_show_title_buttons (GtkHeaderBar *bar)
 }
 
 /**
- * gtk_header_bar_set_show_title_buttons: (attributes org.gtk.Method.set_property=show-title-buttons)
- * @bar: a `GtkHeaderBar`
- * @setting: %TRUE to show standard title buttons
+ * gtk_header_bar_set_show_title_buttons:
+ * @bar: a header bar
+ * @setting: true to show standard title buttons
  *
  * Sets whether this header bar shows the standard window
  * title buttons.
@@ -757,9 +772,9 @@ gtk_header_bar_set_show_title_buttons (GtkHeaderBar *bar,
 }
 
 /**
- * gtk_header_bar_set_decoration_layout: (attributes org.gtk.Method.set_property=decoration-layout)
- * @bar: a `GtkHeaderBar`
- * @layout: (nullable): a decoration layout, or %NULL to unset the layout
+ * gtk_header_bar_set_decoration_layout:
+ * @bar: a header bar
+ * @layout: (nullable): a decoration layout
  *
  * Sets the decoration layout for this header bar.
  *
@@ -792,10 +807,10 @@ gtk_header_bar_set_decoration_layout (GtkHeaderBar *bar,
 }
 
 /**
- * gtk_header_bar_get_decoration_layout: (attributes org.gtk.Method.get_property=decoration-layout)
- * @bar: a `GtkHeaderBar`
+ * gtk_header_bar_get_decoration_layout:
+ * @bar: a header bar
  *
- * Gets the decoration layout of the `GtkHeaderBar`.
+ * Gets the decoration layout of the header bar.
  *
  * Returns: (nullable): the decoration layout
  */
