@@ -147,12 +147,12 @@ static const guint gtk_compose_ignore[] = {
 
 static void     gtk_im_context_simple_finalize           (GObject                  *obj);
 static gboolean gtk_im_context_simple_filter_keypress    (GtkIMContext             *context,
-							  GdkEvent                 *key);
+                                                          GdkEvent                 *key);
 static void     gtk_im_context_simple_reset              (GtkIMContext             *context);
 static void     gtk_im_context_simple_get_preedit_string (GtkIMContext             *context,
-							  char                    **str,
-							  PangoAttrList           **attrs,
-							  int                      *cursor_pos);
+                                                          char                    **str,
+                                                          PangoAttrList           **attrs,
+                                                          int                      *cursor_pos);
 
 static void init_compose_table_async (GCancellable         *cancellable,
                                       GAsyncReadyCallback   callback,
@@ -353,7 +353,7 @@ init_compose_table_thread_cb (GTask            *task,
 
   g_task_return_boolean (task, TRUE);
 
-  gdk_profiler_end_mark (before, "im compose table load (thread)", NULL);
+  gdk_profiler_end_mark (before, "Compose table load (thread)", NULL);
 }
 
 static void
@@ -479,11 +479,11 @@ check_hex (GtkIMContextSimple *context_simple,
 
       ch = gdk_keyval_to_unicode (priv->compose_buffer[i]);
 
-      if (ch == 0)
-        return FALSE;
-
-      if (!g_unichar_isxdigit (ch))
-        return FALSE;
+      if (ch == 0 || !g_unichar_isxdigit (ch))
+        {
+          g_string_free (str, TRUE);
+          return FALSE;
+        }
 
       buf[g_unichar_to_utf8 (ch, buf)] = '\0';
 
@@ -532,7 +532,7 @@ beep_surface (GdkSurface *surface)
 static inline gboolean
 is_dead_key (guint keysym)
 {
-  return GDK_KEY_dead_grave <= keysym && keysym <= GDK_KEY_dead_greek;
+  return GDK_KEY_dead_grave <= keysym && keysym <= GDK_KEY_dead_hamza;
 }
 
 static void
@@ -599,6 +599,7 @@ append_dead_key (GString *string,
     CASE (U, 0x367, 1);
     CASE (small_schwa, 0x1dea, 1);
     CASE (capital_schwa, 0x1dea, 1);
+    CASE (hamza, 0x621, 0);
 #undef CASE
     default:
       g_string_append_unichar (string, gdk_keyval_to_unicode (keysym));
@@ -655,9 +656,9 @@ no_sequence_matches (GtkIMContextSimple *context_simple,
                                                    &translated,
                                                    NULL);
 
-	  gtk_im_context_filter_keypress (context, tmp_event);
-	  gdk_event_unref (tmp_event);
-	}
+          gtk_im_context_filter_keypress (context, tmp_event);
+          gdk_event_unref (tmp_event);
+        }
 
       return gtk_im_context_filter_keypress (context, event);
     }
@@ -707,23 +708,23 @@ no_sequence_matches (GtkIMContextSimple *context_simple,
         }
 
       priv->compose_buffer[0] = 0;
-      if (n_compose > 1)		/* Invalid sequence */
-	{
-	  beep_surface (gdk_event_get_surface (event));
+      if (n_compose > 1)                /* Invalid sequence */
+        {
+          beep_surface (gdk_event_get_surface (event));
           g_signal_emit_by_name (context, "preedit-changed");
           g_signal_emit_by_name (context, "preedit-end");
-	  return TRUE;
-	}
+          return TRUE;
+        }
 
       keyval = gdk_key_event_get_keyval (event);
       ch = gdk_keyval_to_unicode (keyval);
       if (ch != 0 && !g_unichar_iscntrl (ch))
-	{
-	  gtk_im_context_simple_commit_char (context_simple, ch);
-	  return TRUE;
-	}
+        {
+          gtk_im_context_simple_commit_char (context_simple, ch);
+          return TRUE;
+        }
       else
-	return FALSE;
+        return FALSE;
     }
   return FALSE;
 }
@@ -783,7 +784,7 @@ canonical_hex_keyval (GdkEvent *event)
 
 static gboolean
 gtk_im_context_simple_filter_keypress (GtkIMContext *context,
-				       GdkEvent     *event)
+                                       GdkEvent     *event)
 {
   GtkIMContextSimple *context_simple = GTK_IM_CONTEXT_SIMPLE (context);
   GtkIMContextSimplePrivate *priv = context_simple->priv;
@@ -812,37 +813,37 @@ gtk_im_context_simple_filter_keypress (GtkIMContext *context,
     {
       if (priv->in_hex_sequence &&
           (keyval == GDK_KEY_Control_L || keyval == GDK_KEY_Control_R ||
-	   keyval == GDK_KEY_Shift_L || keyval == GDK_KEY_Shift_R))
-	{
+           keyval == GDK_KEY_Shift_L || keyval == GDK_KEY_Shift_R))
+        {
           if (priv->tentative_match->len > 0)
-	    {
+            {
               char *str = g_strdup (priv->tentative_match->str);
-	      gtk_im_context_simple_commit_string (context_simple, str);
+              gtk_im_context_simple_commit_string (context_simple, str);
               g_free (str);
 
-	      return TRUE;
-	    }
-	  else if (n_compose == 0)
-	    {
-	      priv->modifiers_dropped = TRUE;
+              return TRUE;
+            }
+          else if (n_compose == 0)
+            {
+              priv->modifiers_dropped = TRUE;
 
-	      return TRUE;
-	    }
-	  else if (priv->in_hex_sequence)
-	    {
-	      /* invalid hex sequence */
-	      beep_surface (surface);
+              return TRUE;
+            }
+          else if (priv->in_hex_sequence)
+            {
+              /* invalid hex sequence */
+              beep_surface (surface);
 
               g_string_set_size (priv->tentative_match, 0);
-	      priv->in_hex_sequence = FALSE;
-	      priv->compose_buffer[0] = 0;
+              priv->in_hex_sequence = FALSE;
+              priv->compose_buffer[0] = 0;
 
-	      g_signal_emit_by_name (context_simple, "preedit-changed");
-	      g_signal_emit_by_name (context_simple, "preedit-end");
+              g_signal_emit_by_name (context_simple, "preedit-changed");
+              g_signal_emit_by_name (context_simple, "preedit-end");
 
-	      return TRUE;
-	    }
-	}
+              return TRUE;
+            }
+        }
 
       if (priv->in_hex_sequence || priv->in_compose_sequence)
         return TRUE; /* Don't leak random key events during preedit */
@@ -909,12 +910,12 @@ gtk_im_context_simple_filter_keypress (GtkIMContext *context,
 #endif
 
       if (priv->in_hex_sequence && priv->modifiers_dropped &&
-	  (keyval == GDK_KEY_Return ||
-	   keyval == GDK_KEY_ISO_Enter ||
-	   keyval == GDK_KEY_KP_Enter))
-	{
-	  return FALSE;
-	}
+          (keyval == GDK_KEY_Return ||
+           keyval == GDK_KEY_ISO_Enter ||
+           keyval == GDK_KEY_KP_Enter))
+        {
+          return FALSE;
+        }
 
       if (state & no_text_input_mask & ~consumed_modifiers)
         {
@@ -929,15 +930,15 @@ gtk_im_context_simple_filter_keypress (GtkIMContext *context,
   if (priv->in_hex_sequence && have_hex_mods && is_backspace)
     {
       if (n_compose > 0)
-	{
-	  n_compose--;
-	  priv->compose_buffer[n_compose] = 0;
+        {
+          n_compose--;
+          priv->compose_buffer[n_compose] = 0;
           check_hex (context_simple, n_compose);
-	}
+        }
       else
-	{
-	  priv->in_hex_sequence = FALSE;
-	}
+        {
+          priv->in_hex_sequence = FALSE;
+        }
 
       g_signal_emit_by_name (context_simple, "preedit-changed");
 
@@ -964,21 +965,21 @@ gtk_im_context_simple_filter_keypress (GtkIMContext *context,
   if (priv->in_hex_sequence && have_hex_mods && is_hex_start)
     {
       if (priv->tentative_match->len > 0)
-	{
+        {
           char *str = g_strdup (priv->tentative_match->str);
-	  gtk_im_context_simple_commit_string (context_simple, str);
+          gtk_im_context_simple_commit_string (context_simple, str);
           g_free (str);
-	}
+        }
       else
-	{
-	  /* invalid hex sequence */
-	  if (n_compose > 0)
-	    beep_surface (surface);
+        {
+          /* invalid hex sequence */
+          if (n_compose > 0)
+            beep_surface (surface);
 
           g_string_set_size (priv->tentative_match, 0);
-	  priv->in_hex_sequence = FALSE;
-	  priv->compose_buffer[0] = 0;
-	}
+          priv->in_hex_sequence = FALSE;
+          priv->compose_buffer[0] = 0;
+        }
     }
 
   /* Check for hex sequence start */
@@ -999,8 +1000,8 @@ gtk_im_context_simple_filter_keypress (GtkIMContext *context,
     {
       if (priv->in_hex_sequence || priv->in_compose_sequence)
         {
-	  gtk_im_context_simple_reset (context);
-	  return TRUE;
+          gtk_im_context_simple_reset (context);
+          return TRUE;
         }
 
       return FALSE;
@@ -1009,13 +1010,13 @@ gtk_im_context_simple_filter_keypress (GtkIMContext *context,
   if (priv->in_hex_sequence)
     {
       if (hex_keyval && n_compose < 6)
-	priv->compose_buffer[n_compose++] = hex_keyval;
+        priv->compose_buffer[n_compose++] = hex_keyval;
       else if (!is_hex_end)
-	{
-	  /* non-hex character in hex sequence, or sequence too long */
-	  beep_surface (surface);
-	  return TRUE;
-	}
+        {
+          /* non-hex character in hex sequence, or sequence too long */
+          beep_surface (surface);
+          return TRUE;
+        }
     }
   else
     {
@@ -1038,33 +1039,33 @@ gtk_im_context_simple_filter_keypress (GtkIMContext *context,
           /* space or return ends the sequence, and we eat the key */
           if (n_compose > 0 && is_hex_end)
             {
-	      if (priv->tentative_match->len > 0)
-		{
+              if (priv->tentative_match->len > 0)
+                {
                   char *str = g_strdup (priv->tentative_match->str);
-		  gtk_im_context_simple_commit_string (context_simple, str);
+                  gtk_im_context_simple_commit_string (context_simple, str);
                   g_free (str);
 
                   return TRUE;
-		}
-	      else
-		{
-		  /* invalid hex sequence */
-		  beep_surface (surface);
+                }
+              else
+                {
+                  /* invalid hex sequence */
+                  beep_surface (surface);
 
                   g_string_set_size (priv->tentative_match, 0);
-		  priv->in_hex_sequence = FALSE;
-		  priv->compose_buffer[0] = 0;
-		}
+                  priv->in_hex_sequence = FALSE;
+                  priv->compose_buffer[0] = 0;
+                }
             }
           else if (!check_hex (context_simple, n_compose))
-	    beep_surface (surface);
+            beep_surface (surface);
 
-	  g_signal_emit_by_name (context_simple, "preedit-changed");
+          g_signal_emit_by_name (context_simple, "preedit-changed");
 
-	  if (!priv->in_hex_sequence)
-	    g_signal_emit_by_name (context_simple, "preedit-end");
+          if (!priv->in_hex_sequence)
+            g_signal_emit_by_name (context_simple, "preedit-end");
 
-	  return TRUE;
+          return TRUE;
         }
     }
   else /* Then, check for compose sequences */
