@@ -48,6 +48,10 @@
  * Users of this widget should take care to plan behaviour for the common case
  * where the text doesn't fit exactly in the allocated space.
  *
+ * ## CSS nodes
+ *
+ * `GtkInscription` has a single CSS node with the name label.
+ *
  * Since: 4.8
  */
 
@@ -413,7 +417,7 @@ gtk_inscription_measure (GtkWidget      *widget,
     *natural_baseline = PANGO_PIXELS_CEIL (*natural_baseline);
 }
 
-static void
+void
 gtk_inscription_get_layout_location (GtkInscription *self,
                                      float          *x_out,
                                      float          *y_out)
