@@ -122,7 +122,12 @@ gtk_layout_manager_real_get_request_mode (GtkLayoutManager *manager,
        child != NULL;
        child = _gtk_widget_get_next_sibling (child))
     {
-      GtkSizeRequestMode res = gtk_widget_get_request_mode (child);
+      GtkSizeRequestMode res;
+
+      if (!gtk_widget_should_layout (child))
+        continue;
+
+      res = gtk_widget_get_request_mode (child);
 
       switch (res)
         {
@@ -143,8 +148,8 @@ gtk_layout_manager_real_get_request_mode (GtkLayoutManager *manager,
  if (hfw == 0 && wfh == 0)
    return GTK_SIZE_REQUEST_CONSTANT_SIZE;
 
- return hfw > wfh ? GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH
-                  : GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT;
+ return hfw >= wfh ? GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH
+                   : GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT;
 }
 
 static void
