@@ -88,8 +88,10 @@
 #include "config.h"
 
 #include "gtkactionable.h"
+#include "gtkbinlayout.h"
 #include "gtkbuildable.h"
 #include "gtkbuiltiniconprivate.h"
+#include "gtkgizmoprivate.h"
 #include "gtkimage.h"
 #include "gtkmain.h"
 #include "gtkmenubutton.h"
@@ -104,6 +106,7 @@
 #include "gtkbuttonprivate.h"
 #include "gtknative.h"
 #include "gtkwindow.h"
+#include "gtkbuilderprivate.h"
 
 typedef struct _GtkMenuButtonClass   GtkMenuButtonClass;
 typedef struct _GtkMenuButtonPrivate GtkMenuButtonPrivate;
@@ -128,6 +131,7 @@ struct _GtkMenuButton
   gboolean always_show_arrow;
 
   gboolean primary;
+  gboolean can_shrink;
 };
 
 struct _GtkMenuButtonClass
@@ -150,6 +154,8 @@ enum
   PROP_HAS_FRAME,
   PROP_PRIMARY,
   PROP_CHILD,
+  PROP_ACTIVE,
+  PROP_CAN_SHRINK,
   LAST_PROP
 };
 
@@ -208,6 +214,12 @@ gtk_menu_button_set_property (GObject      *object,
       case PROP_CHILD:
         gtk_menu_button_set_child (self, g_value_get_object (value));
         break;
+      case PROP_ACTIVE:
+        gtk_menu_button_set_active (self, g_value_get_boolean (value));
+        break;
+      case PROP_CAN_SHRINK:
+        gtk_menu_button_set_can_shrink (self, g_value_get_boolean (value));
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -253,6 +265,12 @@ gtk_menu_button_get_property (GObject    *object,
       case PROP_CHILD:
         g_value_set_object (value, gtk_menu_button_get_child (self));
         break;
+      case PROP_ACTIVE:
+        g_value_set_boolean (value, gtk_menu_button_get_active (self));
+        break;
+      case PROP_CAN_SHRINK:
+        g_value_set_boolean (value, gtk_menu_button_get_can_shrink (self));
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -283,7 +301,7 @@ gtk_menu_button_state_flags_changed (GtkWidget    *widget,
   if (!gtk_widget_is_sensitive (widget))
     {
       if (self->popover)
-        gtk_widget_hide (self->popover);
+        gtk_widget_set_visible (self->popover, FALSE);
     }
 }
 
@@ -314,6 +332,8 @@ gtk_menu_button_toggled (GtkMenuButton *self)
                                       GTK_ACCESSIBLE_STATE_EXPANDED);
         }
     }
+
+  g_object_notify_by_pspec (G_OBJECT (self), menu_button_props[PROP_ACTIVE]);
 }
 
 static void
@@ -401,7 +421,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
   klass->activate = gtk_menu_button_activate;
 
   /**
-   * GtkMenuButton:menu-model: (attributes org.gtk.Property.get=gtk_menu_button_get_menu_model org.gtk.Property.set=gtk_menu_button_set_menu_model)
+   * GtkMenuButton:menu-model:
    *
    * The `GMenuModel` from which the popup will be created.
    *
@@ -414,7 +434,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                            GTK_PARAM_READWRITE);
 
   /**
-   * GtkMenuButton:direction: (attributes org.gtk.Property.get=gtk_menu_button_get_direction org.gtk.Property.set=gtk_menu_button_set_direction)
+   * GtkMenuButton:direction:
    *
    * The `GtkArrowType` representing the direction in which the
    * menu or popover will be popped out.
@@ -426,7 +446,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:popover: (attributes org.gtk.Property.get=gtk_menu_button_get_popover org.gtk.Property.set=gtk_menu_button_set_popover)
+   * GtkMenuButton:popover:
    *
    * The `GtkPopover` that will be popped up when the button is clicked.
    */
@@ -436,7 +456,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                            G_PARAM_READWRITE);
 
   /**
-   * GtkMenuButton:icon-name: (attributes org.gtk.Property.get=gtk_menu_button_get_icon_name org.gtk.Property.set=gtk_menu_button_set_icon_name)
+   * GtkMenuButton:icon-name:
    *
    * The name of the icon used to automatically populate the button.
    */
@@ -446,7 +466,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:always-show-arrow: (attributes org.gtk.Property.get=gtk_menu_button_get_always_show_arrow org.gtk.Property.set=gtk_menu_button_set_always_show_arrow)
+   * GtkMenuButton:always-show-arrow:
    *
    * Whether to show a dropdown arrow even when using an icon or a custom child.
    *
@@ -458,7 +478,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:label: (attributes org.gtk.Property.get=gtk_menu_button_get_label org.gtk.Property.set=gtk_menu_button_set_label)
+   * GtkMenuButton:label:
    *
    * The label for the button.
    */
@@ -468,7 +488,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:use-underline: (attributes org.gtk.Property.get=gtk_menu_button_get_use_underline org.gtk.Property.set=gtk_menu_button_set_use_underline)
+   * GtkMenuButton:use-underline:
    *
    * If set an underscore in the text indicates a mnemonic.
    */
@@ -478,7 +498,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:has-frame: (attributes org.gtk.Property.get=gtk_menu_button_get_has_frame org.gtk.Property.set=gtk_menu_button_set_has_frame)
+   * GtkMenuButton:has-frame:
    *
    * Whether the button has a frame.
    */
@@ -488,7 +508,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:primary: (attributes org.gtk.Property.get=gtk_menu_button_get_primary org.gtk.Property.set=gtk_menu_button_set_primary)
+   * GtkMenuButton:primary:
    *
    * Whether the menu button acts as a primary menu.
    *
@@ -502,7 +522,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:child: (attributes org.gtk.Property.get=gtk_menu_button_get_child org.gtk.Property.set=gtk_menu_button_set_child)
+   * GtkMenuButton:child:
    *
    * The child widget.
    *
@@ -513,6 +533,31 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                          GTK_TYPE_WIDGET,
                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * GtkMenuButton:active:
+   *
+   * Whether the menu button is active.
+   *
+   * Since: 4.10
+   */
+  menu_button_props[PROP_ACTIVE] =
+    g_param_spec_boolean ("active", NULL, NULL,
+                          FALSE,
+                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * GtkMenuButton:can-shrink:
+   *
+   * Whether the size of the button can be made smaller than the natural
+   * size of its contents.
+   *
+   * Since: 4.12
+   */
+  menu_button_props[PROP_CAN_SHRINK] =
+    g_param_spec_boolean ("can-shrink", NULL, NULL,
+                          FALSE,
+                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (gobject_class, LAST_PROP, menu_button_props);
 
   /**
@@ -572,10 +617,7 @@ set_arrow_type (GtkWidget    *arrow,
       break;
     }
 
-  if (visible)
-    gtk_widget_show (arrow);
-  else
-    gtk_widget_hide (arrow);
+  gtk_widget_set_visible (arrow, visible);
 }
 
 static void
@@ -648,11 +690,6 @@ gtk_menu_button_init (GtkMenuButton *self)
   gtk_widget_set_sensitive (self->button, FALSE);
 
   gtk_widget_add_css_class (GTK_WIDGET (self), "popup");
-
-  gtk_accessible_update_relation (GTK_ACCESSIBLE (self->button),
-                                  GTK_ACCESSIBLE_RELATION_LABELLED_BY, self, NULL,
-                                  GTK_ACCESSIBLE_RELATION_DESCRIBED_BY, self, NULL,
-                                  -1);
 }
 
 static GtkBuildableIface *parent_buildable_iface;
@@ -664,9 +701,14 @@ gtk_menu_button_buildable_add_child (GtkBuildable *buildable,
                                      const char   *type)
 {
   if (GTK_IS_WIDGET (child))
-    gtk_menu_button_set_child (GTK_MENU_BUTTON (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, NULL, "child");
+      gtk_menu_button_set_child (GTK_MENU_BUTTON (buildable), GTK_WIDGET (child));
+    }
   else
-    parent_buildable_iface->add_child (buildable, builder, child, type);
+    {
+      parent_buildable_iface->add_child (buildable, builder, child, type);
+    }
 }
 
 static void
@@ -703,15 +745,15 @@ update_sensitivity (GtkMenuButton *self)
 
   gtk_widget_set_sensitive (self->button, has_popup);
 
-  gtk_accessible_update_property (GTK_ACCESSIBLE (self),
+  gtk_accessible_update_property (GTK_ACCESSIBLE (self->button),
                                   GTK_ACCESSIBLE_PROPERTY_HAS_POPUP, has_popup,
                                   -1);
   if (self->popover != NULL)
-    gtk_accessible_update_relation (GTK_ACCESSIBLE (self),
+    gtk_accessible_update_relation (GTK_ACCESSIBLE (self->button),
                                     GTK_ACCESSIBLE_RELATION_CONTROLS, self->popover, NULL,
                                     -1);
   else
-    gtk_accessible_reset_relation (GTK_ACCESSIBLE (self),
+    gtk_accessible_reset_relation (GTK_ACCESSIBLE (self->button),
                                    GTK_ACCESSIBLE_RELATION_CONTROLS);
 }
 
@@ -724,7 +766,7 @@ menu_deactivate_cb (GtkMenuButton *self)
 }
 
 /**
- * gtk_menu_button_set_menu_model: (attributes org.gtk.Method.set_property=menu-model)
+ * gtk_menu_button_set_menu_model:
  * @menu_button: a `GtkMenuButton`
  * @menu_model: (nullable): a `GMenuModel`, or %NULL to unset and disable the
  *   button
@@ -757,6 +799,11 @@ gtk_menu_button_set_menu_model (GtkMenuButton *menu_button,
       GtkWidget *popover;
 
       popover = gtk_popover_menu_new_from_model (menu_model);
+
+      gtk_accessible_update_relation (GTK_ACCESSIBLE (popover),
+                                      GTK_ACCESSIBLE_RELATION_LABELLED_BY, menu_button, NULL,
+                                      -1);
+
       gtk_menu_button_set_popover (menu_button, popover);
     }
   else
@@ -771,7 +818,7 @@ gtk_menu_button_set_menu_model (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_menu_model: (attributes org.gtk.Method.get_property=menu-model)
+ * gtk_menu_button_get_menu_model:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns the `GMenuModel` used to generate the popup.
@@ -819,7 +866,7 @@ popover_destroy_cb (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_direction: (attributes org.gtk.Method.set_property=direction)
+ * gtk_menu_button_set_direction:
  * @menu_button: a `GtkMenuButton`
  * @direction: a `GtkArrowType`
  *
@@ -851,7 +898,7 @@ gtk_menu_button_set_direction (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_direction: (attributes org.gtk.Method.get_property=direction)
+ * gtk_menu_button_get_direction:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns the direction the popup will be pointing at when popped up.
@@ -893,7 +940,7 @@ gtk_menu_button_dispose (GObject *object)
 }
 
 /**
- * gtk_menu_button_set_popover: (attributes org.gtk.Method.set_property=popover)
+ * gtk_menu_button_set_popover:
  * @menu_button: a `GtkMenuButton`
  * @popover: (nullable): a `GtkPopover`, or %NULL to unset and disable the button
  *
@@ -917,8 +964,7 @@ gtk_menu_button_set_popover (GtkMenuButton *menu_button,
 
   if (menu_button->popover)
     {
-      if (gtk_widget_get_visible (menu_button->popover))
-        gtk_widget_hide (menu_button->popover);
+      gtk_widget_set_visible (menu_button->popover, FALSE);
 
       g_signal_handlers_disconnect_by_func (menu_button->popover,
                                             menu_deactivate_cb,
@@ -950,7 +996,7 @@ gtk_menu_button_set_popover (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_popover: (attributes org.gtk.Method.get_property=popover)
+ * gtk_menu_button_get_popover:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns the `GtkPopover` that pops out of the button.
@@ -969,7 +1015,7 @@ gtk_menu_button_get_popover (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
+ * gtk_menu_button_set_icon_name:
  * @menu_button: a `GtkMenuButton`
  * @icon_name: the icon name
  *
@@ -1001,15 +1047,6 @@ gtk_menu_button_set_icon_name (GtkMenuButton *menu_button,
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_set_halign (box, GTK_ALIGN_CENTER);
 
-  /* Because we are setting only an icon, let the inner button be labelled by us
-   * so the accessible label can be overridden from, for example, an UI file
-   * using GtkMenuButton as a child of something.
-   */
-  gtk_accessible_update_relation (GTK_ACCESSIBLE (menu_button->button),
-                                  GTK_ACCESSIBLE_RELATION_LABELLED_BY, menu_button, NULL,
-                                  GTK_ACCESSIBLE_RELATION_DESCRIBED_BY, menu_button, NULL,
-                                  -1);
-
   image_widget = g_object_new (GTK_TYPE_IMAGE,
                                "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION,
                                "icon-name", icon_name,
@@ -1034,7 +1071,7 @@ gtk_menu_button_set_icon_name (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
+ * gtk_menu_button_get_icon_name:
  * @menu_button: a `GtkMenuButton`
  *
  * Gets the name of the icon shown in the button.
@@ -1053,9 +1090,10 @@ gtk_menu_button_get_icon_name (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_always_show_arrow: (attributes org.gtk.Method.set_property=always-show-arrow)
+ * gtk_menu_button_set_always_show_arrow:
  * @menu_button: a `GtkMenuButton`
- * @always_show_arrow: hether to show a dropdown arrow even when using an icon
+ * @always_show_arrow: whether to show a dropdown arrow even when using an icon
+ * or a custom child
  *
  * Sets whether to show a dropdown arrow even when using an icon or a custom
  * child.
@@ -1081,12 +1119,14 @@ gtk_menu_button_set_always_show_arrow (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_always_show_arrow: (attributes org.gtk.Method.get_property=always-show-arrow)
+ * gtk_menu_button_get_always_show_arrow:
  * @menu_button: a `GtkMenuButton`
  *
- * Gets whether to show a dropdown arrow even when using an icon.
+ * Gets whether to show a dropdown arrow even when using an icon or a custom
+ * child.
  *
- * Returns: whether to show a dropdown arrow even when using an icon
+ * Returns: whether to show a dropdown arrow even when using an icon or a custom
+ * child.
  *
  * Since: 4.4
  */
@@ -1099,7 +1139,7 @@ gtk_menu_button_get_always_show_arrow (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_label: (attributes org.gtk.Method.set_property=label)
+ * gtk_menu_button_set_label:
  * @menu_button: a `GtkMenuButton`
  * @label: the label
  *
@@ -1129,12 +1169,14 @@ gtk_menu_button_set_label (GtkMenuButton *menu_button,
     g_object_notify_by_pspec (G_OBJECT (menu_button), menu_button_props[PROP_CHILD]);
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_widget_set_hexpand (box, FALSE);
   label_widget = gtk_label_new (label);
-  gtk_label_set_xalign (GTK_LABEL (label_widget), 0);
   gtk_label_set_use_underline (GTK_LABEL (label_widget),
                                gtk_button_get_use_underline (GTK_BUTTON (menu_button->button)));
+  gtk_label_set_ellipsize (GTK_LABEL (label_widget),
+                           menu_button->can_shrink ? PANGO_ELLIPSIZE_END
+                                                   : PANGO_ELLIPSIZE_NONE);
   gtk_widget_set_hexpand (label_widget, TRUE);
-  gtk_widget_set_halign (label_widget, GTK_ALIGN_CENTER);
   arrow = gtk_builtin_icon_new ("arrow");
   menu_button->arrow_widget = arrow;
   gtk_box_append (GTK_BOX (box), label_widget);
@@ -1142,11 +1184,6 @@ gtk_menu_button_set_label (GtkMenuButton *menu_button,
   gtk_button_set_child (GTK_BUTTON (menu_button->button), box);
   menu_button->label_widget = label_widget;
 
-  gtk_accessible_update_relation (GTK_ACCESSIBLE (menu_button->button),
-                                  GTK_ACCESSIBLE_RELATION_LABELLED_BY, menu_button->label_widget, NULL,
-                                  GTK_ACCESSIBLE_RELATION_DESCRIBED_BY, menu_button->label_widget, NULL,
-                                  -1);
-
   menu_button->image_widget = NULL;
   menu_button->child = NULL;
 
@@ -1158,7 +1195,7 @@ gtk_menu_button_set_label (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_label: (attributes org.gtk.Method.get_property=label)
+ * gtk_menu_button_get_label:
  * @menu_button: a `GtkMenuButton`
  *
  * Gets the label shown in the button
@@ -1177,7 +1214,7 @@ gtk_menu_button_get_label (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_has_frame: (attributes org.gtk.Method.set_property=has-frame)
+ * gtk_menu_button_set_has_frame:
  * @menu_button: a `GtkMenuButton`
  * @has_frame: whether the button should have a visible frame
  *
@@ -1197,7 +1234,7 @@ gtk_menu_button_set_has_frame (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_has_frame: (attributes org.gtk.Method.get_property=has-frame)
+ * gtk_menu_button_get_has_frame:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns whether the button has a frame.
@@ -1243,10 +1280,10 @@ gtk_menu_button_popdown (GtkMenuButton *menu_button)
 /**
  * gtk_menu_button_set_create_popup_func:
  * @menu_button: a `GtkMenuButton`
- * @func: (nullable): function to call when a popup is about to
- *   be shown, but none has been provided via other means, or %NULL
- *   to reset to default behavior.
- * @user_data: (closure): user data to pass to @func.
+ * @func: (nullable) (scope notified) (closure user_data) (destroy destroy_notify): function
+ *   to call when a popup is about to be shown, but none has been provided via other means,
+ *   or %NULL to reset to default behavior
+ * @user_data: user data to pass to @func
  * @destroy_notify: (nullable): destroy notify for @user_data
  *
  * Sets @func to be called when a popup is about to be shown.
@@ -1281,7 +1318,7 @@ gtk_menu_button_set_create_popup_func (GtkMenuButton                *menu_button
 }
 
 /**
- * gtk_menu_button_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
+ * gtk_menu_button_set_use_underline:
  * @menu_button: a `GtkMenuButton`
  * @use_underline: %TRUE if underlines in the text indicate mnemonics
  *
@@ -1304,7 +1341,7 @@ gtk_menu_button_set_use_underline (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
+ * gtk_menu_button_get_use_underline:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns whether an embedded underline in the text indicates a
@@ -1322,35 +1359,35 @@ gtk_menu_button_get_use_underline (GtkMenuButton *menu_button)
 }
 
 static GList *
-get_menu_bars (GtkWindow *window)
+get_menu_bars (GtkWidget *toplevel)
 {
-  return g_object_get_data (G_OBJECT (window), "gtk-menu-bar-list");
+  return g_object_get_data (G_OBJECT (toplevel), "gtk-menu-bar-list");
 }
 
 static void
-set_menu_bars (GtkWindow *window,
+set_menu_bars (GtkWidget *toplevel,
                GList     *menubars)
 {
-  g_object_set_data (G_OBJECT (window), I_("gtk-menu-bar-list"), menubars);
+  g_object_set_data (G_OBJECT (toplevel), I_("gtk-menu-bar-list"), menubars);
 }
 
 static void
-add_to_window (GtkWindow     *window,
-               GtkMenuButton *button)
+add_to_toplevel (GtkWidget     *toplevel,
+                 GtkMenuButton *button)
 {
-  GList *menubars = get_menu_bars (window);
+  GList *menubars = get_menu_bars (toplevel);
 
-  set_menu_bars (window, g_list_prepend (menubars, button));
+  set_menu_bars (toplevel, g_list_prepend (menubars, button));
 }
 
 static void
-remove_from_window (GtkWindow     *window,
-                    GtkMenuButton *button)
+remove_from_toplevel (GtkWidget     *toplevel,
+                      GtkMenuButton *button)
 {
-  GList *menubars = get_menu_bars (window);
+  GList *menubars = get_menu_bars (toplevel);
 
   menubars = g_list_remove (menubars, button);
-  set_menu_bars (window, menubars);
+  set_menu_bars (toplevel, menubars);
 }
 
 static void
@@ -1363,7 +1400,7 @@ gtk_menu_button_root (GtkWidget *widget)
   if (button->primary)
     {
       GtkWidget *toplevel = GTK_WIDGET (gtk_widget_get_root (widget));
-      add_to_window (GTK_WINDOW (toplevel), button);
+      add_to_toplevel (toplevel, button);
     }
 }
 
@@ -1373,13 +1410,13 @@ gtk_menu_button_unroot (GtkWidget *widget)
   GtkWidget *toplevel;
 
   toplevel = GTK_WIDGET (gtk_widget_get_root (widget));
-  remove_from_window (GTK_WINDOW (toplevel), GTK_MENU_BUTTON (widget));
+  remove_from_toplevel (toplevel, GTK_MENU_BUTTON (widget));
 
   GTK_WIDGET_CLASS (gtk_menu_button_parent_class)->unroot (widget);
 }
 
 /**
- * gtk_menu_button_set_primary: (attributes org.gtk.Method.set_property=primary)
+ * gtk_menu_button_set_primary:
  * @menu_button: a `GtkMenuButton`
  * @primary: whether the menubutton should act as a primary menu
  *
@@ -1406,16 +1443,16 @@ gtk_menu_button_set_primary (GtkMenuButton *menu_button,
   if (toplevel)
     {
       if (menu_button->primary)
-        add_to_window (GTK_WINDOW (toplevel), menu_button);
+        add_to_toplevel (GTK_WIDGET (toplevel), menu_button);
       else
-        remove_from_window (GTK_WINDOW (toplevel), menu_button);
+        remove_from_toplevel (GTK_WIDGET (toplevel), menu_button);
     }
 
   g_object_notify_by_pspec (G_OBJECT (menu_button), menu_button_props[PROP_PRIMARY]);
 }
 
 /**
- * gtk_menu_button_get_primary: (attributes org.gtk.Method.get_property=primary)
+ * gtk_menu_button_get_primary:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns whether the menu button acts as a primary menu.
@@ -1433,7 +1470,7 @@ gtk_menu_button_get_primary (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_child: (attributes org.gtk.Method.set_property=child)
+ * gtk_menu_button_set_child:
  * @menu_button: a `GtkMenuButton`
  * @child: (nullable): the child widget
  *
@@ -1452,10 +1489,13 @@ void
 gtk_menu_button_set_child (GtkMenuButton *menu_button,
                            GtkWidget     *child)
 {
-  GtkWidget *box, *arrow;
+  GtkWidget *box, *arrow, *inner_widget;
 
   g_return_if_fail (GTK_IS_MENU_BUTTON (menu_button));
-  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+  g_return_if_fail (child == NULL || menu_button->child == child || gtk_widget_get_parent (child) == NULL);
+
+  if (menu_button->child == child)
+    return;
 
   g_object_freeze_notify (G_OBJECT (menu_button));
 
@@ -1465,12 +1505,26 @@ gtk_menu_button_set_child (GtkMenuButton *menu_button,
     g_object_notify_by_pspec (G_OBJECT (menu_button), menu_button_props[PROP_ICON_NAME]);
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_widget_set_halign (box, GTK_ALIGN_CENTER);
+  gtk_widget_set_hexpand (box, FALSE);
 
   arrow = gtk_builtin_icon_new ("arrow");
   menu_button->arrow_widget = arrow;
 
-  gtk_box_append (GTK_BOX (box), child);
+  inner_widget = gtk_gizmo_new_with_role ("contents",
+                                          GTK_ACCESSIBLE_ROLE_GROUP,
+                                          NULL,
+                                          NULL,
+                                          NULL,
+                                          NULL,
+                                          (GtkGizmoFocusFunc)gtk_widget_focus_child,
+                                          NULL);
+
+  gtk_widget_set_layout_manager (inner_widget, gtk_bin_layout_new ());
+  gtk_widget_set_hexpand (inner_widget, TRUE);
+  if (child)
+    gtk_widget_set_parent (child, inner_widget);
+
+  gtk_box_append (GTK_BOX (box), inner_widget);
   gtk_box_append (GTK_BOX (box), arrow);
   gtk_button_set_child (GTK_BUTTON (menu_button->button), box);
 
@@ -1487,7 +1541,7 @@ gtk_menu_button_set_child (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_child: (attributes org.gtk.Method.get_property=child)
+ * gtk_menu_button_get_child:
  * @menu_button: a `GtkMenuButton`
  *
  * Gets the child widget of @menu_button.
@@ -1503,3 +1557,101 @@ gtk_menu_button_get_child (GtkMenuButton *menu_button)
 
   return menu_button->child;
 }
+
+/**
+ * gtk_menu_button_set_active:
+ * @menu_button: a `GtkMenuButton`
+ * @active: whether the menu button is active
+ *
+ * Sets whether the menu button is active.
+ *
+ * Since: 4.10
+ */
+void
+gtk_menu_button_set_active (GtkMenuButton *menu_button,
+                            gboolean       active)
+{
+  g_return_if_fail (GTK_IS_MENU_BUTTON (menu_button));
+
+  if (active == gtk_menu_button_get_active (menu_button))
+    return;
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (menu_button->button),
+                                active);
+
+  g_object_notify_by_pspec (G_OBJECT (menu_button), menu_button_props[PROP_ACTIVE]);
+}
+
+/**
+ * gtk_menu_button_get_active:
+ * @menu_button: a `GtkMenuButton`
+ *
+ * Returns whether the menu button is active.
+ *
+ * Returns: TRUE if the button is active
+ *
+ * Since: 4.10
+ */
+gboolean
+gtk_menu_button_get_active (GtkMenuButton *menu_button)
+{
+  g_return_val_if_fail (GTK_IS_MENU_BUTTON (menu_button), FALSE);
+
+  return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (menu_button->button));
+}
+
+/**
+ * gtk_menu_button_set_can_shrink:
+ * @menu_button: a menu button
+ * @can_shrink: whether the button can shrink
+ *
+ * Sets whether the button size can be smaller than the natural size of
+ * its contents.
+ *
+ * For text buttons, setting @can_shrink to true will ellipsize the label.
+ *
+ * For icon buttons, this function has no effect.
+ *
+ * Since: 4.12
+ */
+void
+gtk_menu_button_set_can_shrink (GtkMenuButton *menu_button,
+                                gboolean       can_shrink)
+{
+  g_return_if_fail (GTK_IS_MENU_BUTTON (menu_button));
+
+  can_shrink = !!can_shrink;
+
+  if (menu_button->can_shrink == can_shrink)
+    return;
+
+  menu_button->can_shrink = can_shrink;
+
+  if (menu_button->label_widget != NULL)
+    {
+      gtk_label_set_ellipsize (GTK_LABEL (menu_button->label_widget),
+                               can_shrink ? PANGO_ELLIPSIZE_END
+                                          : PANGO_ELLIPSIZE_NONE);
+    }
+
+  g_object_notify_by_pspec (G_OBJECT (menu_button), menu_button_props[PROP_CAN_SHRINK]);
+}
+
+/**
+ * gtk_menu_button_get_can_shrink:
+ * @menu_button: a button
+ *
+ * Retrieves whether the button can be smaller than the natural
+ * size of its contents.
+ *
+ * Returns: true if the button can shrink, and false otherwise
+ *
+ * Since: 4.12
+ */
+gboolean
+gtk_menu_button_get_can_shrink (GtkMenuButton *menu_button)
+{
+  g_return_val_if_fail (GTK_IS_MENU_BUTTON (menu_button), FALSE);
+
+  return menu_button->can_shrink;
+}
