@@ -38,6 +38,7 @@
 #include "gtktypebuiltins.h"
 #include "gtkwidgetprivate.h"
 #include "gtkbuildable.h"
+#include "gtkbuilderprivate.h"
 
 #include <math.h>
 
@@ -74,6 +75,17 @@
  * The application can set the position of the slider as if it were set
  * by the user, by calling [method@Gtk.Paned.set_position].
  *
+ * # Shortcuts and Gestures
+ *
+ * The following signals have default keybindings:
+ *
+ * - [signal@Gtk.Paned::accept-position]
+ * - [signal@Gtk.Paned::cancel-position]
+ * - [signal@Gtk.Paned::cycle-child-focus]
+ * - [signal@Gtk.Paned::cycle-handle-focus]
+ * - [signal@Gtk.Paned::move-handle]
+ * - [signal@Gtk.Paned::toggle-handle-focus]
+ *
  * # CSS nodes
  *
  * ```
@@ -421,7 +433,7 @@ gtk_paned_class_init (GtkPanedClass *class)
   class->cancel_position = gtk_paned_cancel_position;
 
   /**
-   * GtkPaned:position: (attributes org.gtk.Property.get=gtk_paned_get_position org.gtk.Property.set=gtk_paned_set_position)
+   * GtkPaned:position:
    *
    * Position of the separator in pixels, from the left/top.
    */
@@ -469,7 +481,7 @@ gtk_paned_class_init (GtkPanedClass *class)
                       GTK_PARAM_READABLE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPaned:wide-handle: (attributes org.gtk.Property.get=gtk_paned_get_wide_handle org.gtk.Property.set=gtk_paned_set_wide_handle)
+   * GtkPaned:wide-handle:
    *
    * Whether the `GtkPaned` should provide a stronger visual separation.
    *
@@ -482,7 +494,7 @@ gtk_paned_class_init (GtkPanedClass *class)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPaned:resize-start-child: (attributes org.gtk.Property.get=gtk_paned_get_resize_start_child org.gtk.Property.set=gtk_paned_set_resize_start_child)
+   * GtkPaned:resize-start-child:
    *
    * Determines whether the first child expands and shrinks
    * along with the paned widget.
@@ -493,7 +505,7 @@ gtk_paned_class_init (GtkPanedClass *class)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPaned:resize-end-child: (attributes org.gtk.Property.get=gtk_paned_get_resize_end_child org.gtk.Property.set=gtk_paned_set_resize_end_child)
+   * GtkPaned:resize-end-child:
    *
    * Determines whether the second child expands and shrinks
    * along with the paned widget.
@@ -504,7 +516,7 @@ gtk_paned_class_init (GtkPanedClass *class)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPaned:shrink-start-child: (attributes org.gtk.Property.get=gtk_paned_get_shrink_start_child org.gtk.Property.set=gtk_paned_set_shrink_start_child)
+   * GtkPaned:shrink-start-child:
    *
    * Determines whether the first child can be made smaller
    * than its requisition.
@@ -515,7 +527,7 @@ gtk_paned_class_init (GtkPanedClass *class)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPaned:shrink-end-child: (attributes org.gtk.Property.get=gtk_paned_get_shrink_end_child org.gtk.Property.set=gtk_paned_set_shrink_end_child)
+   * GtkPaned:shrink-end-child:
    *
    * Determines whether the second child can be made smaller
    * than its requisition.
@@ -526,7 +538,7 @@ gtk_paned_class_init (GtkPanedClass *class)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPaned:start-child: (attributes org.gtk.Property.get=gtk_paned_get_start_child org.gtk.Property.set=gtk_paned_set_start_child)
+   * GtkPaned:start-child:
    *
    * The first child.
    */
@@ -536,7 +548,7 @@ gtk_paned_class_init (GtkPanedClass *class)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPaned:end-child: (attributes org.gtk.Property.get=gtk_paned_get_end_child org.gtk.Property.set=gtk_paned_set_end_child)
+   * GtkPaned:end-child:
    *
    * The second child.
    */
@@ -558,6 +570,8 @@ gtk_paned_class_init (GtkPanedClass *class)
    * This is a [keybinding signal](class.SignalAction.html).
    *
    * The default binding is <kbd>F6</kbd>.
+   *
+   * Returns: whether the behavior was cycled
    */
   signals [CYCLE_CHILD_FOCUS] =
     g_signal_new (I_("cycle-child-focus"),
@@ -582,6 +596,8 @@ gtk_paned_class_init (GtkPanedClass *class)
    * This is a [keybinding signal](class.SignalAction.html).
    *
    * The default binding is <kbd>Tab</kbd>.
+   *
+   * Return: whether handle focus was toggled
    */
   signals [TOGGLE_HANDLE_FOCUS] =
     g_signal_new (I_("toggle-handle-focus"),
@@ -603,6 +619,15 @@ gtk_paned_class_init (GtkPanedClass *class)
    * Emitted to move the handle with key bindings.
    *
    * This is a [keybinding signal](class.SignalAction.html).
+   *
+   * The default bindings for this signal are
+   * <kbd>Ctrl</kbd>+<kbd>←</kbd>, <kbd>←</kbd>,
+   * <kbd>Ctrl</kbd>+<kbd>→</kbd>, <kbd>→</kbd>,
+   * <kbd>Ctrl</kbd>+<kbd>↑</kbd>, <kbd>↑</kbd>,
+   * <kbd>Ctrl</kbd>+<kbd>↓</kbd>, <kbd>↓</kbd>,
+   * <kbd>PgUp</kbd>, <kbd>PgDn</kbd>, <kbd>Home</kbd>, <kbd>End</kbd>.
+   *
+   * Returns: whether the handle was moved
    */
   signals[MOVE_HANDLE] =
     g_signal_new (I_("move-handle"),
@@ -628,6 +653,8 @@ gtk_paned_class_init (GtkPanedClass *class)
    * This is a [keybinding signal](class.SignalAction.html).
    *
    * The default binding for this signal is <kbd>F8</kbd>.
+   *
+   * Returns: whether the behavior was cycled
    */
   signals [CYCLE_HANDLE_FOCUS] =
     g_signal_new (I_("cycle-handle-focus"),
@@ -653,6 +680,8 @@ gtk_paned_class_init (GtkPanedClass *class)
    *
    * The default binding for this signal is <kbd>Return</kbd> or
    * <kbd>Space</kbd>.
+   *
+   * Returns: whether the position was accepted
    */
   signals [ACCEPT_POSITION] =
     g_signal_new (I_("accept-position"),
@@ -679,6 +708,8 @@ gtk_paned_class_init (GtkPanedClass *class)
    * This is a [keybinding signal](class.SignalAction.html).
    *
    * The default binding for this signal is <kbd>Escape</kbd>.
+   *
+   * Returns: whether the position was canceled
    */
   signals [CANCEL_POSITION] =
     g_signal_new (I_("cancel-position"),
@@ -789,18 +820,21 @@ gtk_paned_buildable_add_child (GtkBuildable *buildable,
 
   if (g_strcmp0 (type, "start") == 0)
     {
+      gtk_buildable_child_deprecation_warning (buildable, builder, "start", "start-child");
       gtk_paned_set_start_child (self, GTK_WIDGET (child));
       gtk_paned_set_resize_start_child (self, FALSE);
       gtk_paned_set_shrink_start_child (self, TRUE);
     }
   else if (g_strcmp0 (type, "end") == 0)
     {
+      gtk_buildable_child_deprecation_warning (buildable, builder, "end", "end-child");
       gtk_paned_set_end_child (self, GTK_WIDGET (child));
       gtk_paned_set_resize_end_child (self, TRUE);
       gtk_paned_set_shrink_end_child (self, TRUE);
     }
   else if (type == NULL && GTK_IS_WIDGET (child))
     {
+      gtk_buildable_child_deprecation_warning (buildable, builder, NULL, "start-child or end-child");
       if (self->start_child == NULL)
         {
           gtk_paned_set_start_child (self, GTK_WIDGET (child));
@@ -1591,7 +1625,7 @@ gtk_paned_new (GtkOrientation orientation)
 }
 
 /**
- * gtk_paned_set_start_child: (attributes org.gtk.Method.set_property=start-child)
+ * gtk_paned_set_start_child:
  * @paned: a `GtkPaned`
  * @child: (nullable): the widget to add
  *
@@ -1621,7 +1655,7 @@ gtk_paned_set_start_child (GtkPaned  *paned,
 }
 
 /**
- * gtk_paned_get_start_child: (attributes org.gtk.Method.get_property=start-child)
+ * gtk_paned_get_start_child:
  * @paned: a `GtkPaned`
  *
  * Retrieves the start child of the given `GtkPaned`.
@@ -1637,7 +1671,7 @@ gtk_paned_get_start_child (GtkPaned *paned)
 }
 
 /**
- * gtk_paned_set_resize_start_child: (attributes org.gtk.Method.set_property=resize-start-child)
+ * gtk_paned_set_resize_start_child:
  * @paned: a `GtkPaned`
  * @resize: true to let the start child be resized
  *
@@ -1658,7 +1692,7 @@ gtk_paned_set_resize_start_child (GtkPaned *paned,
 }
 
 /**
- * gtk_paned_get_resize_start_child: (attributes org.gtk.Method.get_property=resize-start-child)
+ * gtk_paned_get_resize_start_child:
  * @paned: a `GtkPaned`
  *
  * Returns whether the [property@Gtk.Paned:start-child] can be resized.
@@ -1674,7 +1708,7 @@ gtk_paned_get_resize_start_child (GtkPaned *paned)
 }
 
 /**
- * gtk_paned_set_shrink_start_child: (attributes org.gtk.Method.set_property=shrink-start-child)
+ * gtk_paned_set_shrink_start_child:
  * @paned: a `GtkPaned`
  * @resize: true to let the start child be shrunk
  *
@@ -1695,7 +1729,7 @@ gtk_paned_set_shrink_start_child (GtkPaned *paned,
 }
 
 /**
- * gtk_paned_get_shrink_start_child: (attributes org.gtk.Method.get_property=shrink-start-child)
+ * gtk_paned_get_shrink_start_child:
  * @paned: a `GtkPaned`
  *
  * Returns whether the [property@Gtk.Paned:start-child] can shrink.
@@ -1711,7 +1745,7 @@ gtk_paned_get_shrink_start_child (GtkPaned *paned)
 }
 
 /**
- * gtk_paned_set_end_child: (attributes org.gtk.Method.set_property=end-child)
+ * gtk_paned_set_end_child:
  * @paned: a `GtkPaned`
  * @child: (nullable): the widget to add
  *
@@ -1741,7 +1775,7 @@ gtk_paned_set_end_child (GtkPaned  *paned,
 }
 
 /**
- * gtk_paned_get_end_child: (attributes org.gtk.Method.get_property=end-child)
+ * gtk_paned_get_end_child:
  * @paned: a `GtkPaned`
  *
  * Retrieves the end child of the given `GtkPaned`.
@@ -1757,7 +1791,7 @@ gtk_paned_get_end_child (GtkPaned *paned)
 }
 
 /**
- * gtk_paned_set_resize_end_child: (attributes org.gtk.Method.set_property=resize-end-child)
+ * gtk_paned_set_resize_end_child:
  * @paned: a `GtkPaned`
  * @resize: true to let the end child be resized
  *
@@ -1778,7 +1812,7 @@ gtk_paned_set_resize_end_child (GtkPaned *paned,
 }
 
 /**
- * gtk_paned_get_resize_end_child: (attributes org.gtk.Method.get_property=resize-end-child)
+ * gtk_paned_get_resize_end_child:
  * @paned: a `GtkPaned`
  *
  * Returns whether the [property@Gtk.Paned:end-child] can be resized.
@@ -1794,7 +1828,7 @@ gtk_paned_get_resize_end_child (GtkPaned *paned)
 }
 
 /**
- * gtk_paned_set_shrink_end_child: (attributes org.gtk.Method.set_property=shrink-end-child)
+ * gtk_paned_set_shrink_end_child:
  * @paned: a `GtkPaned`
  * @resize: true to let the end child be shrunk
  *
@@ -1815,7 +1849,7 @@ gtk_paned_set_shrink_end_child (GtkPaned *paned,
 }
 
 /**
- * gtk_paned_get_shrink_end_child: (attributes org.gtk.Method.get_property=shrink-end-child)
+ * gtk_paned_get_shrink_end_child:
  * @paned: a `GtkPaned`
  *
  * Returns whether the [property@Gtk.Paned:end-child] can shrink.
@@ -1831,7 +1865,7 @@ gtk_paned_get_shrink_end_child (GtkPaned *paned)
 }
 
 /**
- * gtk_paned_get_position: (attributes org.gtk.Method.get_property=position)
+ * gtk_paned_get_position:
  * @paned: a `GtkPaned` widget
  *
  * Obtains the position of the divider between the two panes.
@@ -1847,7 +1881,7 @@ gtk_paned_get_position (GtkPaned  *paned)
 }
 
 /**
- * gtk_paned_set_position: (attributes org.gtk.Method.set_property=position)
+ * gtk_paned_set_position:
  * @paned: a `GtkPaned` widget
  * @position: pixel position of divider, a negative value means that the position
  *   is unset
@@ -2529,7 +2563,7 @@ gtk_paned_toggle_handle_focus (GtkPaned *paned)
 }
 
 /**
- * gtk_paned_set_wide_handle: (attributes org.gtk.Method.set_property=wide-handle)
+ * gtk_paned_set_wide_handle:
  * @paned: a `GtkPaned`
  * @wide: the new value for the [property@Gtk.Paned:wide-handle] property
  *
@@ -2556,7 +2590,7 @@ gtk_paned_set_wide_handle (GtkPaned *paned,
 }
 
 /**
- * gtk_paned_get_wide_handle: (attributes org.gtk.Method.get_property=wide-handle)
+ * gtk_paned_get_wide_handle:
  * @paned: a `GtkPaned`
  *
  * Gets whether the separator should be wide.
