@@ -229,6 +229,19 @@ gtk_picture_measure (GtkWidget      *widget,
                                            &min_width, &min_height);
     }
 
+  if (for_size > 0 && self->content_fit == GTK_CONTENT_FIT_SCALE_DOWN)
+    {
+      double opposite_intrinsic_size;
+
+      if (orientation == GTK_ORIENTATION_HORIZONTAL)
+        opposite_intrinsic_size = gdk_paintable_get_intrinsic_height (self->paintable);
+      else
+        opposite_intrinsic_size = gdk_paintable_get_intrinsic_width (self->paintable);
+
+      if (opposite_intrinsic_size != 0 && opposite_intrinsic_size < for_size)
+        for_size = opposite_intrinsic_size;
+    }
+
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
     {
       gdk_paintable_compute_concrete_size (self->paintable,
@@ -875,6 +888,8 @@ gtk_picture_set_paintable (GtkPicture   *self,
 
   if (size_changed)
     gtk_widget_queue_resize (GTK_WIDGET (self));
+  else
+    gtk_widget_queue_draw (GTK_WIDGET (self));
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PAINTABLE]);
 
@@ -1010,6 +1025,7 @@ gtk_picture_set_content_fit (GtkPicture    *self,
                              GtkContentFit  content_fit)
 {
   gboolean notify_keep_aspect_ratio;
+  gboolean queue_resize;
 
   g_return_if_fail (GTK_IS_PICTURE (self));
 
@@ -1018,10 +1034,15 @@ gtk_picture_set_content_fit (GtkPicture    *self,
 
   notify_keep_aspect_ratio = (content_fit == GTK_CONTENT_FIT_FILL ||
                               self->content_fit == GTK_CONTENT_FIT_FILL);
+  queue_resize = (content_fit == GTK_CONTENT_FIT_SCALE_DOWN ||
+                  self->content_fit == GTK_CONTENT_FIT_SCALE_DOWN);
 
   self->content_fit = content_fit;
 
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  if (queue_resize)
+    gtk_widget_queue_resize (GTK_WIDGET (self));
+  else
+    gtk_widget_queue_draw (GTK_WIDGET (self));
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_CONTENT_FIT]);
 
