@@ -67,6 +67,17 @@
  * </section>
  * ```
  *
+ * # Shortcuts and Gestures
+ *
+ * `GtkPopover` supports the following keyboard shortcuts:
+ *
+ * - <kbd>Escape</kbd> closes the popover.
+ * - <kbd>Alt</kbd> makes the mnemonics visible.
+ *
+ * The following signals have default keybindings:
+ *
+ * - [signal@Gtk.Popover::activate-default]
+ *
  * # CSS nodes
  *
  * ```
@@ -123,6 +134,7 @@
 #include "gtkrenderbackgroundprivate.h"
 #include "gtkshortcutmanager.h"
 #include "gtkbuildable.h"
+#include "gtkbuilderprivate.h"
 #include "gtktooltipprivate.h"
 #include "gtkcssboxesimplprivate.h"
 #include "gtknativeprivate.h"
@@ -155,7 +167,6 @@ typedef struct {
   gboolean autohide;
   gboolean has_arrow;
   gboolean mnemonics_visible;
-  gboolean disable_auto_mnemonics;
   gboolean cascade_popdown;
 
   int x_offset;
@@ -496,7 +507,7 @@ create_popup_layout (GtkPopover *popover)
   compute_surface_pointing_to (popover, &rect);
 
   style = gtk_css_node_get_style (gtk_widget_get_css_node (GTK_WIDGET (priv->contents_widget)));
-  gtk_css_shadow_value_get_extents (style->background->box_shadow, &shadow_width);
+  gtk_css_shadow_value_get_extents (style->used->box_shadow, &shadow_width);
 
   switch (priv->position)
     {
@@ -785,10 +796,6 @@ static void
 gtk_popover_focus_in (GtkWidget *widget)
 {
   GtkPopover *popover = GTK_POPOVER (widget);
-  GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
-
-  if (priv->disable_auto_mnemonics)
-    return;
 
   if (gtk_widget_get_visible (widget))
     {
@@ -801,10 +808,6 @@ static void
 gtk_popover_focus_out (GtkWidget *widget)
 {
   GtkPopover *popover = GTK_POPOVER (widget);
-  GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
-
-  if (priv->disable_auto_mnemonics)
-    return;
 
   gtk_popover_set_mnemonics_visible (popover, FALSE);
 }
@@ -815,11 +818,6 @@ update_mnemonics_visible (GtkPopover      *popover,
                           GdkModifierType  state,
                           gboolean         visible)
 {
-  GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
-
-  if (priv->disable_auto_mnemonics)
-    return;
-
   if ((keyval == GDK_KEY_Alt_L || keyval == GDK_KEY_Alt_R) &&
       ((state & (gtk_accelerator_get_default_mod_mask ()) & ~(GDK_ALT_MASK)) == 0))
     {
@@ -1018,8 +1016,6 @@ gtk_popover_realize (GtkWidget *widget)
   priv->renderer = gsk_renderer_new_for_surface (priv->surface);
 
   gtk_native_realize (GTK_NATIVE (popover));
-
-  gtk_native_update_opaque_region (GTK_NATIVE (popover), priv->contents_widget, TRUE, TRUE, 0);
 }
 
 static void
@@ -1263,11 +1259,11 @@ gtk_popover_get_gap_coords (GtkPopover *popover,
 
   style = gtk_css_node_get_style (gtk_widget_get_css_node (priv->contents_widget));
   border_radius = round (get_border_radius (widget));
-  border_top = _gtk_css_number_value_get (style->border->border_top_width, 100);
-  border_right = _gtk_css_number_value_get (style->border->border_right_width, 100);
-  border_bottom = _gtk_css_number_value_get (style->border->border_bottom_width, 100);
+  border_top = gtk_css_number_value_get (style->border->border_top_width, 100);
+  border_right = gtk_css_number_value_get (style->border->border_right_width, 100);
+  border_bottom = gtk_css_number_value_get (style->border->border_bottom_width, 100);
 
-  gtk_css_shadow_value_get_extents (style->background->box_shadow, &shadow_width);
+  gtk_css_shadow_value_get_extents (style->used->box_shadow, &shadow_width);
 
   if (pos == GTK_POS_BOTTOM)
     {
@@ -1344,10 +1340,10 @@ get_border (GtkCssNode *node,
 
   style = gtk_css_node_get_style (node);
 
-  border->top = _gtk_css_number_value_get (style->border->border_top_width, 100);
-  border->right = _gtk_css_number_value_get (style->border->border_right_width, 100);
-  border->bottom = _gtk_css_number_value_get (style->border->border_bottom_width, 100);
-  border->left = _gtk_css_number_value_get (style->border->border_left_width, 100);
+  border->top = gtk_css_number_value_get (style->border->border_top_width, 100);
+  border->right = gtk_css_number_value_get (style->border->border_right_width, 100);
+  border->bottom = gtk_css_number_value_get (style->border->border_bottom_width, 100);
+  border->left = gtk_css_number_value_get (style->border->border_left_width, 100);
 }
 
 static void
@@ -1441,7 +1437,7 @@ gtk_popover_update_shape (GtkPopover *popover)
       content_css_node =
         gtk_widget_get_css_node (GTK_WIDGET (priv->contents_widget));
       style = gtk_css_node_get_style (content_css_node);
-      gtk_css_shadow_value_get_extents (style->background->box_shadow, &shadow_width);
+      gtk_css_shadow_value_get_extents (style->used->box_shadow, &shadow_width);
 
       input_rect.x = shadow_width.left;
       input_rect.y = shadow_width.top;
@@ -1456,9 +1452,6 @@ gtk_popover_update_shape (GtkPopover *popover)
       gdk_surface_set_input_region (priv->surface, region);
       cairo_region_destroy (region);
     }
-
-  if (_gtk_widget_get_realized (GTK_WIDGET (popover)))
-    gtk_native_update_opaque_region (GTK_NATIVE (popover), priv->contents_widget, TRUE, TRUE, 0);
 }
 
 static int
@@ -1505,7 +1498,7 @@ gtk_popover_measure (GtkWidget      *widget,
   GtkBorder shadow_width;
 
   style = gtk_css_node_get_style (gtk_widget_get_css_node (GTK_WIDGET (priv->contents_widget)));
-  gtk_css_shadow_value_get_extents (style->background->box_shadow, &shadow_width);
+  gtk_css_shadow_value_get_extents (style->used->box_shadow, &shadow_width);
 
   if (for_size >= 0)
     {
@@ -1559,7 +1552,7 @@ gtk_popover_size_allocate (GtkWidget *widget,
   GtkBorder shadow_width;
 
   style = gtk_css_node_get_style (gtk_widget_get_css_node (GTK_WIDGET (priv->contents_widget)));
-  gtk_css_shadow_value_get_extents (style->background->box_shadow, &shadow_width);
+  gtk_css_shadow_value_get_extents (style->used->box_shadow, &shadow_width);
 
   switch (priv->final_position)
     {
@@ -1652,7 +1645,7 @@ create_arrow_render_node (GtkPopover *popover)
     {
       const GdkRGBA *border_color;
 
-      border_color = gtk_css_color_value_get_rgba (style->border->border_left_color ? style->border->border_left_color : style->core->color);
+      border_color = gtk_css_color_value_get_rgba (style->used->border_left_color);
 
       gtk_popover_apply_tail_path (popover, cr);
       gdk_cairo_set_source_rgba (cr, border_color);
@@ -1879,7 +1872,7 @@ gtk_popover_class_init (GtkPopoverClass *klass)
   klass->activate_default = gtk_popover_activate_default;
 
   /**
-   * GtkPopover:pointing-to: (attributes org.gtk.Property.get=gtk_popover_get_pointing_to org.gtk.Property.set=gtk_popover_set_pointing_to)
+   * GtkPopover:pointing-to:
    *
    * Rectangle in the parent widget that the popover points to.
    */
@@ -1889,7 +1882,7 @@ gtk_popover_class_init (GtkPopoverClass *klass)
                           GTK_PARAM_READWRITE);
 
   /**
-   * GtkPopover:position: (attributes org.gtk.Property.get=gtk_popover_get_position org.gtk.Property.set=gtk_popover_set_position)
+   * GtkPopover:position:
    *
    * How to place the popover, relative to its parent.
    */
@@ -1899,7 +1892,7 @@ gtk_popover_class_init (GtkPopoverClass *klass)
                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPopover:autohide: (attributes org.gtk.Property.get=gtk_popover_get_autohide org.gtk.Property.set=gtk_popover_set_autohide)
+   * GtkPopover:autohide:
    *
    * Whether to dismiss the popover on outside clicks.
    */
@@ -1909,7 +1902,7 @@ gtk_popover_class_init (GtkPopoverClass *klass)
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPopover:default-widget: (attributes org.gtk.Popover.set=gtk_popover_set_default_widget)
+   * GtkPopover:default-widget:
    *
    * The default widget inside the popover.
    */
@@ -1919,7 +1912,7 @@ gtk_popover_class_init (GtkPopoverClass *klass)
                            GTK_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPopover:has-arrow: (attributes org.gtk.Popover.get=gtk_popover_get_has_arrow org.gtk.Property.set=gtk_popover_set_has_arrow)
+   * GtkPopover:has-arrow:
    *
    * Whether to draw an arrow.
    */
@@ -1929,7 +1922,7 @@ gtk_popover_class_init (GtkPopoverClass *klass)
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPopover:mnemonics-visible: (attributes org.gtk.Property.get=gtk_popover_get_mnemonics_visible org.gtk.Property.set=gtk_popover_set_mnemonics_visible)
+   * GtkPopover:mnemonics-visible:
    *
    * Whether mnemonics are currently visible in this popover.
    */
@@ -1939,7 +1932,7 @@ gtk_popover_class_init (GtkPopoverClass *klass)
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPopover:child: (attributes org.gtk.Property.get=gtk_popover_get_child org.gtk.Property.set=gtk_popover_set_child)
+   * GtkPopover:child:
    *
    * The child widget.
    */
@@ -1949,7 +1942,7 @@ gtk_popover_class_init (GtkPopoverClass *klass)
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPopover:cascade-popdown: (attributes org.gtk.Property.get=gtk_popover_get_cascade_popdown org.gtk.Property.set=gtk_popover_set_cascade_popdown)
+   * GtkPopover:cascade-popdown:
    *
    * Whether the popover pops down after a child popover.
    *
@@ -1985,6 +1978,8 @@ gtk_popover_class_init (GtkPopoverClass *klass)
    * Emitted whend the user activates the default widget.
    *
    * This is a [keybinding signal](class.SignalAction.html).
+   *
+   * The default binding for this signal is <kbd>Enter</kbd>.
    */
   signals[ACTIVATE_DEFAULT] =
     g_signal_new (I_("activate-default"),
@@ -2030,7 +2025,7 @@ gtk_popover_new (void)
 }
 
 /**
- * gtk_popover_set_child: (attributes org.gtk.Method.set_property=child)
+ * gtk_popover_set_child:
  * @popover: a `GtkPopover`
  * @child: (nullable): the child widget
  *
@@ -2060,7 +2055,7 @@ gtk_popover_set_child (GtkPopover *popover,
 }
 
 /**
- * gtk_popover_get_child: (attributes org.gtk.Method.get_property=child)
+ * gtk_popover_get_child:
  * @popover: a `GtkPopover`
  *
  * Gets the child widget of @popover.
@@ -2079,7 +2074,7 @@ gtk_popover_get_child (GtkPopover *popover)
 
 
 /**
- * gtk_popover_set_default_widget: (attributes org.gtk.Method.set_property=default-widget)
+ * gtk_popover_set_default_widget:
  * @popover: a `GtkPopover`
  * @widget: (nullable): a child widget of @popover to set as
  *   the default, or %NULL to unset the default widget for the popover
@@ -2143,9 +2138,14 @@ gtk_popover_buildable_add_child (GtkBuildable *buildable,
                                  const char   *type)
 {
   if (GTK_IS_WIDGET (child))
-    gtk_popover_set_child (GTK_POPOVER (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, NULL, "child");
+      gtk_popover_set_child (GTK_POPOVER (buildable), GTK_WIDGET (child));
+    }
   else
-    parent_buildable_iface->add_child (buildable, builder, child, type);
+    {
+      parent_buildable_iface->add_child (buildable, builder, child, type);
+    }
 }
 
 static void
@@ -2157,7 +2157,7 @@ gtk_popover_buildable_init (GtkBuildableIface *iface)
 }
 
 /**
- * gtk_popover_set_pointing_to: (attributes org.gtk.Method.set_property=pointing-to)
+ * gtk_popover_set_pointing_to:
  * @popover: a `GtkPopover`
  * @rect: (nullable): rectangle to point to
  *
@@ -2190,7 +2190,7 @@ gtk_popover_set_pointing_to (GtkPopover         *popover,
 }
 
 /**
- * gtk_popover_get_pointing_to: (attributes org.gtk.Method.get_property=pointing-to)
+ * gtk_popover_get_pointing_to:
  * @popover: a `GtkPopover`
  * @rect: (out): location to store the rectangle
  *
@@ -2235,7 +2235,7 @@ gtk_popover_get_pointing_to (GtkPopover   *popover,
 }
 
 /**
- * gtk_popover_set_position: (attributes org.gtk.Method.set_property=position)
+ * gtk_popover_set_position:
  * @popover: a `GtkPopover`
  * @position: preferred popover position
  *
@@ -2271,7 +2271,7 @@ gtk_popover_set_position (GtkPopover      *popover,
 }
 
 /**
- * gtk_popover_get_position: (attributes org.gtk.Method.get_property=position)
+ * gtk_popover_get_position:
  * @popover: a `GtkPopover`
  *
  * Returns the preferred position of @popover.
@@ -2289,7 +2289,7 @@ gtk_popover_get_position (GtkPopover *popover)
 }
 
 /**
- * gtk_popover_set_autohide: (attributes org.gtk.Method.set_property=autohide)
+ * gtk_popover_set_autohide:
  * @popover: a `GtkPopover`
  * @autohide: %TRUE to dismiss the popover on outside clicks
  *
@@ -2324,7 +2324,7 @@ gtk_popover_set_autohide (GtkPopover *popover,
 }
 
 /**
- * gtk_popover_get_autohide: (attributes org.gtk.Method.get_property=autohide)
+ * gtk_popover_get_autohide:
  * @popover: a `GtkPopover`
  *
  * Returns whether the popover is modal.
@@ -2362,17 +2362,20 @@ static void
 cascade_popdown (GtkPopover *popover)
 {
   GtkWidget *parent;
+  GtkWidget *new_focus;
 
   /* Do not trigger cascade close from non-modal popovers */
   if (!gtk_popover_get_autohide (popover))
     return;
 
   parent = gtk_widget_get_parent (GTK_WIDGET (popover));
+  new_focus = parent;
 
   while (parent)
     {
       if (GTK_IS_POPOVER (parent))
         {
+          new_focus = gtk_widget_get_parent (parent);
           if (gtk_popover_get_cascade_popdown (GTK_POPOVER (parent)))
             gtk_widget_set_visible (parent, FALSE);
           else
@@ -2381,6 +2384,9 @@ cascade_popdown (GtkPopover *popover)
 
       parent = gtk_widget_get_parent (parent);
     }
+
+    if (new_focus)
+      gtk_widget_grab_focus (new_focus);
 }
 
 /**
@@ -2397,6 +2403,9 @@ gtk_popover_popdown (GtkPopover *popover)
 {
   g_return_if_fail (GTK_IS_POPOVER (popover));
 
+  if (!gtk_widget_get_visible (GTK_WIDGET (popover)))
+    return;
+
   gtk_widget_set_visible (GTK_WIDGET (popover), FALSE);
 
   cascade_popdown (popover);
@@ -2411,7 +2420,7 @@ gtk_popover_get_contents_widget (GtkPopover *popover)
 }
 
 /**
- * gtk_popover_set_has_arrow: (attributes org.gtk.Method.set_property=has-arrow)
+ * gtk_popover_set_has_arrow:
  * @popover: a `GtkPopover`
  * @has_arrow: %TRUE to draw an arrow
  *
@@ -2436,7 +2445,7 @@ gtk_popover_set_has_arrow (GtkPopover *popover,
 }
 
 /**
- * gtk_popover_get_has_arrow: (attributes org.gtk.Method.get_property=has-arrow)
+ * gtk_popover_get_has_arrow:
  * @popover: a `GtkPopover`
  *
  * Gets whether this popover is showing an arrow
@@ -2455,7 +2464,7 @@ gtk_popover_get_has_arrow (GtkPopover *popover)
 }
 
 /**
- * gtk_popover_set_mnemonics_visible: (attributes org.gtk.Method.set_property=mnemonics-visible)
+ * gtk_popover_set_mnemonics_visible:
  * @popover: a `GtkPopover`
  * @mnemonics_visible: the new value
  *
@@ -2485,7 +2494,7 @@ gtk_popover_set_mnemonics_visible (GtkPopover *popover,
 }
 
 /**
- * gtk_popover_get_mnemonics_visible: (attributes org.gtk.Method.get_property=mnemonics-visible)
+ * gtk_popover_get_mnemonics_visible:
  * @popover: a `GtkPopover`
  *
  * Gets whether mnemonics are visible.
@@ -2503,14 +2512,6 @@ gtk_popover_get_mnemonics_visible (GtkPopover *popover)
   return priv->mnemonics_visible;
 }
 
-void
-gtk_popover_disable_auto_mnemonics (GtkPopover *popover)
-{
-  GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
-
-  priv->disable_auto_mnemonics = TRUE;
-}
-
 /**
  * gtk_popover_set_offset:
  * @popover: a `GtkPopover`
@@ -2566,7 +2567,7 @@ gtk_popover_get_offset (GtkPopover *popover,
 }
 
 /**
- * gtk_popover_set_cascade_popdown: (attributes org.gtk.Method.set_property=cascade-popdown)
+ * gtk_popover_set_cascade_popdown:
  * @popover: A `GtkPopover`
  * @cascade_popdown: %TRUE if the popover should follow a child closing
  *
@@ -2589,7 +2590,7 @@ gtk_popover_set_cascade_popdown (GtkPopover *popover,
 }
 
 /**
- * gtk_popover_get_cascade_popdown: (attributes org.gtk.Method.get_property=cascade-popdown)
+ * gtk_popover_get_cascade_popdown:
  * @popover: a `GtkPopover`
  *
  * Returns whether the popover will close after a modal child is closed.
