@@ -131,6 +131,12 @@
  * [method@WidgetClass.add_binding_action] or
  * [method@Gtk.ShortcutController.add_shortcut].
  *
+ * # Shortcuts and Gestures
+ *
+ * `GtkPopoverMenu` supports the following keyboard shortcuts:
+ *
+ * - <kbd>Space</kbd> activates the default widget.
+ *
  * # CSS Nodes
  *
  * `GtkPopoverMenu` is just a subclass of `GtkPopover` that adds custom content
@@ -349,7 +355,6 @@ gtk_popover_menu_init (GtkPopoverMenu *popover)
     }
   g_free (controllers);
 
-  gtk_popover_disable_auto_mnemonics (GTK_POPOVER (popover));
   gtk_popover_set_cascade_popdown (GTK_POPOVER (popover), TRUE);
 }
 
@@ -614,7 +619,7 @@ gtk_popover_menu_class_init (GtkPopoverMenuClass *klass)
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GtkPopoverMenu:menu-model: (attributes org.gtk.Property.get=gtk_popover_menu_get_menu_model org.gtk.Property.set=gtk_popover_menu_set_menu_model)
+   * GtkPopoverMenu:menu-model:
    *
    * The model from which the menu is made.
    */
@@ -625,7 +630,7 @@ gtk_popover_menu_class_init (GtkPopoverMenuClass *klass)
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GtkPopoverMenu:flags: (attributes org.gtk.Property.get=gtk_popover_menu_get_flags org.gtk.Property.set=gtk_popover_menu_set_flags)
+   * GtkPopoverMenu:flags:
    *
    * The flags that @popover uses to create/display a menu from its model.
    *
@@ -818,7 +823,7 @@ gtk_popover_menu_new_from_model_full (GMenuModel          *model,
 }
 
 /**
- * gtk_popover_menu_set_menu_model: (attributes org.gtk.Method.set_property=menu-model)
+ * gtk_popover_menu_set_menu_model:
  * @popover: a `GtkPopoverMenu`
  * @model: (nullable): a `GMenuModel`
  *
@@ -843,7 +848,7 @@ gtk_popover_menu_set_menu_model (GtkPopoverMenu *popover,
 }
 
 /**
- * gtk_popover_menu_set_flags: (attributes org.gtk.Method.set_property=flags)
+ * gtk_popover_menu_set_flags:
  * @popover: a `GtkPopoverMenu`
  * @flags: a set of `GtkPopoverMenuFlags`
  *
@@ -873,7 +878,7 @@ gtk_popover_menu_set_flags (GtkPopoverMenu      *popover,
 }
 
 /**
- * gtk_popover_menu_get_menu_model: (attributes org.gtk.Method.get_property=menu-model)
+ * gtk_popover_menu_get_menu_model:
  * @popover: a `GtkPopoverMenu`
  *
  * Returns the menu model used to populate the popover.
@@ -889,7 +894,7 @@ gtk_popover_menu_get_menu_model (GtkPopoverMenu *popover)
 }
 
 /**
- * gtk_popover_menu_get_flags: (attributes org.gtk.Method.get_property=flags)
+ * gtk_popover_menu_get_flags:
  * @popover: a `GtkPopoverMenu`
  *
  * Returns the flags that @popover uses to create/display a menu from its model.
