@@ -30,6 +30,7 @@
 #include "gtktypebuiltins.h"
 #include "gtkwidgetprivate.h"
 #include "gtkbuildable.h"
+#include "gtkbuilderprivate.h"
 
 /**
  * GtkRevealer:
@@ -138,9 +139,14 @@ gtk_revealer_buildable_add_child (GtkBuildable *buildable,
                                   const char   *type)
 {
   if (GTK_IS_WIDGET (child))
-    gtk_revealer_set_child (GTK_REVEALER (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, NULL, "child");
+      gtk_revealer_set_child (GTK_REVEALER (buildable), GTK_WIDGET (child));
+    }
   else
-    parent_buildable_iface->add_child (buildable, builder, child, type);
+    {
+      parent_buildable_iface->add_child (buildable, builder, child, type);
+    }
 }
 
 static void
@@ -309,7 +315,7 @@ gtk_revealer_class_init (GtkRevealerClass *klass)
   widget_class->get_request_mode = gtk_revealer_get_request_mode;
 
   /**
-   * GtkRevealer:transition-type: (attributes org.gtk.Property.get=gtk_revealer_get_transition_type org.gtk.Property.set=gtk_revealer_set_transition_type)
+   * GtkRevealer:transition-type:
    *
    * The type of animation used to transition.
    */
@@ -320,7 +326,7 @@ gtk_revealer_class_init (GtkRevealerClass *klass)
                        GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkRevealer:transition-duration: (attributes org.gtk.Property.get=gtk_revealer_get_transition_duration org.gtk.Property.set=gtk_revealer_set_transition_duration)
+   * GtkRevealer:transition-duration:
    *
    * The animation duration, in milliseconds.
    */
@@ -330,7 +336,7 @@ gtk_revealer_class_init (GtkRevealerClass *klass)
                        GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkRevealer:reveal-child: (attributes org.gtk.Property.get=gtk_revealer_get_reveal_child org.gtk.Property.set=gtk_revealer_set_reveal_child)
+   * GtkRevealer:reveal-child:
    *
    * Whether the revealer should reveal the child.
    */
@@ -340,7 +346,7 @@ gtk_revealer_class_init (GtkRevealerClass *klass)
                           GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkRevealer:child-revealed: (attributes org.gtk.Property.get=gtk_revealer_get_child_revealed)
+   * GtkRevealer:child-revealed:
    *
    * Whether the child is revealed and the animation target reached.
    */
@@ -350,7 +356,7 @@ gtk_revealer_class_init (GtkRevealerClass *klass)
                           GTK_PARAM_READABLE);
 
   /**
-   * GtkRevealer:child: (attributes org.gtk.Property.get=gtk_revealer_get_child org.gtk.Property.set=gtk_revealer_set_child)
+   * GtkRevealer:child:
    *
    * The child widget.
    */
@@ -502,7 +508,10 @@ gtk_revealer_size_allocate (GtkWidget *widget,
       else if (ceil (min * hscale) == width)
         child_width = min;
       else
-        child_width = floor (width / hscale);
+        {
+          double d = floor (width / hscale);
+          child_width = MIN (d, G_MAXINT);
+        }
       child_height = height;
     }
   else if (vscale < 1.0)
@@ -515,7 +524,10 @@ gtk_revealer_size_allocate (GtkWidget *widget,
       else if (ceil (min * vscale) == height)
         child_height = min;
       else
-        child_height = floor (height / vscale);
+        {
+          double d = floor (height / vscale);
+          child_height = MIN (d, G_MAXINT);
+        }
     }
   else
     {
@@ -668,7 +680,7 @@ gtk_revealer_start_animation (GtkRevealer *revealer,
 }
 
 /**
- * gtk_revealer_set_reveal_child: (attributes org.gtk.Method.set_property=reveal-child)
+ * gtk_revealer_set_reveal_child:
  * @revealer: a `GtkRevealer`
  * @reveal_child: %TRUE to reveal the child
  *
@@ -690,7 +702,7 @@ gtk_revealer_set_reveal_child (GtkRevealer *revealer,
 }
 
 /**
- * gtk_revealer_get_reveal_child: (attributes org.gtk.Method.get_property=reveal-child)
+ * gtk_revealer_get_reveal_child:
  * @revealer: a `GtkRevealer`
  *
  * Returns whether the child is currently revealed.
@@ -711,7 +723,7 @@ gtk_revealer_get_reveal_child (GtkRevealer *revealer)
 }
 
 /**
- * gtk_revealer_get_child_revealed: (attributes org.gtk.Method.get_property=child-revealed)
+ * gtk_revealer_get_child_revealed:
  * @revealer: a `GtkRevealer`
  *
  * Returns whether the child is fully revealed.
@@ -743,39 +755,43 @@ gtk_revealer_measure (GtkWidget      *widget,
                       int            *natural_baseline)
 {
   GtkRevealer *revealer = GTK_REVEALER (widget);
-  double scale;
+  double scale, opposite_scale;
 
-  scale = get_child_size_scale (revealer, OPPOSITE_ORIENTATION (orientation));
-
-  if (for_size >= 0)
+  if (revealer->child == NULL || !_gtk_widget_get_visible (revealer->child))
     {
-      if (scale == 0)
-        return;
-      else
-        for_size = MIN (G_MAXINT, ceil (for_size / scale));
-    }
-
-  if (revealer->child != NULL && _gtk_widget_get_visible (revealer->child))
-    {
-      gtk_widget_measure (revealer->child,
-                          orientation,
-                          for_size,
-                          minimum, natural,
-                          NULL, NULL);
-    }
-  else
-    {
-      *minimum = 0;
-      *natural = 0;
+      *minimum = *natural = 0;
+      return;
     }
 
   scale = get_child_size_scale (revealer, orientation);
+  opposite_scale = get_child_size_scale (revealer, OPPOSITE_ORIENTATION (orientation));
+  if (scale == 0)
+    {
+      *minimum = *natural = 0;
+      return;
+    }
+  else if (opposite_scale == 0)
+    {
+      for_size = -1;
+    }
+  else if (for_size >= 0)
+    {
+      double d = floor (for_size / opposite_scale);
+      for_size = MIN (G_MAXINT, d);
+    }
+
+  gtk_widget_measure (revealer->child,
+                      orientation,
+                      for_size,
+                      minimum, natural,
+                      NULL, NULL);
+
   *minimum = ceil (*minimum * scale);
   *natural = ceil (*natural * scale);
 }
 
 /**
- * gtk_revealer_get_transition_duration: (attributes org.gtk.Method.get_property=transition-duration)
+ * gtk_revealer_get_transition_duration:
  * @revealer: a `GtkRevealer`
  *
  * Returns the amount of time (in milliseconds) that
@@ -792,7 +808,7 @@ gtk_revealer_get_transition_duration (GtkRevealer *revealer)
 }
 
 /**
- * gtk_revealer_set_transition_duration: (attributes org.gtk.Method.set_property=transition-duration)
+ * gtk_revealer_set_transition_duration:
  * @revealer: a `GtkRevealer`
  * @duration: the new duration, in milliseconds
  *
@@ -812,7 +828,7 @@ gtk_revealer_set_transition_duration (GtkRevealer *revealer,
 }
 
 /**
- * gtk_revealer_get_transition_type: (attributes org.gtk.Method.get_property=transition-type)
+ * gtk_revealer_get_transition_type:
  * @revealer: a `GtkRevealer`
  *
  * Gets the type of animation that will be used
@@ -829,7 +845,7 @@ gtk_revealer_get_transition_type (GtkRevealer *revealer)
 }
 
 /**
- * gtk_revealer_set_transition_type: (attributes org.gtk.Method.set_property=transition-type)
+ * gtk_revealer_set_transition_type:
  * @revealer: a `GtkRevealer`
  * @transition: the new transition type
  *
@@ -853,7 +869,7 @@ gtk_revealer_set_transition_type (GtkRevealer               *revealer,
 }
 
 /**
- * gtk_revealer_set_child: (attributes org.gtk.Method.set_property=child)
+ * gtk_revealer_set_child:
  * @revealer: a `GtkRevealer`
  * @child: (nullable): the child widget
  *
@@ -864,7 +880,10 @@ gtk_revealer_set_child (GtkRevealer *revealer,
                         GtkWidget   *child)
 {
   g_return_if_fail (GTK_IS_REVEALER (revealer));
-  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+  g_return_if_fail (child == NULL || revealer->child == child || gtk_widget_get_parent (child) == NULL);
+
+  if (revealer->child == child)
+    return;
 
   g_clear_pointer (&revealer->child, gtk_widget_unparent);
 
@@ -879,7 +898,7 @@ gtk_revealer_set_child (GtkRevealer *revealer,
 }
 
 /**
- * gtk_revealer_get_child: (attributes org.gtk.Method.get_property=child)
+ * gtk_revealer_get_child:
  * @revealer: a `GtkRevealer`
  *
  * Gets the child widget of @revealer.
