@@ -78,6 +78,15 @@
  * `GtkSearchEntry` provides only minimal API and should be used with
  * the [iface@Gtk.Editable] API.
  *
+ * ## Shortcuts and Gestures
+ *
+ * The following signals have default keybindings:
+ *
+ * - [signal@Gtk.SearchEntry::activate]
+ * - [signal@Gtk.SearchEntry::next-match]
+ * - [signal@Gtk.SearchEntry::previous-match]
+ * - [signal@Gtk.SearchEntry::stop-search]
+ *
  * ## CSS Nodes
  *
  * ```
@@ -528,7 +537,7 @@ gtk_search_entry_class_init (GtkSearchEntryClass *klass)
    *
    * Emitted when the entry is activated.
    *
-   * The keybindings for this signal are all forms of the Enter key.
+   * The keybindings for this signal are all forms of the <kbd>Enter</kbd> key.
    */
   signals[ACTIVATE] =
     g_signal_new (I_("activate"),
@@ -568,7 +577,7 @@ gtk_search_entry_class_init (GtkSearchEntryClass *klass)
    * Applications should connect to it, to implement moving
    * between matches.
    *
-   * The default bindings for this signal is Ctrl-g.
+   * The default bindings for this signal is <kbd>Ctrl</kbd>+<kbd>g</kbd>.
    */
   signals[NEXT_MATCH] =
     g_signal_new (I_("next-match"),
@@ -591,7 +600,8 @@ gtk_search_entry_class_init (GtkSearchEntryClass *klass)
    * Applications should connect to it, to implement moving
    * between matches.
    *
-   * The default bindings for this signal is Ctrl-Shift-g.
+   * The default bindings for this signal is
+   * <kbd>Ctrl</kbd>+<kbd>Shift</kbd>+<kbd>g</kbd>.
    */
   signals[PREVIOUS_MATCH] =
     g_signal_new (I_("previous-match"),
@@ -613,7 +623,7 @@ gtk_search_entry_class_init (GtkSearchEntryClass *klass)
    * Applications should connect to it, to implement hiding
    * the search entry in this case.
    *
-   * The default bindings for this signal is Escape.
+   * The default bindings for this signal is <kbd>Escape</kbd>.
    */
   signals[STOP_SEARCH] =
     g_signal_new (I_("stop-search"),
@@ -638,6 +648,16 @@ gtk_search_entry_class_init (GtkSearchEntryClass *klass)
                   NULL,
                   G_TYPE_NONE, 0);
 
+#ifdef __APPLE__
+  gtk_widget_class_add_binding_signal (widget_class,
+                                       GDK_KEY_g, GDK_META_MASK,
+                                       "next-match",
+                                       NULL);
+  gtk_widget_class_add_binding_signal (widget_class,
+                                       GDK_KEY_g, GDK_SHIFT_MASK | GDK_META_MASK,
+                                       "previous-match",
+                                       NULL);
+#else
   gtk_widget_class_add_binding_signal (widget_class,
                                        GDK_KEY_g, GDK_CONTROL_MASK,
                                        "next-match",
@@ -646,6 +666,8 @@ gtk_search_entry_class_init (GtkSearchEntryClass *klass)
                                        GDK_KEY_g, GDK_SHIFT_MASK | GDK_CONTROL_MASK,
                                        "previous-match",
                                        NULL);
+#endif
+
   gtk_widget_class_add_binding_signal (widget_class,
                                        GDK_KEY_Escape, 0,
                                        "stop-search",
@@ -827,10 +849,6 @@ gtk_search_entry_init (GtkSearchEntry *entry)
                              GTK_EVENT_CONTROLLER (catchall));
 
   gtk_widget_add_css_class (GTK_WIDGET (entry), I_("search"));
-
-  gtk_accessible_update_property (GTK_ACCESSIBLE (entry),
-                                  GTK_ACCESSIBLE_PROPERTY_HAS_POPUP, TRUE,
-                                  -1);
 }
 
 /**
@@ -973,7 +991,7 @@ gtk_search_entry_get_key_capture_widget (GtkSearchEntry *entry)
 }
 
 /**
- * gtk_search_entry_set_search_delay: (attributes org.gtk.Property.set_property=search-delay)
+ * gtk_search_entry_set_search_delay:
  * @entry: a `GtkSearchEntry`
  * @delay: a delay in milliseconds
  *
@@ -1000,7 +1018,7 @@ gtk_search_entry_set_search_delay (GtkSearchEntry *entry,
 }
 
 /**
- * gtk_search_entry_get_search_delay: (attributes org.gtk.Property.get_property=search-delay)
+ * gtk_search_entry_get_search_delay:
  * @entry: a `GtkSearchEntry`
  *
  * Get the delay to be used between the last keypress and the
