@@ -492,7 +492,7 @@ gtk_stack_page_class_init (GtkStackPageClass *class)
   object_class->constructed = gtk_stack_page_constructed;
 
   /**
-   * GtkStackPage:child: (attributes org.gtk.Property.get=gtk_stack_page_get_child)
+   * GtkStackPage:child:
    *
    * The child that this page is for.
    */
@@ -502,7 +502,7 @@ gtk_stack_page_class_init (GtkStackPageClass *class)
                          GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
   /**
-   * GtkStackPage:name: (attributes org.gtk.Property.get=gtk_stack_page_get_name org.gtk.Property.set=gtk_stack_page_set_name)
+   * GtkStackPage:name:
    *
    * The name of the child page.
    */
@@ -512,7 +512,7 @@ gtk_stack_page_class_init (GtkStackPageClass *class)
                          GTK_PARAM_READWRITE);
 
   /**
-   * GtkStackPage:title: (attributes org.gtk.Property.get=gtk_stack_page_get_title org.gtk.Property.set=gtk_stack_page_set_title)
+   * GtkStackPage:title:
    *
    * The title of the child page.
    */
@@ -522,7 +522,7 @@ gtk_stack_page_class_init (GtkStackPageClass *class)
                          GTK_PARAM_READWRITE);
 
   /**
-   * GtkStackPage:icon-name: (attributes org.gtk.Property.get=gtk_stack_page_get_icon_name org.gtk.Property.set=gtk_stack_page_set_icon_name)
+   * GtkStackPage:icon-name:
    *
    * The icon name of the child page.
    */
@@ -532,7 +532,7 @@ gtk_stack_page_class_init (GtkStackPageClass *class)
                          GTK_PARAM_READWRITE);
 
   /**
-   * GtkStackPage:needs-attention: (attributes org.gtk.Property.get=gtk_stack_page_get_needs_attention org.gtk.Property.set=gtk_stack_page_set_needs_attention)
+   * GtkStackPage:needs-attention:
    *
    * Whether the page requires the user attention.
    *
@@ -546,7 +546,7 @@ gtk_stack_page_class_init (GtkStackPageClass *class)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkStackPage:visible: (attributes org.gtk.Property.get=gtk_stack_page_get_visible org.gtk.Property.set=gtk_stack_page_set_visible)
+   * GtkStackPage:visible:
    *
    * Whether this page is visible.
    */
@@ -556,7 +556,7 @@ gtk_stack_page_class_init (GtkStackPageClass *class)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkStackPage:use-underline: (attributes org.gtk.Property.get=gtk_stack_page_get_use_underline org.gtk.Property.set=gtk_stack_page_set_use_underline)
+   * GtkStackPage:use-underline:
    *
    * If set, an underline in the title indicates a mnemonic.
    */
@@ -617,8 +617,7 @@ gtk_stack_pages_get_item (GListModel *model,
   GtkStackPrivate *priv = gtk_stack_get_instance_private (pages->stack);
   GtkStackPage *page;
 
-
-  if (position > priv->children->len - 1)
+  if (position >= priv->children->len)
     return NULL;
 
   page = g_ptr_array_index (priv->children, position);
@@ -642,7 +641,7 @@ gtk_stack_pages_is_selected (GtkSelectionModel *model,
   GtkStackPrivate *priv = gtk_stack_get_instance_private (pages->stack);
   GtkStackPage *page;
 
-  if (position > priv->children->len - 1)
+  if (position >= priv->children->len)
     return FALSE;
 
   page = g_ptr_array_index (priv->children, position);
@@ -664,6 +663,9 @@ gtk_stack_pages_select_item (GtkSelectionModel *model,
   GtkStackPrivate *priv = gtk_stack_get_instance_private (pages->stack);
   GtkStackPage *page;
 
+  if (position >= priv->children->len)
+    return FALSE;
+
   page = g_ptr_array_index (priv->children, position);
 
   set_visible_child (pages->stack, page, priv->transition_type, priv->transition_duration);
@@ -809,9 +811,12 @@ gtk_stack_accessible_get_first_accessible_child (GtkAccessible *accessible)
 {
   GtkStack *stack = GTK_STACK (accessible);
   GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
-  GtkStackPage *page = g_ptr_array_index (priv->children, 0);
+  GtkAccessible *page_accessible = NULL;
 
-  return GTK_ACCESSIBLE (g_object_ref (page));
+  if (priv->children->len > 0)
+    page_accessible = GTK_ACCESSIBLE (g_object_ref (g_ptr_array_index (priv->children, 0)));
+
+  return page_accessible;
 }
 
 static void
@@ -958,7 +963,7 @@ gtk_stack_class_init (GtkStackClass *klass)
   widget_class->get_request_mode = gtk_stack_get_request_mode;
 
   /**
-   * GtkStack:hhomogeneous: (attributes org.gtk.Property.get=gtk_stack_get_hhomogeneous org.gtk.Property.set=gtk_stack_set_hhomogeneous)
+   * GtkStack:hhomogeneous:
    *
    * %TRUE if the stack allocates the same width for all children.
    */
@@ -968,7 +973,7 @@ gtk_stack_class_init (GtkStackClass *klass)
                             GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkStack:vhomogeneous: (attributes org.gtk.Property.get=gtk_stack_get_vhomogeneous org.gtk.Property.set=gtk_stack_set_vhomogeneous)
+   * GtkStack:vhomogeneous:
    *
    * %TRUE if the stack allocates the same height for all children.
    */
@@ -978,7 +983,7 @@ gtk_stack_class_init (GtkStackClass *klass)
                             GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkStack:visible-child: (attributes org.gtk.Property.get=gtk_stack_get_visible_child org.gtk.Property.set=gtk_stack_set_visible_child)
+   * GtkStack:visible-child:
    *
    * The widget currently visible in the stack.
    */
@@ -988,7 +993,7 @@ gtk_stack_class_init (GtkStackClass *klass)
                            GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkStack:visible-child-name: (attributes org.gtk.Property.get=gtk_stack_get_visible_child_name org.gtk.Property.set=gtk_stack_set_visible_child_name)
+   * GtkStack:visible-child-name:
    *
    * The name of the widget currently visible in the stack.
    */
@@ -998,7 +1003,7 @@ gtk_stack_class_init (GtkStackClass *klass)
                            GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkStack:transition-duration: (attributes org.gtk.Property.get=gtk_stack_get_transition_duration org.gtk.Property.set=gtk_stack_set_transition_duration)
+   * GtkStack:transition-duration:
    *
    * The animation duration, in milliseconds.
    */
@@ -1008,7 +1013,7 @@ gtk_stack_class_init (GtkStackClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkStack:transition-type: (attributes org.gtk.Property.get=gtk_stack_get_transition_type org.gtk.Property.set=gtk_stack_set_transition_type)
+   * GtkStack:transition-type:
    *
    * The type of animation used to transition.
    */
@@ -1018,7 +1023,7 @@ gtk_stack_class_init (GtkStackClass *klass)
                          GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkStack:transition-running: (attributes org.gtk.Property.get=gtk_stack_get_transition_running)
+   * GtkStack:transition-running:
    *
    * Whether or not the transition is currently running.
    */
@@ -1028,7 +1033,7 @@ gtk_stack_class_init (GtkStackClass *klass)
                             GTK_PARAM_READABLE);
 
   /**
-   * GtkStack:interpolate-size: (attributes org.gtk.Property.get=gtk_stack_get_interpolate_size org.gtk.Property.set=gtk_stack_set_interpolate_size)
+   * GtkStack:interpolate-size:
    *
    * Whether or not the size should smoothly change during the transition.
    */
@@ -1038,7 +1043,7 @@ gtk_stack_class_init (GtkStackClass *klass)
                             GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkStack:pages: (attributes org.gtk.Property.get=gtk_stack_get_pages)
+   * GtkStack:pages:
    *
    * A selection model with the stack pages.
    */
@@ -1854,7 +1859,7 @@ gtk_stack_get_child_by_name (GtkStack    *stack,
 }
 
 /**
- * gtk_stack_page_get_child: (attributes org.gtk.Method.get_property=child)
+ * gtk_stack_page_get_child:
  * @self: a `GtkStackPage`
  *
  * Returns the stack child to which @self belongs.
@@ -1868,7 +1873,7 @@ gtk_stack_page_get_child (GtkStackPage *self)
 }
 
 /**
- * gtk_stack_set_hhomogeneous: (attributes org.gtk.Method.set_property=hhomogeneous)
+ * gtk_stack_set_hhomogeneous:
  * @stack: a `GtkStack`
  * @hhomogeneous: %TRUE to make @stack horizontally homogeneous
  *
@@ -1900,7 +1905,7 @@ gtk_stack_set_hhomogeneous (GtkStack *stack,
 }
 
 /**
- * gtk_stack_get_hhomogeneous: (attributes org.gtk.Method.get_property=hhomogeneous)
+ * gtk_stack_get_hhomogeneous:
  * @stack: a `GtkStack`
  *
  * Gets whether @stack is horizontally homogeneous.
@@ -1918,7 +1923,7 @@ gtk_stack_get_hhomogeneous (GtkStack *stack)
 }
 
 /**
- * gtk_stack_set_vhomogeneous: (attributes org.gtk.Method.set_property=vhomogeneous)
+ * gtk_stack_set_vhomogeneous:
  * @stack: a `GtkStack`
  * @vhomogeneous: %TRUE to make @stack vertically homogeneous
  *
@@ -1950,7 +1955,7 @@ gtk_stack_set_vhomogeneous (GtkStack *stack,
 }
 
 /**
- * gtk_stack_get_vhomogeneous: (attributes org.gtk.Method.get_property=vhomogeneous)
+ * gtk_stack_get_vhomogeneous:
  * @stack: a `GtkStack`
  *
  * Gets whether @stack is vertically homogeneous.
@@ -1968,7 +1973,7 @@ gtk_stack_get_vhomogeneous (GtkStack *stack)
 }
 
 /**
- * gtk_stack_get_transition_duration: (attributes org.gtk.Method.get_property=transition-duration)
+ * gtk_stack_get_transition_duration:
  * @stack: a `GtkStack`
  *
  * Returns the amount of time (in milliseconds) that
@@ -1987,7 +1992,7 @@ gtk_stack_get_transition_duration (GtkStack *stack)
 }
 
 /**
- * gtk_stack_set_transition_duration: (attributes org.gtk.Method.set_property=transition-duration)
+ * gtk_stack_set_transition_duration:
  * @stack: a `GtkStack`
  * @duration: the new duration, in milliseconds
  *
@@ -2011,7 +2016,7 @@ gtk_stack_set_transition_duration (GtkStack *stack,
 }
 
 /**
- * gtk_stack_get_transition_type: (attributes org.gtk.Method.get_property=transition-type)
+ * gtk_stack_get_transition_type:
  * @stack: a `GtkStack`
  *
  * Gets the type of animation that will be used
@@ -2030,7 +2035,7 @@ gtk_stack_get_transition_type (GtkStack *stack)
 }
 
 /**
- * gtk_stack_set_transition_type: (attributes org.gtk.Method.set_property=transition-type)
+ * gtk_stack_set_transition_type:
  * @stack: a `GtkStack`
  * @transition: the new transition type
  *
@@ -2060,7 +2065,7 @@ gtk_stack_set_transition_type (GtkStack              *stack,
 }
 
 /**
- * gtk_stack_get_transition_running: (attributes org.gtk.Method.get_property=transition-running)
+ * gtk_stack_get_transition_running:
  * @stack: a `GtkStack`
  *
  * Returns whether the @stack is currently in a transition from one page to
@@ -2079,7 +2084,7 @@ gtk_stack_get_transition_running (GtkStack *stack)
 }
 
 /**
- * gtk_stack_set_interpolate_size: (attributes org.gtk.Method.set_property=interpolate-size)
+ * gtk_stack_set_interpolate_size:
  * @stack: A `GtkStack`
  * @interpolate_size: the new value
  *
@@ -2109,7 +2114,7 @@ gtk_stack_set_interpolate_size (GtkStack *stack,
 }
 
 /**
- * gtk_stack_get_interpolate_size: (attributes org.gtk.Method.get_property=interpolate-size)
+ * gtk_stack_get_interpolate_size:
  * @stack: A `GtkStack`
  *
  * Returns whether the `GtkStack` is set up to interpolate between
@@ -2129,7 +2134,7 @@ gtk_stack_get_interpolate_size (GtkStack *stack)
 
 
 /**
- * gtk_stack_get_visible_child: (attributes org.gtk.Method.get_property=visible-child)
+ * gtk_stack_get_visible_child:
  * @stack: a `GtkStack`
  *
  * Gets the currently visible child of @stack.
@@ -2149,7 +2154,7 @@ gtk_stack_get_visible_child (GtkStack *stack)
 }
 
 /**
- * gtk_stack_get_visible_child_name: (attributes org.gtk.Method.get_property=visible-child-name)
+ * gtk_stack_get_visible_child_name:
  * @stack: a `GtkStack`
  *
  * Returns the name of the currently visible child of @stack.
@@ -2173,7 +2178,7 @@ gtk_stack_get_visible_child_name (GtkStack *stack)
 }
 
 /**
- * gtk_stack_set_visible_child: (attributes org.gtk.Method.set_property=visible-child)
+ * gtk_stack_set_visible_child:
  * @stack: a `GtkStack`
  * @child: a child of @stack
  *
@@ -2212,7 +2217,7 @@ gtk_stack_set_visible_child (GtkStack  *stack,
 }
 
 /**
- * gtk_stack_set_visible_child_name: (attributes org.gtk.Method.set_property=visible-child-name)
+ * gtk_stack_set_visible_child_name:
  * @stack: a `GtkStack`
  * @name: the name of the child to make visible
  *
@@ -2323,9 +2328,30 @@ gtk_stack_compute_expand (GtkWidget *widget,
 static GtkSizeRequestMode
 gtk_stack_get_request_mode (GtkWidget *widget)
 {
+  GtkStack *stack = GTK_STACK (widget);
+  GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
   GtkWidget *w;
   int wfh = 0, hfw = 0;
 
+  if (!priv->homogeneous[GTK_ORIENTATION_VERTICAL] &&
+      !priv->homogeneous[GTK_ORIENTATION_HORIZONTAL])
+    {
+      GtkSizeRequestMode lv_mode;
+
+      /* Only the visible child, and perhaps the last visible child
+       * during a transition, matter.  Attempt to return constant-size
+       * when we can.  */
+      if (priv->last_visible_child)
+        lv_mode = gtk_widget_get_request_mode (priv->last_visible_child->widget);
+      else
+        lv_mode = GTK_SIZE_REQUEST_CONSTANT_SIZE;
+
+      if (lv_mode == GTK_SIZE_REQUEST_CONSTANT_SIZE && priv->visible_child)
+        return gtk_widget_get_request_mode (priv->visible_child->widget);
+      else
+        return lv_mode;
+    }
+
   for (w = gtk_widget_get_first_child (widget);
        w != NULL;
        w = gtk_widget_get_next_sibling (w))
@@ -2648,6 +2674,33 @@ gtk_stack_snapshot (GtkWidget   *widget,
     }
 }
 
+static void
+adjust_child_allocation (GtkWidget *child,
+                         int       *width,
+                         int       *height)
+{
+  int min;
+
+  if (gtk_widget_get_request_mode (child) == GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT)
+    {
+      gtk_widget_measure (child, GTK_ORIENTATION_VERTICAL, -1,
+                          &min, NULL, NULL, NULL);
+      *height = MAX (*height, min);
+      gtk_widget_measure (child, GTK_ORIENTATION_HORIZONTAL, *height,
+                          &min, NULL, NULL, NULL);
+      *width = MAX (*width, min);
+    }
+  else /* GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH or CONSTANT_SIZE */
+    {
+      gtk_widget_measure (child, GTK_ORIENTATION_HORIZONTAL, -1,
+                          &min, NULL, NULL, NULL);
+      *width = MAX (*width, min);
+      gtk_widget_measure (child, GTK_ORIENTATION_VERTICAL, *width,
+                          &min, NULL, NULL, NULL);
+      *height = MAX (*height, min);
+    }
+}
+
 static void
 gtk_stack_size_allocate (GtkWidget *widget,
                          int        width,
@@ -2660,17 +2713,10 @@ gtk_stack_size_allocate (GtkWidget *widget,
 
   if (priv->last_visible_child)
     {
-      int child_width, child_height;
-      int min, nat;
+      int child_width = width, child_height = height;
 
-      gtk_widget_measure (priv->last_visible_child->widget, GTK_ORIENTATION_HORIZONTAL,
-                          -1,
-                          &min, &nat, NULL, NULL);
-      child_width = MAX (min, width);
-      gtk_widget_measure (priv->last_visible_child->widget, GTK_ORIENTATION_VERTICAL,
-                          child_width,
-                          &min, &nat, NULL, NULL);
-      child_height = MAX (min, height);
+      adjust_child_allocation (priv->last_visible_child->widget,
+                               &child_width, &child_height);
 
       gtk_widget_size_allocate (priv->last_visible_child->widget,
                                 &(GtkAllocation) { 0, 0, child_width, child_height }, -1);
@@ -2683,16 +2729,9 @@ gtk_stack_size_allocate (GtkWidget *widget,
 
   if (priv->visible_child)
     {
-      int min_width;
-      int min_height;
-
-      gtk_widget_measure (priv->visible_child->widget, GTK_ORIENTATION_HORIZONTAL,
-                          height, &min_width, NULL, NULL, NULL);
-      child_allocation.width = MAX (child_allocation.width, min_width);
-
-      gtk_widget_measure (priv->visible_child->widget, GTK_ORIENTATION_VERTICAL,
-                          child_allocation.width, &min_height, NULL, NULL, NULL);
-      child_allocation.height = MAX (child_allocation.height, min_height);
+      adjust_child_allocation (priv->visible_child->widget,
+                               &child_allocation.width,
+                               &child_allocation.height);
 
       if (child_allocation.width > width)
         {
@@ -2718,7 +2757,27 @@ gtk_stack_size_allocate (GtkWidget *widget,
     }
 }
 
-#define LERP(a, b, t) ((a) + (((b) - (a)) * (1.0 - (t))))
+static inline double
+lerp (int    a,
+      int    b,
+      double progress)
+{
+  return a * (1.0 - progress) + b * progress;
+}
+
+/*
+ * Given lerp (a, b, progress) -> r, find b. In other words: we know what size
+ * we're interpolating *from*, the current size, and the current interpolation
+ * progress; guess which size we're interpolating *to*.
+ */
+static inline double
+inverse_lerp (int    a,
+              int    r,
+              double progress)
+{
+  return (r - a * (1.0 - progress)) / progress;
+}
+
 static void
 gtk_stack_measure (GtkWidget      *widget,
                    GtkOrientation  orientation,
@@ -2732,8 +2791,52 @@ gtk_stack_measure (GtkWidget      *widget,
   GtkStackPrivate *priv = gtk_stack_get_instance_private (stack);
   GtkStackPage *child_info;
   GtkWidget *child;
-  int child_min, child_nat;
   guint idx;
+  int child_min, child_nat, child_for_size;
+  int last_size, last_opposite_size;
+  double t;
+
+  if (priv->interpolate_size && priv->last_visible_child)
+    {
+      t = gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE);
+
+      if (orientation == GTK_ORIENTATION_HORIZONTAL)
+        {
+          last_size = priv->last_visible_widget_width;
+          last_opposite_size = priv->last_visible_widget_height;
+        }
+      else
+        {
+          last_size = priv->last_visible_widget_height;
+          last_opposite_size = priv->last_visible_widget_width;
+        }
+
+      /* Work out which for_size we want to pass for measuring our children */
+      if (for_size == -1 || for_size == last_opposite_size ||
+          priv->homogeneous[OPPOSITE_ORIENTATION(orientation)])
+        child_for_size = for_size;
+      else if (t <= 0.0)
+        /* We're going to return last_size anyway */
+        child_for_size = -1;
+      else
+        {
+          double d = inverse_lerp (last_opposite_size, for_size, t);
+          /* inverse_lerp is numerically unstable due to its use of floating-
+           * point division, potentially by a very small value when progress is
+           * close to zero. So we sanity check the return value.
+           */
+          if (isnan (d) || isinf (d) || d < 0 || d >= G_MAXINT)
+            child_for_size = -1;
+          else
+            child_for_size = floor (d);
+        }
+    }
+  else
+    {
+      t = 1.0;
+      last_size = 0;
+      child_for_size = for_size;
+    }
 
   *minimum = 0;
   *natural = 0;
@@ -2746,37 +2849,46 @@ gtk_stack_measure (GtkWidget      *widget,
       if (!priv->homogeneous[orientation] &&
           priv->visible_child != child_info)
         continue;
+      if (!gtk_widget_get_visible (child))
+        continue;
 
-      if (gtk_widget_get_visible (child))
+      if (!priv->homogeneous[OPPOSITE_ORIENTATION(orientation)] && priv->visible_child != child_info)
         {
-          if (!priv->homogeneous[OPPOSITE_ORIENTATION(orientation)] && priv->visible_child != child_info)
-            {
-              int min_for_size;
+          int measure_for_size;
 
-              gtk_widget_measure (child, OPPOSITE_ORIENTATION (orientation), -1, &min_for_size, NULL, NULL, NULL);
-
-              gtk_widget_measure (child, orientation, MAX (min_for_size, for_size), &child_min, &child_nat, NULL, NULL);
-            }
+          /* Make sure to measure at least for the minimum size */
+          if (child_for_size == -1)
+            measure_for_size = -1;
           else
-            gtk_widget_measure (child, orientation, for_size, &child_min, &child_nat, NULL, NULL);
+            {
+              gtk_widget_measure (child, OPPOSITE_ORIENTATION (orientation),
+                                  -1,
+                                  &measure_for_size, NULL,
+                                  NULL, NULL);
+              measure_for_size = MAX (measure_for_size, child_for_size);
+            }
 
-          *minimum = MAX (*minimum, child_min);
-          *natural = MAX (*natural, child_nat);
+          gtk_widget_measure (child, orientation,
+                              measure_for_size,
+                              &child_min, &child_nat,
+                              NULL, NULL);
         }
-    }
-
-  if (priv->last_visible_child != NULL && !priv->homogeneous[orientation])
-    {
-      double t = priv->interpolate_size ? gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE) : 1.0;
-      int last_size;
-
-      if (orientation == GTK_ORIENTATION_HORIZONTAL)
-        last_size = priv->last_visible_widget_width;
       else
-        last_size = priv->last_visible_widget_height;
+        gtk_widget_measure (child, orientation,
+                            child_for_size,
+                            &child_min, &child_nat,
+                            NULL, NULL);
 
-      *minimum = LERP (*minimum, last_size, t);
-      *natural = LERP (*natural, last_size, t);
+      *minimum = MAX (*minimum, child_min);
+      *natural = MAX (*natural, child_nat);
+  }
+
+  if (priv->last_visible_child != NULL &&
+      priv->interpolate_size &&
+      !priv->homogeneous[orientation])
+    {
+      *minimum = ceil (lerp (last_size, *minimum, t));
+      *natural = ceil (lerp (last_size, *natural, t));
     }
 }
 
@@ -2793,7 +2905,7 @@ gtk_stack_init (GtkStack *stack)
 }
 
 /**
- * gtk_stack_get_pages: (attributes org.gtk.Method.get_property=pages)
+ * gtk_stack_get_pages:
  * @stack: a `GtkStack`
  *
  * Returns a `GListModel` that contains the pages of the stack.
@@ -2821,7 +2933,7 @@ gtk_stack_get_pages (GtkStack *stack)
 }
 
 /**
- * gtk_stack_page_get_visible: (attributes org.gtk.Method.get_property=visible)
+ * gtk_stack_page_get_visible:
  * @self: a `GtkStackPage`
  *
  * Returns whether @page is visible in its `GtkStack`.
@@ -2840,7 +2952,7 @@ gtk_stack_page_get_visible (GtkStackPage *self)
 }
 
 /**
- * gtk_stack_page_set_visible: (attributes org.gtk.Method.set_property=visible)
+ * gtk_stack_page_set_visible:
  * @self: a `GtkStackPage`
  * @visible: The new property value
  *
@@ -2866,7 +2978,7 @@ gtk_stack_page_set_visible (GtkStackPage *self,
 }
 
 /**
- * gtk_stack_page_get_needs_attention: (attributes org.gtk.Method.get_property=needs-attention)
+ * gtk_stack_page_get_needs_attention:
  * @self: a `GtkStackPage`
  *
  * Returns whether the page is marked as “needs attention”.
@@ -2881,7 +2993,7 @@ gtk_stack_page_get_needs_attention (GtkStackPage *self)
 }
 
 /**
- * gtk_stack_page_set_needs_attention: (attributes org.gtk.Method.set_property=needs-attention)
+ * gtk_stack_page_set_needs_attention:
  * @self: a `GtkStackPage`
  * @setting: the new value to set
  *
@@ -2901,7 +3013,7 @@ gtk_stack_page_set_needs_attention (GtkStackPage *self,
 }
 
 /**
- * gtk_stack_page_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
+ * gtk_stack_page_get_use_underline:
  * @self: a `GtkStackPage`
  *
  * Gets whether underlines in the page title indicate mnemonics.
@@ -2915,7 +3027,7 @@ gtk_stack_page_get_use_underline (GtkStackPage *self)
 }
 
 /**
- * gtk_stack_page_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
+ * gtk_stack_page_set_use_underline:
  * @self: a `GtkStackPage`
  * @setting: the new value to set
  *
@@ -2936,7 +3048,7 @@ gtk_stack_page_set_use_underline (GtkStackPage *self,
 
 
 /**
- * gtk_stack_page_get_name: (attributes org.gtk.Method.get_property=name)
+ * gtk_stack_page_get_name:
  * @self: a `GtkStackPage`
  *
  * Returns the name of the page.
@@ -2952,7 +3064,7 @@ gtk_stack_page_get_name (GtkStackPage *self)
 }
 
 /**
- * gtk_stack_page_set_name: (attributes org.gtk.Method.set_property=name)
+ * gtk_stack_page_set_name:
  * @self: a `GtkStackPage`
  * @setting: (transfer none): the new value to set
  *
@@ -3003,7 +3115,7 @@ gtk_stack_page_set_name (GtkStackPage *self,
 }
 
 /**
- * gtk_stack_page_get_title: (attributes org.gtk.Method.get_property=title)
+ * gtk_stack_page_get_title:
  * @self: a `GtkStackPage`
  *
  * Gets the page title.
@@ -3019,7 +3131,7 @@ gtk_stack_page_get_title (GtkStackPage *self)
 }
 
 /**
- * gtk_stack_page_set_title: (attributes org.gtk.Method.set_property=title)
+ * gtk_stack_page_set_title:
  * @self: a `GtkStackPage`
  * @setting: (transfer none): the new value to set
  *
@@ -3044,7 +3156,7 @@ gtk_stack_page_set_title (GtkStackPage *self,
 }
 
 /**
- * gtk_stack_page_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
+ * gtk_stack_page_get_icon_name:
  * @self: a `GtkStackPage`
  *
  * Returns the icon name of the page.
@@ -3060,7 +3172,7 @@ gtk_stack_page_get_icon_name (GtkStackPage *self)
 }
 
 /**
- * gtk_stack_page_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
+ * gtk_stack_page_set_icon_name:
  * @self: a `GtkStackPage`
  * @setting: (transfer none): the new value to set
  *
