@@ -23,6 +23,7 @@
 #include "gtkbinlayout.h"
 #include "gtkbox.h"
 #include "gtkbuildable.h"
+#include "gtkbuilderprivate.h"
 #include "gtkdragsourceprivate.h"
 #include "gtkgestureclick.h"
 #include "gtkgesturedrag.h"
@@ -536,7 +537,7 @@ gtk_window_handle_class_init (GtkWindowHandleClass *klass)
   widget_class->unrealize = gtk_window_handle_unrealize;
 
   /**
-   * GtkWindowHandle:child: (attributes org.gtk.Property.get=gtk_window_handle_get_child org.gtk.Property.set=gtk_window_handle_set_child)
+   * GtkWindowHandle:child:
    *
    * The child widget.
    */
@@ -576,9 +577,14 @@ gtk_window_handle_buildable_add_child (GtkBuildable *buildable,
                                        const char   *type)
 {
   if (GTK_IS_WIDGET (child))
-    gtk_window_handle_set_child (GTK_WINDOW_HANDLE (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, NULL, "child");
+      gtk_window_handle_set_child (GTK_WINDOW_HANDLE (buildable), GTK_WIDGET (child));
+    }
   else
-    parent_buildable_iface->add_child (buildable, builder, child, type);
+    {
+      parent_buildable_iface->add_child (buildable, builder, child, type);
+    }
 }
 
 static void
@@ -603,7 +609,7 @@ gtk_window_handle_new (void)
 }
 
 /**
- * gtk_window_handle_get_child: (attributes org.gtk.Method.get_property=child)
+ * gtk_window_handle_get_child:
  * @self: a `GtkWindowHandle`
  *
  * Gets the child widget of @self.
@@ -619,7 +625,7 @@ gtk_window_handle_get_child (GtkWindowHandle *self)
 }
 
 /**
- * gtk_window_handle_set_child: (attributes org.gtk.Method.set_property=child)
+ * gtk_window_handle_set_child:
  * @self: a `GtkWindowHandle`
  * @child: (nullable): the child widget
  *
